#ifndef _ncl_dchandle_h
#define _ncl_dchandle_h 1
/*
The ANSI C Numerical Class Library.
Copyright (C) 2000 E. Robert Tisdale

This file is part of The ANSI C Numerical Class Library.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#ifdef NCL_INLINE
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif /* NCL_REVEAL */
#endif /* NCL_INLINE */

#ifdef NCL_MACROS
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif /* NCL_REVEAL */
#endif /* NCL_MACROS */

/* Include real handle type definitions.	*/
#include<ncl_dhandle.h>

/* Include scalar type definitions.		*/
#include<ncl_dcscalar.h>

#ifdef NCL_REVEAL
/*  Reveal type definitions after applications are thoroughly tested.	*/

typedef struct {
  /* private: */
  ncl_double*	P;
  } ncl_dchandle;

#else  /* NCL_REVEAL */
/* Conceal type definitions until applications are thoroughly tested.	*/

#define NCL_DCHANDLE_SIZE NCL_HANDLE_SIZE
typedef int ncl_dchidden_handle[NCL_DCHANDLE_SIZE/sizeof(int)];

typedef struct {
  /* private: */
  ncl_dchidden_handle	H;
  } ncl_dchandle;

#endif /* NCL_REVEAL */

#ifdef NCL_INLINE

inline static
ncl_double*		/* pointer to the underlying data array	*/
(ncl_dch_pointer)(ncl_dchandle* ph) { return ph->P; }

inline static
ncl_dchandle		/* automatic storage constructor	*/
(ncl_dch_create)(ncl_dscalar* p) {
  ncl_dchandle h; h.P = p; return h; }

inline static
void				/* automatic storage  destructor	*/
(ncl_dch_destroy)(ncl_dchandle* ph) {
  }

inline static
int
(ncl_dch_empty)(const ncl_dchandle* ph) {
  return ph->P == NULL; }

inline static
ncl_dcscalar		/* h[o]					*/
(ncl_dch_get)(const ncl_dchandle* ph,
  ncl_offset o) {
  return ((ncl_dcscalar*)(ph->P))[o]; }

inline static
void				/* h[o] <-- x				*/
(ncl_dch_set)(      ncl_dchandle* ph,
  ncl_offset o, ncl_dcscalar x) {
  ((ncl_dcscalar*)(ph->P))[o] = x; }

/* Both ncl_dch_get and ncl_dch_set assume that			*/
/* sizeof(ncl_dcscalar) == 2*sizeof(ncl_dscalar))			*/

#else  /* NCL_INLINE */

ncl_double*		/* pointer to the underlying data array	*/
(ncl_dch_pointer)(ncl_dchandle* ph);

ncl_dchandle		/* automatic storage constructor	*/
(ncl_dch_create)(ncl_dscalar* p);

void				/* automatic storage  destructor	*/
(ncl_dch_destroy)(ncl_dchandle* ph);

int
(ncl_dch_empty)(const ncl_dchandle* ph);

ncl_dcscalar		/* h[o]					*/
(ncl_dch_get)(const ncl_dchandle* ph,
  ncl_offset o);

void				/* h[o] <-- x				*/
(ncl_dch_set)(      ncl_dchandle* ph,
  ncl_offset o, ncl_dcscalar x);

#endif /* NCL_INLINE */

#ifdef NCL_MACROS

#define ncl_dch_pointer(ph) ((ph)->P)
#define ncl_dch_create(ps) (*((ncl_dchandle*)(&(ps))))
/* Note: The pointer ncl_dscalar* ps must be an lvalue!		*/
#define ncl_dch_destroy(ph)
#define ncl_dch_empty(ph) ((ph)->P == NULL)
#define ncl_dch_get(ph, o) (((ncl_dcscalar*)((ph)->P))[o])
#define ncl_dch_set(ph, o, x) (((ncl_dcscalar*)((ph)->P))[o] = (x))

/* Both ncl_dch_get and ncl_dch_set assume that			*/
/* sizeof(ncl_dcscalar) == 2*sizeof(ncl_dscalar))			*/

#endif /* NCL_MACROS */

#endif /* _ncl_dchandle_h */
