#ifndef _ncl_dcmatrix_h
#define _ncl_dcmatrix_h 1
/*
The ANSI C Numerical Class Library.
Copyright (C) 2000 E. Robert Tisdale

This file is part of The ANSI C Numerical Class Library.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#ifdef NCL_INLINE
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif /* NCL_REVEAL */
#endif /* NCL_INLINE */

#ifdef NCL_MACROS
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif /* NCL_REVEAL */
#endif /* NCL_MACROS */

/* Include double precision real matrix type definitions.		*/
#include<ncl_dmatrix.h>

/* Include double precision complex handle type definitions.	*/
#include<ncl_dcvector.h>

#ifdef NCL_REVEAL
/*  Reveal type definitions after applications are thoroughly tested.	*/

typedef struct {		/* submatrix class definition		*/
  /* private: */
  ncl_dchandle	H;
  ncl_offset	O;
  ncl_extent	N1;
  ncl_stride	S1;
  ncl_extent	N2;
  ncl_stride	S2;
  ncl_______	_;
  } ncl_dcsubmatrix;
/* A submatrix does not own the array storage that it references.	*/
/* It does not allocate any array storage when it is constructed	*/
/* nor does it deallocate any array storage when it is destroyed.	*/

#else  /* NCL_REVEAL */
/* Conceal type definitions until applications are thoroughly tested.	*/

#define NCL_DCSUBMATRIX_SIZE NCL_SUBMATRIX_SIZE
typedef int ncl_dchidden_submatrix[NCL_DCSUBMATRIX_SIZE/sizeof(int)];

typedef struct {		/* submatrix class definition		*/
  /* private: */
  ncl_dchidden_submatrix	M;
  } ncl_dcsubmatrix;
/* A submatrix does not own the array storage that it references.	*/
/* It does not allocate any array storage when it is constructed	*/
/* nor does it deallocate any array storage when it is destroyed.	*/

#endif /* NCL_REVEAL */

typedef ncl_dcsubmatrix ncl_dcmatrix;
/* A matrix owns the array storage that it references.			*/ 
/* It allocates array storage for itself when it is constructed		*/
/* and deallocates it's array storage when it is destroyed.		*/

#ifdef NCL_INLINE

inline static			/* define  submatrix operations		*/
ncl_dchandle
(ncl_dcsubm_handle)(const ncl_dcsubmatrix* pM) { return pM->H; }

inline static
ncl_offset
(ncl_dcsubm_offset)(const ncl_dcsubmatrix* pM) { return pM->O; }

inline static
ncl_extent
(ncl_dcsubm_extent1)(const ncl_dcsubmatrix* pM) { return pM->N1; }

inline static
ncl_stride
(ncl_dcsubm_stride1)(const ncl_dcsubmatrix* pM) { return pM->S1; }

inline static
ncl_extent
(ncl_dcsubm_extent2)(const ncl_dcsubmatrix* pM) { return pM->N2; }

inline static
ncl_stride
(ncl_dcsubm_stride2)(const ncl_dcsubmatrix* pM) { return pM->S2; }

inline static
void				/* initializer				*/
(ncl_dcsubm_init)(ncl_dcsubmatrix* pM,
    ncl_dchandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  pM->H  = h; pM->O  = o;
  pM->N1 = n; pM->S1 = s1;
  pM->N2 = m; pM->S2 = s2; pM->_ = ncl_other; }

inline static
void				/* initializer clone			*/
(ncl_dcsubm_initclone)(ncl_dcsubmatrix* pM,
		const ncl_dcsubmatrix* pN) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pN);
  ncl_offset	o  = ncl_dcsubm_offset(pN);
  ncl_extent	n  = ncl_dcsubm_extent1(pN);
  ncl_stride	s1 = ncl_dcsubm_stride1(pN);
  ncl_extent	m  = ncl_dcsubm_extent2(pN);
  ncl_stride	s2 = ncl_dcsubm_stride2(pN);
  ncl_dcsubm_init(pM, h, o, m, s2, n, s1); }

inline static
ncl_dcsubmatrix		/* automatic storage constructor	*/
(ncl_dcsubm_create)(
    ncl_dchandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  ncl_dcsubmatrix	M;
  ncl_dcsubm_init(&M, h, o, m, s2, n, s1);
  return M; }

inline static
ncl_dcsubmatrix		/* automatic storage clone constructor	*/
(ncl_dcsubm_createclone)(const ncl_dcsubmatrix* pN) {
  ncl_dcsubmatrix	M = *pN; M._ = ncl_other;
  return M; }

inline static
void				/* automatic storage  destructor	*/
(ncl_dcsubm_destroy)(ncl_dcsubmatrix* pM) {
  pM->_ = ~ncl_other; }

inline static
ncl_dcsubmatrix*		/*      free storage constructor	*/
(ncl_dcsubm_new)(
    ncl_dchandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  ncl_dcsubmatrix*	pM
    = (ncl_dcsubmatrix*)malloc(sizeof(ncl_dcsubmatrix));
  if (pM)
    ncl_dcsubm_init(pM, h, o, m, s2, n, s1);
  return pM; }

inline static
ncl_dcsubmatrix*		/*      free storage clone constructor	*/
(ncl_dcsubm_newclone)(const ncl_dcsubmatrix* pN) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pN);
  ncl_offset	o  = ncl_dcsubm_offset(pN);
  ncl_extent	n  = ncl_dcsubm_extent1(pN);
  ncl_stride	s1 = ncl_dcsubm_stride1(pN);
  ncl_extent	m  = ncl_dcsubm_extent2(pN);
  ncl_stride	s2 = ncl_dcsubm_stride2(pN);
  return ncl_dcsubm_new(h, o, m, s2, n, s1); }

inline static
void				/*      free storage  destructor	*/
(ncl_dcsubm_delete)(ncl_dcsubmatrix* pM) {
  ncl_dcsubm_destroy(pM);
  free((void*)pM); }

inline static
ncl_dcsubmatrix*		/*                 reconstructor	*/
(ncl_dcsubm_resize)(ncl_dcsubmatrix* pM,
    ncl_dchandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  ncl_dcsubm_init(pM, h, o, m, s2, n, s1);
  return pM; }

inline static
ncl_dcsubmatrix*		/*           clone reconstructor	*/
(ncl_dcsubm_resizeclone)(ncl_dcsubmatrix* pM,
		const ncl_dcsubmatrix* pN) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pN);
  ncl_offset	o  = ncl_dcsubm_offset(pN);
  ncl_extent	n  = ncl_dcsubm_extent1(pN);
  ncl_stride	s1 = ncl_dcsubm_stride1(pN);
  ncl_extent	m  = ncl_dcsubm_extent2(pN);
  ncl_stride	s2 = ncl_dcsubm_stride2(pN);
  return ncl_dcsubm_resize(pM, h, o, m, s2, n, s1); }

inline static
int
(ncl_dcsubm_empty)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h = ncl_dcsubm_handle(pM);
  ncl_extent	n = ncl_dcsubm_extent1(pM);
  ncl_extent	m = ncl_dcsubm_extent2(pM);
  return ncl_dch_empty(&h) || 0 == n || 0 == m; }

inline static
ncl_dcscalar		/* M[i][j]				*/
(ncl_dcsubm_get)(const ncl_dcsubmatrix* pM,
    ncl_offset i, ncl_offset j) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  return ncl_dch_get(&h, o + (ncl_stride)j*s1 + (ncl_stride)i*s2); }

inline static
void				/* M[i][j] = x				*/
(ncl_dcsubm_set)(      ncl_dcsubmatrix* pM,
    ncl_offset i, ncl_offset j, ncl_dcscalar x) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  ncl_dch_set(&h, o + (ncl_stride)j*s1 + (ncl_stride)i*s2, x); }

inline static
ncl_dcsubvector		/*  vector view	*/
(ncl_dcsubm_subv1)(const ncl_dcsubmatrix* pM, ncl_offset j) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  return ncl_dcsubv_create(h, o + (ncl_stride)j*s1, m, s2); }

inline static
ncl_dcsubvector		/*  vector view	*/
(ncl_dcsubm_subv2)(const ncl_dcsubmatrix* pM, ncl_offset i) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  return ncl_dcsubv_create(h, o + (ncl_stride)i*s2, n, s1); }

inline static
ncl_dcsubmatrix		/*  matrix view	*/
(ncl_dcsubm_subm1)(const ncl_dcsubmatrix* pM,
    ncl_offset j, ncl_extent n, ncl_stride s) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  return ncl_dcsubm_create(h, o + (ncl_stride)j*s1, m, s2, n, s*s1); }
  
inline static
ncl_dcsubmatrix		/*  matrix view	*/
(ncl_dcsubm_subm2)(const ncl_dcsubmatrix* pM,
    ncl_offset i, ncl_extent m, ncl_stride s) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  return ncl_dcsubm_create(h, o + (ncl_stride)i*s2, m, s*s2, n, s1); }
  
inline static
ncl_dcsubmatrix		/*  matrix view	*/
(ncl_dcsubm_subm)(const ncl_dcsubmatrix* pM,
    ncl_offset i, ncl_extent m, ncl_stride s2,
    ncl_offset j, ncl_extent n, ncl_stride s1) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_stride	s1_= ncl_dcsubm_stride1(pM);
  ncl_stride	s2_= ncl_dcsubm_stride2(pM);
  return ncl_dcsubm_create(h, o + (ncl_stride)j*s1 + (ncl_stride)i*s2,
    m, s2*s2_, n, s1*s1_); }
  
inline static
ncl_dcsubmatrix		/* reverse view	*/
(ncl_dcsubm_r1)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  o += (0 < n)? (ncl_stride)(n-1)*s1: 0;
  return ncl_dcsubm_create(h, o, m, +s2, n, -s1); }

inline static
ncl_dcsubmatrix		/* reverse view	*/
(ncl_dcsubm_r2)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  o += (0 < m)? (ncl_stride)(m-1)*s2: 0;
  return ncl_dcsubm_create(h, o, m, -s2, n, +s1); }

inline static
ncl_dcsubmatrix		/* reverse view	*/
(ncl_dcsubm_r)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  o += (0 < n)? (ncl_stride)(n-1)*s1: 0;
  o += (0 < m)? (ncl_stride)(m-1)*s2: 0;
  return ncl_dcsubm_create(h, o, m, -s2, n, -s1); }

inline static
ncl_dcsubmatrix	/*       transpose view	*/
(ncl_dcsubm_t)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  return ncl_dcsubm_create(h, o, n, s1, m, s2); }

inline static
ncl_dcsubvector	/*        diagonal view	*/
(ncl_dcsubm_diag)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  return ncl_dcsubv_create(h, o, ((m < n)? m: n), s1 + s2); }

inline static
ncl_dcsubmatrix		/* columns 0, 2, ...,   2j, ...	*/
(ncl_dcsubm_even)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  return ncl_dcsubm_create(h, o, m, s2, (n+1) >> 1, s1 << 1); }

inline static
ncl_dcsubmatrix		/* columns 1, 3, ..., 1+2j, ...	*/
(ncl_dcsubm_odd)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  return ncl_dcsubm_create(h, o, m, s2, n >> 1, s1 << 1); }

inline static
ncl_dcsubmatrix*		/* M[i] <-> M[k]			*/
(ncl_dcsubm_vswap)(ncl_dcsubmatrix* pM, ncl_offset i, ncl_offset k) {
  ncl_dcsubvector v = ncl_dcsubm_subv2(pM, i);
  ncl_dcsubvector u = ncl_dcsubm_subv2(pM, k);
  ncl_dcsubv_vswap(&v, &u);
  ncl_dcsubv_destroy(&u);
  ncl_dcsubv_destroy(&v);
  return pM; }

#else  /* NCL_INLINE */
				/* declare submatrix operations		*/
ncl_dchandle
(ncl_dcsubm_handle)(const ncl_dcsubmatrix* pM);

ncl_offset
(ncl_dcsubm_offset)(const ncl_dcsubmatrix* pM);

ncl_extent
(ncl_dcsubm_extent1)(const ncl_dcsubmatrix* pM);

ncl_stride
(ncl_dcsubm_stride1)(const ncl_dcsubmatrix* pM);

ncl_extent
(ncl_dcsubm_extent2)(const ncl_dcsubmatrix* pM);

ncl_stride
(ncl_dcsubm_stride2)(const ncl_dcsubmatrix* pM);

void				/* initializer				*/
(ncl_dcsubm_init)(ncl_dcsubmatrix* pM,
    ncl_dchandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1);

void				/* initializer				*/
(ncl_dcsubm_initclone)(ncl_dcsubmatrix* pM,
		const ncl_dcsubmatrix* pN);

ncl_dcsubmatrix		/* automatic storage constructor	*/
(ncl_dcsubm_create)(
    ncl_dchandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1);

ncl_dcsubmatrix		/* automatic storage clone constructor	*/
(ncl_dcsubm_createclone)(const ncl_dcsubmatrix* pN);

void				/* automatic storage  destructor	*/
(ncl_dcsubm_destroy)(ncl_dcsubmatrix* pM);

ncl_dcsubmatrix*		/*      free storage constructor	*/
(ncl_dcsubm_new)(
    ncl_dchandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1);

ncl_dcsubmatrix*		/*      free storage clone constructor	*/
(ncl_dcsubm_newclone)(const ncl_dcsubmatrix* pN);

void				/*      free storage  destructor	*/
(ncl_dcsubm_delete)(ncl_dcsubmatrix* pM);

ncl_dcsubmatrix*		/*                 reconstructor	*/
(ncl_dcsubm_resize)(ncl_dcsubmatrix* pM,
    ncl_dchandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1);

ncl_dcsubmatrix*		/*           clone reconstructor	*/
(ncl_dcsubm_resizeclone)(ncl_dcsubmatrix* pM,
		const ncl_dcsubmatrix* pN);

int
(ncl_dcsubm_empty)(const ncl_dcsubmatrix* pM);

ncl_dcscalar		/* M[i][j]				*/
(ncl_dcsubm_get)(const ncl_dcsubmatrix* pM,
    ncl_offset i, ncl_offset j);

void				/* M[i][j] = x				*/
(ncl_dcsubm_set)(      ncl_dcsubmatrix* pM,
    ncl_offset i, ncl_offset j, ncl_dcscalar x);

ncl_dcsubvector		/*  vector view	*/
(ncl_dcsubm_subv2)(const ncl_dcsubmatrix* pM, ncl_offset i);

ncl_dcsubvector		/*  vector view	*/
(ncl_dcsubm_subv1)(const ncl_dcsubmatrix* pM, ncl_offset j);

ncl_dcsubmatrix		/*  matrix view	*/
(ncl_dcsubm_subm1)(const ncl_dcsubmatrix* pM,
    ncl_offset j, ncl_extent n, ncl_stride s);

ncl_dcsubmatrix		/*  matrix view	*/
(ncl_dcsubm_subm2)(const ncl_dcsubmatrix* pM,
    ncl_offset i, ncl_extent m, ncl_stride s);

ncl_dcsubmatrix		/*  matrix view	*/
(ncl_dcsubm_subm)(const ncl_dcsubmatrix* pM,
    ncl_offset i, ncl_extent m, ncl_stride s2,
    ncl_offset j, ncl_extent n, ncl_stride s1);

ncl_dcsubmatrix		/* reverse view	*/
(ncl_dcsubm_r1)(const ncl_dcsubmatrix* pM);

ncl_dcsubmatrix		/* reverse view	*/
(ncl_dcsubm_r2)(const ncl_dcsubmatrix* pM);

ncl_dcsubmatrix		/* reverse view	*/
(ncl_dcsubm_r)(const ncl_dcsubmatrix* pM);

ncl_dcsubmatrix	/*       transpose view	*/
(ncl_dcsubm_t)(const ncl_dcsubmatrix* pM);

ncl_dcsubvector	/*        diagonal view	*/
(ncl_dcsubm_diag)(const ncl_dcsubmatrix* pM);

ncl_dcsubmatrix		/* columns 0, 2, ...,   2j, ...		*/
(ncl_dcsubm_even)(const ncl_dcsubmatrix* pM);

ncl_dcsubmatrix		/* columns 1, 3, ..., 1+2j, ...		*/
(ncl_dcsubm_odd)(const ncl_dcsubmatrix* pM);

ncl_dcsubmatrix*		/* M[i] <-> M[k]			*/
(ncl_dcsubm_vswap)(ncl_dcsubmatrix* pM, ncl_offset i, ncl_offset k);

ncl_dcsubmatrix*		/* M <-> N				*/
(ncl_dcsubm_mswap)(ncl_dcsubmatrix* pM, ncl_dcsubmatrix* pN);

#endif /* NCL_INLINE */

ncl_dcsubmatrix*
(ncl_dcsubm_reverse)(ncl_dcsubmatrix* pM);

					/* matrix-scalar operations	*/
ncl_dcsubmatrix*			/* M  = x */
(ncl_dcsubm_fill)(ncl_dcsubmatrix* pM, ncl_dcscalar x);

ncl_dcsubmatrix*			/* M *= x */
(ncl_dcsubm_smul)(ncl_dcsubmatrix* pM, ncl_dcscalar x);

ncl_dcsubmatrix*			/* M /= x */
(ncl_dcsubm_sdiv)(ncl_dcsubmatrix* pM, ncl_dcscalar x);

ncl_dcsubmatrix*			/* M += x */
(ncl_dcsubm_sadd)(ncl_dcsubmatrix* pM, ncl_dcscalar x);

ncl_dcsubmatrix*			/* M -= x */
(ncl_dcsubm_ssub)(ncl_dcsubmatrix* pM, ncl_dcscalar x);

					/* matrix-matrix operations	*/
ncl_dcsubmatrix*			/* M  = N */
(ncl_dcsubm_mcpy)(ncl_dcsubmatrix* pM, const ncl_dcsubmatrix* pN);

ncl_dcsubmatrix*			/* M *= N */
(ncl_dcsubm_mmul)(ncl_dcsubmatrix* pM, const ncl_dcsubmatrix* pN);

ncl_dcsubmatrix*			/* M /= N */
(ncl_dcsubm_mdiv)(ncl_dcsubmatrix* pM, const ncl_dcsubmatrix* pN);

ncl_dcsubmatrix*			/* M += N */
(ncl_dcsubm_madd)(ncl_dcsubmatrix* pM, const ncl_dcsubmatrix* pN);

ncl_dcsubmatrix*			/* M -= N */
(ncl_dcsubm_msub)(ncl_dcsubmatrix* pM, const ncl_dcsubmatrix* pN);

int
(ncl_dcsubm_fprintf)(FILE* stream, const ncl_dcsubmatrix* pM,
  int width, unsigned int precision, ncl_fmtflags flags, unsigned int columns);

#ifdef NCL_INLINE

inline static			/* define     matrix operations		*/
ncl_dcmatrix		/* automatic storage constructor	*/
(ncl_dcm_create)(ncl_extent m, ncl_extent n) {
  ncl_dcsubmatrix	M;
  ncl_dscalar*	ps
    = (ncl_dscalar*)((0 < m*n)? malloc(m*n*sizeof(ncl_dcscalar)): NULL);
  ncl_dcsubm_init(&M, ncl_dch_create(ps),
    (ncl_offset)0, m, (ncl_stride)n, n, (ncl_stride)1); M._ = ncl_owner;
  return M; }

inline static
ncl_dcmatrix		/* automatic storage clone constructor	*/
(ncl_dcm_createclone)(const ncl_dcsubmatrix* pN) {
  ncl_extent	n = ncl_dcsubm_extent1(pN);
  ncl_extent	m = ncl_dcsubm_extent2(pN);
  ncl_dcmatrix	M = ncl_dcm_create(m, n);
  ncl_dcsubm_mcpy(&M, pN);
  return M; }

inline static
void				/* automatic storage  destructor	*/
(ncl_dcm_destroy)(ncl_dcmatrix* pM) {
  ncl_dchandle	h = ncl_dcsubm_handle(pM);
  pM->_ = ~ncl_owner;
  free((void*)(ncl_dch_pointer(&h))); }

inline static
ncl_dcmatrix*		/*      free storage constructor	*/
(ncl_dcm_new)(ncl_extent m, ncl_extent n) {
  ncl_dcsubmatrix*	pM
    = (ncl_dcsubmatrix*)malloc(sizeof(ncl_dcsubmatrix));
  if (pM) {
    ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < m*n)? malloc(m*n*sizeof(ncl_dcscalar)): NULL);
    ncl_dcsubm_init(pM, ncl_dch_create(ps),
    (ncl_offset)0, m, n, n, (ncl_stride)1); pM->_ = ncl_owner;
    }
  return pM; }

inline static
ncl_dcmatrix*		/*      free storage clone constructor	*/
(ncl_dcm_newclone)(const ncl_dcsubmatrix* pN) {
  ncl_extent	n = ncl_dcsubm_extent1(pN);
  ncl_extent	m = ncl_dcsubm_extent2(pN);
  return ncl_dcsubm_mcpy(ncl_dcm_new(m, n), pN); }

inline static
void				/*      free storage  destructor	*/
(ncl_dcm_delete)(ncl_dcmatrix* pM) {
  ncl_dcm_destroy(pM);
  free((void*)pM); }

inline static
ncl_dcmatrix*		/*                 reconstructor	*/
(ncl_dcm_resize)(ncl_dcmatrix* pM, ncl_extent m, ncl_extent n) {
  ncl_dchandle	h = ncl_dcsubm_handle(pM);
  if (!ncl_dch_empty(&h)) {
    free((void*)(ncl_dch_pointer(&h)));
    }
  { ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < m*n)? malloc(m*n*sizeof(ncl_dcscalar)): NULL);
    ncl_dcsubm_init(pM, ncl_dch_create(ps),
      (ncl_offset)0, m, n, n, (ncl_stride)1); pM->_ = ncl_owner;
    }
  return pM; }
  
inline static
ncl_dcmatrix*		/*           clone reconstructor	*/
(ncl_dcm_resizeclone)(ncl_dcmatrix* pM,
		const ncl_dcsubmatrix* pN) {
  ncl_extent	n = ncl_dcsubm_extent1(pN);
  ncl_extent	m = ncl_dcsubm_extent2(pN);
  return ncl_dcsubm_mcpy(ncl_dcm_resize(pM, m, n), pN); }

#else  /* NCL_INLINE */
				/* declare   matrix operations		*/
ncl_dcmatrix		/* automatic storage constructor	*/
(ncl_dcm_create)(ncl_extent m, ncl_extent n);

ncl_dcmatrix		/* automatic storage clone constructor	*/
(ncl_dcm_createclone)(const ncl_dcsubmatrix* pN);

void				/* automatic storage  destructor	*/
(ncl_dcm_destroy)(ncl_dcmatrix* pM);

ncl_dcmatrix*		/*      free storage constructor	*/
(ncl_dcm_new)(ncl_extent m, ncl_extent n);

ncl_dcmatrix*		/*      free storage clone constructor	*/
(ncl_dcm_newclone)(const ncl_dcsubmatrix* pN);

void				/*      free storage  destructor	*/
(ncl_dcm_delete)(ncl_dcmatrix* pM);

ncl_dcmatrix*		/*                 reconstructor	*/
(ncl_dcm_resize)(ncl_dcmatrix* pM, ncl_extent m, ncl_extent n);

ncl_dcmatrix*		/*           clone reconstructor	*/
(ncl_dcm_resizeclone)(ncl_dcmatrix* pM,
		const ncl_dcsubmatrix* pN);

#endif /* NCL_INLINE */

#ifdef NCL_MACROS

#define ncl_dcsubm_handle(pM)  ((pM)->H)
#define ncl_dcsubm_offset(pM)  ((pM)->O)
#define ncl_dcsubm_extent1(pM) ((pM)->N1)
#define ncl_dcsubm_stride1(pM) ((pM)->S1)
#define ncl_dcsubm_extent2(pM) ((pM)->N2)
#define ncl_dcsubm_stride2(pM) ((pM)->S2)
#define ncl_dcsubm_init(pM, h, o, n2, s2, n1, s1) \
do { \
  (pM)->H  = (h);  (pM)->O  = (o); \
  (pM)->N1 = (n1); (pM)->S1 = (s1); \
  (pM)->N2 = (n2); (pM)->S2 = (s2); (pM)->_ = ncl_other; \
  } while (0)
#define ncl_dcsubm_initclone(pM, pN) ncl_dcsubm_init(pM, \
ncl_dcsubm_handle(pN), ncl_dcsubm_offset(pN), \
ncl_dcsubm_extent2(pN), ncl_dcsubm_stride2(pN), \
ncl_dcsubm_extent1(pN), ncl_dcsubm_stride1(pN))
#define ncl_dcsubm_createclone(pN) ncl_dcsubm_create( \
ncl_dcsubm_handle(pN), ncl_dcsubm_offset(pN), \
ncl_dcsubm_extent2(pN), ncl_dcsubm_stride2(pN), \
ncl_dcsubm_extent1(pN), ncl_dcsubm_stride1(pN))
#define ncl_dcsubm_newclone(pN) ncl_dcsubm_new( \
ncl_dcsubm_handle(pN), ncl_dcsubm_offset(pN), \
ncl_dcsubm_extent2(pN), ncl_dcsubm_stride2(pN), \
ncl_dcsubm_extent1(pN), ncl_dcsubm_stride1(pN))
#define ncl_dcsubm_destroy(pM) ((pM)->_ = ~ncl_other)
#define ncl_dcsubm_delete(pM) \
do { ncl_dcsubm_destroy(pM); free((void*)(pM)); } while (0)
#define ncl_dcsubm_empty(pM) \
 (ncl_dch_empty(&((pM)->H)) || \
0 == ncl_dcsubm_extent1(pM) || \
0 == ncl_dcsubm_extent2(pM))
#define ncl_dcsubm_get(pM, i, j) \
ncl_dch_get(&((pM)->H), ncl_dcsubm_offset(pM) \
  + (ncl_stride)(j)*ncl_dcsubm_stride1(pM) \
  + (ncl_stride)(i)*ncl_dcsubm_stride2(pM))
#define ncl_dcsubm_set(pM, i, j, x) \
ncl_dch_set(&((pM)->H), ncl_dcsubm_offset(pM) \
  + (ncl_stride)(j)*ncl_dcsubm_stride1(pM) \
  + (ncl_stride)(i)*ncl_dcsubm_stride2(pM), (x))
#define ncl_dcm_destroy(pM) \
do { \
  (pM)->_ = ~ncl_owner; \
  free((void*)(ncl_dch_pointer(&ncl_dcsubm_handle(pM)))); \
  } while (0)
#define ncl_dcm_delete(pM) \
do { ncl_dcm_destroy(pM); free((void*)(pM)); } while (0)

#endif /* NCL_MACROS */

#endif /* _ncl_dcmatrix_h */
