#ifndef _ncl_dcscalar_h
#define _ncl_dcscalar_h 1
/*
The ANSI C Numerical Class Library.
Copyright (C) 2000 E. Robert Tisdale

This file is part of The ANSI C Numerical Class Library.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

/* Include real scalar type definitions.	*/
#include<ncl_dscalar.h>

typedef struct {
  /* public: */
  ncl_double	R, I;
  } ncl_dcomplex;

typedef ncl_dcomplex	ncl_dcscalar;

#ifdef NCL_INLINE

inline static
ncl_dcomplex
(ncl_dcmplx)(ncl_double r, ncl_double i) {
  ncl_dcomplex	c;
  c.R = r; c.I = i;
  return c; }

inline static
ncl_dscalar
(ncl_dcreal)(ncl_dcomplex c) {
  return c.R; }

inline static
ncl_dscalar
(ncl_dcimag)(ncl_dcomplex c) {
  return c.I; }

#else  /* NCL_INLINE */

ncl_dcomplex
(ncl_dcmplx)(ncl_double r, ncl_double i);

ncl_double
(ncl_dcreal)(ncl_dcomplex c);

ncl_double
(ncl_dcimag)(ncl_dcomplex c);

#endif /* NCL_INLINE */

#ifdef NCL_MACROS

#define ncl_dcreal(c) ((c).R)
#define ncl_dcimag(c) ((c).R)

#endif /* NCL_MACROS */

#define NCL_DCSET(c, r, i) do { (c).R = (r); (c).I = (i); }
#define NCL_DCMUL(t, x, y) \
do { \
  (t).R = (x).R*(y).R - (x).I*(y).I; \
  (t).I = (x).R*(y).I + (x).I*(y).R; \
  } while (0)
#define NCL_DCDIV(t, x, y) \
do { \
  (t).I = ((x).R*(x).R + (x).I*(x).I); \
  (t).R = ((x).R*(y).R + (x).I*(y).I)/(t).I; \
  (t).I = ((x).I*(y).R - (x).R*(y).I)/(t).I; \
  } while (0)
#define NCL_DCADD(t, x, y) \
do { \
  (t).R = (x).R + (y).R; \
  (t).I = (x).I + (y).I; \
  } while (0)
#define NCL_DCSUB(t, x, y) \
do { \
  (t).R = (x).R - (y).R; \
  (t).I = (x).I - (y).I; \
  } while (0)

int
(ncl_dcfprintf)(FILE *stream, ncl_dcscalar x,
    int width, unsigned int precision, ncl_fmtflags flags);

#endif /* _ncl_dcscalar_h */
