#ifndef _ncl_dcvector_h
#define _ncl_dcvector_h 1
/*
The ANSI C Numerical Class Library.
Copyright (C) 2000 E. Robert Tisdale

This file is part of The ANSI C Numerical Class Library.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#ifdef NCL_INLINE
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif /* NCL_REVEAL */
#endif /* NCL_INLINE */

#ifdef NCL_MACROS
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif /* NCL_REVEAL */
#endif /* NCL_MACROS */

/* Include double precision real vector type definitions.		*/
#include<ncl_dvector.h>

/* Include double precision complex handle type definitions.	*/
#include<ncl_dchandle.h>

#ifdef NCL_REVEAL
/*  Reveal type definitions after applications are thoroughly tested.	*/

typedef struct {		/* subvector class definition		*/
  /* private: */
  ncl_dchandle	H;
  ncl_offset	O;
  ncl_extent	N;
  ncl_stride	S;
  ncl_______	_;
  } ncl_dcsubvector;
/* A subvector does not own the array storage that it references.	*/
/* It does not allocate any array storage when it is constructed	*/
/* nor does it deallocate any array storage when it is destroyed.	*/

#else  /* NCL_REVEAL */
/* Conceal type definitions until applications are thoroughly tested.	*/

#define NCL_DCSUBVECTOR_SIZE NCL_SUBVECTOR_SIZE
typedef int ncl_dchidden_subvector[NCL_DCSUBVECTOR_SIZE/sizeof(int)];

typedef struct {		/* subvector class definition		*/
  /* private: */
  ncl_dchidden_subvector	V;
  } ncl_dcsubvector;
/* A subvector does not own the array storage that it references.	*/
/* It does not allocate any array storage when it is constructed	*/
/* nor does it deallocate any array storage when it is destroyed.	*/

#endif /* NCL_REVEAL */

typedef ncl_dcsubvector ncl_dcvector;
/* A vector owns the array storage that it references.			*/ 
/* It allocates array storage for itself when it is constructed		*/
/* and deallocates it's array storage when it is destroyed.		*/

#ifdef NCL_INLINE

inline static			/* define  subvector operations		*/
ncl_dchandle
(ncl_dcsubv_handle)(const ncl_dcsubvector* pv) { return pv->H; }

inline static
ncl_offset
(ncl_dcsubv_offset)(const ncl_dcsubvector* pv) { return pv->O; }

inline static
ncl_extent
(ncl_dcsubv_extent)(const ncl_dcsubvector* pv) { return pv->N; }

inline static
ncl_stride
(ncl_dcsubv_stride)(const ncl_dcsubvector* pv) { return pv->S; }

inline static
void				/* initializer				*/
(ncl_dcsubv_init)(ncl_dcsubvector* pv,
    ncl_dchandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  pv->H = h; pv->O = o; pv->N = n; pv->S = s; pv->_ = ncl_other; }

inline static
void				/* initializer clone			*/
(ncl_dcsubv_initclone)(ncl_dcsubvector* pv,
		const ncl_dcsubvector* pu) {
  ncl_dchandle	h = ncl_dcsubv_handle(pu);
  ncl_offset	o = ncl_dcsubv_offset(pu);
  ncl_extent	n = ncl_dcsubv_extent(pu);
  ncl_stride	s = ncl_dcsubv_stride(pu);
  ncl_dcsubv_init(pv, h, o, n, s); }

inline static
ncl_dcsubvector		/* automatic storage constructor	*/
(ncl_dcsubv_create)(
    ncl_dchandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  ncl_dcsubvector	v;
  ncl_dcsubv_init(&v, h, o, n, s);
  return v; }

inline static
ncl_dcsubvector		/* automatic storage clone constructor	*/
(ncl_dcsubv_createclone)(const ncl_dcsubvector* pu) {
  ncl_dcsubvector	v = *pu; v._ = ncl_other;
  return v; }

inline static
void				/* automatic storage  destructor	*/
(ncl_dcsubv_destroy)(ncl_dcsubvector* pv) {
  pv->_ = ~ncl_other; }

inline static
ncl_dcsubvector*		/*      free storage constructor	*/
(ncl_dcsubv_new)(
    ncl_dchandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  ncl_dcsubvector*	pv
    = (ncl_dcsubvector*)malloc(sizeof(ncl_dcsubvector));
  if (pv)
    ncl_dcsubv_init(pv, h, o, n, s);
  return pv; }

inline static
ncl_dcsubvector*		/*      free storage clone constructor	*/
(ncl_dcsubv_newclone)(const ncl_dcsubvector* pu) {
  ncl_dchandle	h = ncl_dcsubv_handle(pu);
  ncl_offset	o = ncl_dcsubv_offset(pu);
  ncl_extent	n = ncl_dcsubv_extent(pu);
  ncl_stride	s = ncl_dcsubv_stride(pu);
  return ncl_dcsubv_new(h, o, n, s); }

inline static
void				/*      free storage  destructor	*/
(ncl_dcsubv_delete)(ncl_dcsubvector* pv) {
  ncl_dcsubv_destroy(pv);
  free((void*)pv); }

inline static
ncl_dcsubvector*		/*                 reconstructor	*/
(ncl_dcsubv_resize)(ncl_dcsubvector* pv,
    ncl_dchandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  ncl_dcsubv_init(pv, h, o, n, s);
  return pv; }

inline static
ncl_dcsubvector*		/*           clone reconstructor	*/
(ncl_dcsubv_resizeclone)(ncl_dcsubvector* pv,
		const ncl_dcsubvector* pu) {
  ncl_dchandle	h = ncl_dcsubv_handle(pu);
  ncl_offset	o = ncl_dcsubv_offset(pu);
  ncl_extent	n = ncl_dcsubv_extent(pu);
  ncl_stride	s = ncl_dcsubv_stride(pu);
  return ncl_dcsubv_resize(pv, h, o, n, s); }

inline static
int
(ncl_dcsubv_empty)(const ncl_dcsubvector* pv) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_extent	n = ncl_dcsubv_extent(pv);
  return ncl_dch_empty(&h) || 0 == n; }

inline static
ncl_dcscalar
(ncl_dcsubv_get)(const ncl_dcsubvector* pv,
    ncl_offset j) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
  return ncl_dch_get(&h, o + (ncl_stride)j*s); }

inline static
void
(ncl_dcsubv_set)(      ncl_dcsubvector* pv,
    ncl_offset j, ncl_dcscalar x) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
  ncl_dch_set(&h, o + (ncl_stride)j*s, x); }

inline static
ncl_dcsubvector		/*  vector view	*/
(ncl_dcsubv_subv)(const ncl_dcsubvector* pv,
    ncl_offset j, ncl_extent n, ncl_stride s) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_stride	s_= ncl_dcsubv_stride(pv);
  return ncl_dcsubv_create(h, o + (ncl_stride)j*s, n, s*s_); }
  
inline static
ncl_dcsubvector		/* reverse view	*/
(ncl_dcsubv_r)(const ncl_dcsubvector* pv) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_extent	n = ncl_dcsubv_extent(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
  o += (0 < n)? (ncl_stride)(n-1)*s: 0;
  return ncl_dcsubv_create(h, o, n, -s); }

inline static
ncl_dcsubvector		/* elements 0, 2, ...,   2j, ...	*/
(ncl_dcsubv_even)(const ncl_dcsubvector* pv) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_extent	n = ncl_dcsubv_extent(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
  return ncl_dcsubv_create(h, o, (n+1) >> 1, s << 1); }

inline static
ncl_dcsubvector		/* elements 1, 3, ..., 1+2j, ...	*/
(ncl_dcsubv_odd)(const ncl_dcsubvector* pv) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_extent	n = ncl_dcsubv_extent(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
  return ncl_dcsubv_create(h, o, n >> 1, s << 1); }

inline static
ncl_dcsubvector*		/* v[j] <-> v[k]			*/
(ncl_dcsubv_sswap)(ncl_dcsubvector* pv, ncl_offset j, ncl_offset k) {
  ncl_dcscalar	t = ncl_dcsubv_get(pv, j);
  ncl_dcsubv_set(pv, j, ncl_dcsubv_get(pv, k));
  ncl_dcsubv_set(pv, k, t);
  return pv; }

#else  /* NCL_INLINE */
				/* declare subvector operations		*/
ncl_dchandle
(ncl_dcsubv_handle)(const ncl_dcsubvector* pv);

ncl_offset
(ncl_dcsubv_offset)(const ncl_dcsubvector* pv);

ncl_extent
(ncl_dcsubv_extent)(const ncl_dcsubvector* pv);

ncl_stride
(ncl_dcsubv_stride)(const ncl_dcsubvector* pv);

void				/* initializer				*/
(ncl_dcsubv_init)(ncl_dcsubvector* pv,
    ncl_dchandle h, ncl_offset o, ncl_extent n, ncl_stride s);

void				/* initializer				*/
(ncl_dcsubv_initclone)(ncl_dcsubvector* pv,
		const ncl_dcsubvector* pu);

ncl_dcsubvector		/* automatic storage constructor	*/
(ncl_dcsubv_create)(
    ncl_dchandle h, ncl_offset o, ncl_extent n, ncl_stride s);

ncl_dcsubvector		/* automatic storage clone constructor	*/
(ncl_dcsubv_createclone)(const ncl_dcsubvector* pu);

void				/* automatic storage  destructor	*/
(ncl_dcsubv_destroy)(ncl_dcsubvector* pv);

ncl_dcsubvector*		/*      free storage constructor	*/
(ncl_dcsubv_new)(
    ncl_dchandle h, ncl_offset o, ncl_extent n, ncl_stride s);

ncl_dcsubvector*		/*      free storage clone constructor	*/
(ncl_dcsubv_newclone)(const ncl_dcsubvector* pu);

void				/*      free storage  destructor	*/
(ncl_dcsubv_delete)(ncl_dcsubvector* pv);

ncl_dcsubvector*		/*                 reconstructor	*/
(ncl_dcsubv_resize)(ncl_dcsubvector* pv,
    ncl_dchandle h, ncl_offset o, ncl_extent n, ncl_stride s);

ncl_dcsubvector*		/*           clone reconstructor	*/
(ncl_dcsubv_resizeclone)(ncl_dcsubvector* pv,
		const ncl_dcsubvector* pu);

int
(ncl_dcsubv_empty)(const ncl_dcsubvector* pv);

ncl_dcscalar
(ncl_dcsubv_get)(const ncl_dcsubvector* pv,
    ncl_offset j);

void
(ncl_dcsubv_set)(      ncl_dcsubvector* pv,
    ncl_offset j, ncl_dcscalar x);

ncl_dcsubvector		/*  vector view	*/
(ncl_dcsubv_subv)(const ncl_dcsubvector* pv,
    ncl_offset j, ncl_extent n, ncl_stride s);

ncl_dcsubvector		/* reverse view	*/
(ncl_dcsubv_r)(const ncl_dcsubvector* pv);

ncl_dcsubvector		/* elements 0, 2, ...,   2j, ...	*/
(ncl_dcsubv_even)(const ncl_dcsubvector* pv);

ncl_dcsubvector		/* elements 1, 3, ..., 1+2j, ...	*/
(ncl_dcsubv_odd)(const ncl_dcsubvector* pv);

ncl_dcsubvector*		/* v[j] <-> v[k]			*/
(ncl_dcsubv_sswap)(ncl_dcsubvector* pv, ncl_offset j, ncl_offset k);

#endif /* NCL_INLINE */

ncl_dcsubvector*		/* v <-> u				*/
(ncl_dcsubv_vswap)(ncl_dcsubvector* pv, ncl_dcsubvector* pu);

ncl_dcsubvector*
(ncl_dcsubv_reverse)(ncl_dcsubvector* pv);

					/* vector-scalar operations	*/
ncl_dcsubvector*			/* v  = x */
(ncl_dcsubv_fill)(ncl_dcsubvector* pv, ncl_dcscalar x);

ncl_dcsubvector*			/* v *= x */
(ncl_dcsubv_smul)(ncl_dcsubvector* pv, ncl_dcscalar x);

ncl_dcsubvector*			/* v /= x */
(ncl_dcsubv_sdiv)(ncl_dcsubvector* pv, ncl_dcscalar x);

ncl_dcsubvector*			/* v += x */
(ncl_dcsubv_sadd)(ncl_dcsubvector* pv, ncl_dcscalar x);

ncl_dcsubvector*			/* v -= x */
(ncl_dcsubv_ssub)(ncl_dcsubvector* pv, ncl_dcscalar x);

					/* vector-vector operations	*/
ncl_dcsubvector*			/* v  = w */
(ncl_dcsubv_vcpy)(ncl_dcsubvector* pv, const ncl_dcsubvector* pw);

ncl_dcsubvector*			/* v *= w */
(ncl_dcsubv_vmul)(ncl_dcsubvector* pv, const ncl_dcsubvector* pw);

ncl_dcsubvector*			/* v /= w */
(ncl_dcsubv_vdiv)(ncl_dcsubvector* pv, const ncl_dcsubvector* pw);

ncl_dcsubvector*			/* v += w */
(ncl_dcsubv_vadd)(ncl_dcsubvector* pv, const ncl_dcsubvector* pw);

ncl_dcsubvector*			/* v -= w */
(ncl_dcsubv_vsub)(ncl_dcsubvector* pv, const ncl_dcsubvector* pw);

int
(ncl_dcsubv_fprintf)(FILE* stream, const ncl_dcsubvector* pv,
  int width, unsigned int precision, ncl_fmtflags flags, unsigned int columns);

#ifdef NCL_INLINE

inline static			/* define     vector operations		*/
ncl_dcvector		/* automatic storage constructor	*/
(ncl_dcv_create)(ncl_extent n) {
  ncl_dcsubvector	v;
  ncl_dscalar*	ps
    = (ncl_dscalar*)((0 < n)? malloc(n*sizeof(ncl_dcscalar)): NULL);
  ncl_dcsubv_init(&v, ncl_dch_create(ps),
    (ncl_offset)0, n, (ncl_stride)1); v._ = ncl_owner;
  return v; }

inline static
ncl_dcvector		/* automatic storage clone constructor	*/
(ncl_dcv_createclone)(const ncl_dcsubvector* pu) {
  ncl_dcvector	v = ncl_dcv_create(ncl_dcsubv_extent(pu));
  ncl_dcsubv_vcpy(&v, pu);
  return v; }

inline static
void				/* automatic storage  destructor	*/
(ncl_dcv_destroy)(ncl_dcvector* pv) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  pv->_ = ~ncl_owner;
  free((void*)(ncl_dch_pointer(&h)));
  }

inline static
ncl_dcvector*		/*      free storage constructor	*/
(ncl_dcv_new)(ncl_extent n) {
  ncl_dcsubvector*	pv
    = (ncl_dcsubvector*)malloc(sizeof(ncl_dcsubvector));
  if (pv) {
    ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < n)? malloc(n*sizeof(ncl_dcscalar)): NULL);
    ncl_dcsubv_init(pv, ncl_dch_create(ps),
    (ncl_offset)0, n, (ncl_stride)1); pv->_ = ncl_owner;
    }
  return pv; }

inline static
ncl_dcvector*		/*      free storage clone constructor	*/
(ncl_dcv_newclone)(const ncl_dcsubvector* pu) {
  return ncl_dcsubv_vcpy(ncl_dcv_new(ncl_dcsubv_extent(pu)), pu); }

inline static
void				/*      free storage  destructor	*/
(ncl_dcv_delete)(ncl_dcvector* pv) {
  ncl_dcv_destroy(pv);
  free((void*)pv); }

inline static
ncl_dcvector*		/*                 reconstructor	*/
(ncl_dcv_resize)(ncl_dcvector* pv, ncl_extent n) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  if (!ncl_dch_empty(&h)) {
    free((void*)(ncl_dch_pointer(&h)));
    }
  { ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < n)? malloc(n*sizeof(ncl_dcscalar)): NULL);
    ncl_dcsubv_init(pv, ncl_dch_create(ps),
      (ncl_offset)0, n, (ncl_stride)1); pv->_ = ncl_owner;
    }
  return pv; }
  
inline static
ncl_dcvector*		/*           clone reconstructor	*/
(ncl_dcv_resizeclone)(
    ncl_dcvector* pv, const ncl_dcsubvector* pu) {
  return ncl_dcsubv_vcpy(
    ncl_dcv_resize(pv, ncl_dcsubv_extent(pu)), pu); }

#else  /* NCL_INLINE */
				/* declare   vector operations		*/
ncl_dcvector		/* automatic storage constructor	*/
(ncl_dcv_create)(ncl_extent n);

ncl_dcvector		/* automatic storage clone constructor	*/
(ncl_dcv_createclone)(const ncl_dcsubvector* pu);

void				/* automatic storage  destructor	*/
(ncl_dcv_destroy)(ncl_dcvector* pv);

ncl_dcvector*		/*      free storage constructor	*/
(ncl_dcv_new)(ncl_extent n);

ncl_dcvector*		/*      free storage clone constructor	*/
(ncl_dcv_newclone)(const ncl_dcsubvector* pu);

void				/*      free storage  destructor	*/
(ncl_dcv_delete)(ncl_dcvector* pv);

ncl_dcvector*		/*                 reconstructor	*/
(ncl_dcv_resize)(ncl_dcvector* pv, ncl_extent n);

ncl_dcvector*		/*           clone reconstructor	*/
(ncl_dcv_resizeclone)(ncl_dcvector* pv,
		const ncl_dcsubvector* pu);

#endif /* NCL_INLINE */

#ifdef NCL_MACROS

#define ncl_dcsubv_handle(pv) ((pv)->H)
#define ncl_dcsubv_offset(pv) ((pv)->O)
#define ncl_dcsubv_extent(pv) ((pv)->N)
#define ncl_dcsubv_stride(pv) ((pv)->S)
#define ncl_dcsubv_init(pv, h, o, n, s) \
do { \
  (pv)->H = (h); (pv)->O = (o); \
  (pv)->N = (n); (pv)->S = (s); (pv)->_ = ncl_other; \
  } while (0)
#define ncl_dcsubv_initclone(pv, pu) ncl_dcsubv_init(pv, \
ncl_dcsubv_handle(pu), ncl_dcsubv_offset(pu), \
ncl_dcsubv_extent(pu), ncl_dcsubv_stride(pu))
#define ncl_dcsubv_createclone(pu) ncl_dcsubv_create( \
ncl_dcsubv_handle(pu), ncl_dcsubv_offset(pu), \
ncl_dcsubv_extent(pu), ncl_dcsubv_stride(pu))
#define ncl_dcsubv_newclone(pu) ncl_dcsubv_new( \
ncl_dcsubv_handle(pu), ncl_dcsubv_offset(pu), \
ncl_dcsubv_extent(pu), ncl_dcsubv_stride(pu))
#define ncl_dcsubv_destroy(pv) ((pv)->_ = ~ncl_other)
#define ncl_dcsubv_delete(pv) \
do { ncl_dcsubv_destroy(pv); free((void*)(pv)); } while (0)
#define ncl_dcsubv_empty(pv) \
(ncl_dch_empty(&((pv)->H)) || 0 == ncl_dcsubv_extent(pv))
#define ncl_dcsubv_get(pv, j) ncl_dch_get(&((pv)->H), \
ncl_dcsubv_offset(pv) + (ncl_stride)(j)*ncl_dcsubv_stride(pv))
#define ncl_dcsubv_set(pv, j, x) ncl_dch_set(&((pv)->H), \
ncl_dcsubv_offset(pv) + (ncl_stride)(j)*ncl_dcsubv_stride(pv), (x))
#define ncl_dcv_destroy(pv) \
do { \
  (pv)->_ = ~ncl_owner; \
  free((void*)(ncl_dch_pointer(&ncl_dcsubv_handle(pv)))); \
  } while (0)
#define ncl_dcv_delete(pv) \
do { ncl_dcv_destroy(pv); free((void*)(pv)); } while (0)

#endif /* NCL_MACROS */

#endif /* _ncl_dcvector_h */
