#ifndef _ncl_dhandle_h
#define _ncl_dhandle_h 1
/*
The ANSI C Numerical Class Library.
Copyright (C) 2000 E. Robert Tisdale

This file is part of The ANSI C Numerical Class Library.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#ifdef NCL_INLINE
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif /* NCL_REVEAL */
#endif /* NCL_INLINE */

#ifdef NCL_MACROS
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif /* NCL_REVEAL */
#endif /* NCL_MACROS */

/* Include scalar type definitions.		*/
#include<ncl_dscalar.h>

#ifdef NCL_REVEAL
/*  Reveal type definitions after applications are thoroughly tested.	*/

typedef struct {
  /* private: */
  ncl_double*	P;
  } ncl_dhandle;

#else  /* NCL_REVEAL */
/* Conceal type definitions until applications are thoroughly tested.	*/

#define NCL_DHANDLE_SIZE NCL_HANDLE_SIZE
typedef int ncl_dhidden_handle[NCL_DHANDLE_SIZE/sizeof(int)];

typedef struct {
  /* private: */
  ncl_dhidden_handle	H;
  } ncl_dhandle;

#endif /* NCL_REVEAL */

#ifdef NCL_INLINE

inline static
ncl_double*		/* pointer to the underlying data array	*/
(ncl_dh_pointer)(ncl_dhandle* ph) { return ph->P; }

inline static
ncl_dhandle		/* automatic storage constructor	*/
(ncl_dh_create)(ncl_dscalar* p) {
  ncl_dhandle h; h.P = p; return h; }

inline static
void				/* automatic storage  destructor	*/
(ncl_dh_destroy)(ncl_dhandle* ph) {
  }

inline static
int
(ncl_dh_empty)(const ncl_dhandle* ph) {
  return ph->P == NULL; }

inline static
ncl_dscalar		/* h[o]					*/
(ncl_dh_get)(const ncl_dhandle* ph,
  ncl_offset o) {
  return ((ncl_dscalar*)(ph->P))[o]; }

inline static
void				/* h[o] <-- x				*/
(ncl_dh_set)(      ncl_dhandle* ph,
  ncl_offset o, ncl_dscalar x) {
  ((ncl_dscalar*)(ph->P))[o] = x; }

#else  /* NCL_INLINE */

ncl_double*		/* pointer to the underlying data array	*/
(ncl_dh_pointer)(ncl_dhandle* ph);

ncl_dhandle		/* automatic storage constructor	*/
(ncl_dh_create)(ncl_dscalar* p);

void				/* automatic storage  destructor	*/
(ncl_dh_destroy)(ncl_dhandle* ph);

int
(ncl_dh_empty)(const ncl_dhandle* ph);

ncl_dscalar		/* h[o]					*/
(ncl_dh_get)(const ncl_dhandle* ph,
  ncl_offset o);

void				/* h[o] <-- x				*/
(ncl_dh_set)(      ncl_dhandle* ph,
  ncl_offset o, ncl_dscalar x);

#endif /* NCL_INLINE */

#ifdef NCL_MACROS

#define ncl_dh_pointer(ph) ((ph)->P)
#define ncl_dh_create(ps) (*((ncl_dhandle*)(&(ps))))
/* Note: The pointer ncl_dscalar* ps must be an lvalue!		*/
#define ncl_dh_destroy(ph)
#define ncl_dh_empty(ph) ((ph)->P == NULL)
#define ncl_dh_get(ph, o) (((ncl_dscalar*)((ph)->P))[o])
#define ncl_dh_set(ph, o, x) (((ncl_dscalar*)((ph)->P))[o] = (x))

#endif /* NCL_MACROS */

#endif /* _ncl_dhandle_h */
