#ifndef _ncl_dmatrix_h
#define _ncl_dmatrix_h 1
/*
The ANSI C Numerical Class Library.
Copyright (C) 2000 E. Robert Tisdale

This file is part of The ANSI C Numerical Class Library.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#ifdef NCL_INLINE
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif /* NCL_REVEAL */
#endif /* NCL_INLINE */

#ifdef NCL_MACROS
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif /* NCL_REVEAL */
#endif /* NCL_MACROS */

/* Include double precision  handle type definitions.	*/
#include<ncl_dvector.h>

#ifdef NCL_REVEAL
/*  Reveal type definitions after applications are thoroughly tested.	*/

typedef struct {		/* submatrix class definition		*/
  /* private: */
  ncl_dhandle	H;
  ncl_offset	O;
  ncl_extent	N1;
  ncl_stride	S1;
  ncl_extent	N2;
  ncl_stride	S2;
  ncl_______	_;
  } ncl_dsubmatrix;
/* A submatrix does not own the array storage that it references.	*/
/* It does not allocate any array storage when it is constructed	*/
/* nor does it deallocate any array storage when it is destroyed.	*/

#else  /* NCL_REVEAL */
/* Conceal type definitions until applications are thoroughly tested.	*/

#define NCL_DSUBMATRIX_SIZE NCL_SUBMATRIX_SIZE
typedef int ncl_dhidden_submatrix[NCL_DSUBMATRIX_SIZE/sizeof(int)];

typedef struct {		/* submatrix class definition		*/
  /* private: */
  ncl_dhidden_submatrix	M;
  } ncl_dsubmatrix;
/* A submatrix does not own the array storage that it references.	*/
/* It does not allocate any array storage when it is constructed	*/
/* nor does it deallocate any array storage when it is destroyed.	*/

#endif /* NCL_REVEAL */

typedef ncl_dsubmatrix ncl_dmatrix;
/* A matrix owns the array storage that it references.			*/ 
/* It allocates array storage for itself when it is constructed		*/
/* and deallocates it's array storage when it is destroyed.		*/

#ifdef NCL_INLINE

inline static			/* define  submatrix operations		*/
ncl_dhandle
(ncl_dsubm_handle)(const ncl_dsubmatrix* pM) { return pM->H; }

inline static
ncl_offset
(ncl_dsubm_offset)(const ncl_dsubmatrix* pM) { return pM->O; }

inline static
ncl_extent
(ncl_dsubm_extent1)(const ncl_dsubmatrix* pM) { return pM->N1; }

inline static
ncl_stride
(ncl_dsubm_stride1)(const ncl_dsubmatrix* pM) { return pM->S1; }

inline static
ncl_extent
(ncl_dsubm_extent2)(const ncl_dsubmatrix* pM) { return pM->N2; }

inline static
ncl_stride
(ncl_dsubm_stride2)(const ncl_dsubmatrix* pM) { return pM->S2; }

inline static
void				/* initializer				*/
(ncl_dsubm_init)(ncl_dsubmatrix* pM,
    ncl_dhandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  pM->H  = h; pM->O  = o;
  pM->N1 = n; pM->S1 = s1;
  pM->N2 = m; pM->S2 = s2; pM->_ = ncl_other; }

inline static
void				/* initializer clone			*/
(ncl_dsubm_initclone)(ncl_dsubmatrix* pM,
		const ncl_dsubmatrix* pN) {
  ncl_dhandle	h  = ncl_dsubm_handle(pN);
  ncl_offset	o  = ncl_dsubm_offset(pN);
  ncl_extent	n  = ncl_dsubm_extent1(pN);
  ncl_stride	s1 = ncl_dsubm_stride1(pN);
  ncl_extent	m  = ncl_dsubm_extent2(pN);
  ncl_stride	s2 = ncl_dsubm_stride2(pN);
  ncl_dsubm_init(pM, h, o, m, s2, n, s1); }

inline static
ncl_dsubmatrix		/* automatic storage constructor	*/
(ncl_dsubm_create)(
    ncl_dhandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  ncl_dsubmatrix	M;
  ncl_dsubm_init(&M, h, o, m, s2, n, s1);
  return M; }

inline static
ncl_dsubmatrix		/* automatic storage clone constructor	*/
(ncl_dsubm_createclone)(const ncl_dsubmatrix* pN) {
  ncl_dsubmatrix	M = *pN; M._ = ncl_other;
  return M; }

inline static
void				/* automatic storage  destructor	*/
(ncl_dsubm_destroy)(ncl_dsubmatrix* pM) {
  pM->_ = ~ncl_other; }

inline static
ncl_dsubmatrix*		/*      free storage constructor	*/
(ncl_dsubm_new)(
    ncl_dhandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  ncl_dsubmatrix*	pM
    = (ncl_dsubmatrix*)malloc(sizeof(ncl_dsubmatrix));
  if (pM)
    ncl_dsubm_init(pM, h, o, m, s2, n, s1);
  return pM; }

inline static
ncl_dsubmatrix*		/*      free storage clone constructor	*/
(ncl_dsubm_newclone)(const ncl_dsubmatrix* pN) {
  ncl_dhandle	h  = ncl_dsubm_handle(pN);
  ncl_offset	o  = ncl_dsubm_offset(pN);
  ncl_extent	n  = ncl_dsubm_extent1(pN);
  ncl_stride	s1 = ncl_dsubm_stride1(pN);
  ncl_extent	m  = ncl_dsubm_extent2(pN);
  ncl_stride	s2 = ncl_dsubm_stride2(pN);
  return ncl_dsubm_new(h, o, m, s2, n, s1); }

inline static
void				/*      free storage  destructor	*/
(ncl_dsubm_delete)(ncl_dsubmatrix* pM) {
  ncl_dsubm_destroy(pM);
  free((void*)pM); }

inline static
ncl_dsubmatrix*		/*                 reconstructor	*/
(ncl_dsubm_resize)(ncl_dsubmatrix* pM,
    ncl_dhandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  ncl_dsubm_init(pM, h, o, m, s2, n, s1);
  return pM; }

inline static
ncl_dsubmatrix*		/*           clone reconstructor	*/
(ncl_dsubm_resizeclone)(ncl_dsubmatrix* pM,
		const ncl_dsubmatrix* pN) {
  ncl_dhandle	h  = ncl_dsubm_handle(pN);
  ncl_offset	o  = ncl_dsubm_offset(pN);
  ncl_extent	n  = ncl_dsubm_extent1(pN);
  ncl_stride	s1 = ncl_dsubm_stride1(pN);
  ncl_extent	m  = ncl_dsubm_extent2(pN);
  ncl_stride	s2 = ncl_dsubm_stride2(pN);
  return ncl_dsubm_resize(pM, h, o, m, s2, n, s1); }

inline static
int
(ncl_dsubm_empty)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h = ncl_dsubm_handle(pM);
  ncl_extent	n = ncl_dsubm_extent1(pM);
  ncl_extent	m = ncl_dsubm_extent2(pM);
  return ncl_dh_empty(&h) || 0 == n || 0 == m; }

inline static
ncl_dscalar		/* M[i][j]				*/
(ncl_dsubm_get)(const ncl_dsubmatrix* pM,
    ncl_offset i, ncl_offset j) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  return ncl_dh_get(&h, o + (ncl_stride)j*s1 + (ncl_stride)i*s2); }

inline static
void				/* M[i][j] = x				*/
(ncl_dsubm_set)(      ncl_dsubmatrix* pM,
    ncl_offset i, ncl_offset j, ncl_dscalar x) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  ncl_dh_set(&h, o + (ncl_stride)j*s1 + (ncl_stride)i*s2, x); }

inline static
ncl_dsubvector		/*  vector view	*/
(ncl_dsubm_subv1)(const ncl_dsubmatrix* pM, ncl_offset j) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  return ncl_dsubv_create(h, o + (ncl_stride)j*s1, m, s2); }

inline static
ncl_dsubvector		/*  vector view	*/
(ncl_dsubm_subv2)(const ncl_dsubmatrix* pM, ncl_offset i) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  return ncl_dsubv_create(h, o + (ncl_stride)i*s2, n, s1); }

inline static
ncl_dsubmatrix		/*  matrix view	*/
(ncl_dsubm_subm1)(const ncl_dsubmatrix* pM,
    ncl_offset j, ncl_extent n, ncl_stride s) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  return ncl_dsubm_create(h, o + (ncl_stride)j*s1, m, s2, n, s*s1); }
  
inline static
ncl_dsubmatrix		/*  matrix view	*/
(ncl_dsubm_subm2)(const ncl_dsubmatrix* pM,
    ncl_offset i, ncl_extent m, ncl_stride s) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  return ncl_dsubm_create(h, o + (ncl_stride)i*s2, m, s*s2, n, s1); }
  
inline static
ncl_dsubmatrix		/*  matrix view	*/
(ncl_dsubm_subm)(const ncl_dsubmatrix* pM,
    ncl_offset i, ncl_extent m, ncl_stride s2,
    ncl_offset j, ncl_extent n, ncl_stride s1) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_stride	s1_= ncl_dsubm_stride1(pM);
  ncl_stride	s2_= ncl_dsubm_stride2(pM);
  return ncl_dsubm_create(h, o + (ncl_stride)j*s1 + (ncl_stride)i*s2,
    m, s2*s2_, n, s1*s1_); }
  
inline static
ncl_dsubmatrix		/* reverse view	*/
(ncl_dsubm_r1)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  o += (0 < n)? (ncl_stride)(n-1)*s1: 0;
  return ncl_dsubm_create(h, o, m, +s2, n, -s1); }

inline static
ncl_dsubmatrix		/* reverse view	*/
(ncl_dsubm_r2)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  o += (0 < m)? (ncl_stride)(m-1)*s2: 0;
  return ncl_dsubm_create(h, o, m, -s2, n, +s1); }

inline static
ncl_dsubmatrix		/* reverse view	*/
(ncl_dsubm_r)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  o += (0 < n)? (ncl_stride)(n-1)*s1: 0;
  o += (0 < m)? (ncl_stride)(m-1)*s2: 0;
  return ncl_dsubm_create(h, o, m, -s2, n, -s1); }

inline static
ncl_dsubmatrix	/*       transpose view	*/
(ncl_dsubm_t)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  return ncl_dsubm_create(h, o, n, s1, m, s2); }

inline static
ncl_dsubvector	/*        diagonal view	*/
(ncl_dsubm_diag)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  return ncl_dsubv_create(h, o, ((m < n)? m: n), s1 + s2); }

inline static
ncl_dsubmatrix		/* columns 0, 2, ...,   2j, ...	*/
(ncl_dsubm_even)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  return ncl_dsubm_create(h, o, m, s2, (n+1) >> 1, s1 << 1); }

inline static
ncl_dsubmatrix		/* columns 1, 3, ..., 1+2j, ...	*/
(ncl_dsubm_odd)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  return ncl_dsubm_create(h, o, m, s2, n >> 1, s1 << 1); }

inline static
ncl_dsubmatrix*		/* M[i] <-> M[k]			*/
(ncl_dsubm_vswap)(ncl_dsubmatrix* pM, ncl_offset i, ncl_offset k) {
  ncl_dsubvector v = ncl_dsubm_subv2(pM, i);
  ncl_dsubvector u = ncl_dsubm_subv2(pM, k);
  ncl_dsubv_vswap(&v, &u);
  ncl_dsubv_destroy(&u);
  ncl_dsubv_destroy(&v);
  return pM; }

#else  /* NCL_INLINE */
				/* declare submatrix operations		*/
ncl_dhandle
(ncl_dsubm_handle)(const ncl_dsubmatrix* pM);

ncl_offset
(ncl_dsubm_offset)(const ncl_dsubmatrix* pM);

ncl_extent
(ncl_dsubm_extent1)(const ncl_dsubmatrix* pM);

ncl_stride
(ncl_dsubm_stride1)(const ncl_dsubmatrix* pM);

ncl_extent
(ncl_dsubm_extent2)(const ncl_dsubmatrix* pM);

ncl_stride
(ncl_dsubm_stride2)(const ncl_dsubmatrix* pM);

void				/* initializer				*/
(ncl_dsubm_init)(ncl_dsubmatrix* pM,
    ncl_dhandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1);

void				/* initializer				*/
(ncl_dsubm_initclone)(ncl_dsubmatrix* pM,
		const ncl_dsubmatrix* pN);

ncl_dsubmatrix		/* automatic storage constructor	*/
(ncl_dsubm_create)(
    ncl_dhandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1);

ncl_dsubmatrix		/* automatic storage clone constructor	*/
(ncl_dsubm_createclone)(const ncl_dsubmatrix* pN);

void				/* automatic storage  destructor	*/
(ncl_dsubm_destroy)(ncl_dsubmatrix* pM);

ncl_dsubmatrix*		/*      free storage constructor	*/
(ncl_dsubm_new)(
    ncl_dhandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1);

ncl_dsubmatrix*		/*      free storage clone constructor	*/
(ncl_dsubm_newclone)(const ncl_dsubmatrix* pN);

void				/*      free storage  destructor	*/
(ncl_dsubm_delete)(ncl_dsubmatrix* pM);

ncl_dsubmatrix*		/*                 reconstructor	*/
(ncl_dsubm_resize)(ncl_dsubmatrix* pM,
    ncl_dhandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1);

ncl_dsubmatrix*		/*           clone reconstructor	*/
(ncl_dsubm_resizeclone)(ncl_dsubmatrix* pM,
		const ncl_dsubmatrix* pN);

int
(ncl_dsubm_empty)(const ncl_dsubmatrix* pM);

ncl_dscalar		/* M[i][j]				*/
(ncl_dsubm_get)(const ncl_dsubmatrix* pM,
    ncl_offset i, ncl_offset j);

void				/* M[i][j] = x				*/
(ncl_dsubm_set)(      ncl_dsubmatrix* pM,
    ncl_offset i, ncl_offset j, ncl_dscalar x);

ncl_dsubvector		/*  vector view	*/
(ncl_dsubm_subv2)(const ncl_dsubmatrix* pM, ncl_offset i);

ncl_dsubvector		/*  vector view	*/
(ncl_dsubm_subv1)(const ncl_dsubmatrix* pM, ncl_offset j);

ncl_dsubmatrix		/*  matrix view	*/
(ncl_dsubm_subm1)(const ncl_dsubmatrix* pM,
    ncl_offset j, ncl_extent n, ncl_stride s);

ncl_dsubmatrix		/*  matrix view	*/
(ncl_dsubm_subm2)(const ncl_dsubmatrix* pM,
    ncl_offset i, ncl_extent m, ncl_stride s);

ncl_dsubmatrix		/*  matrix view	*/
(ncl_dsubm_subm)(const ncl_dsubmatrix* pM,
    ncl_offset i, ncl_extent m, ncl_stride s2,
    ncl_offset j, ncl_extent n, ncl_stride s1);

ncl_dsubmatrix		/* reverse view	*/
(ncl_dsubm_r1)(const ncl_dsubmatrix* pM);

ncl_dsubmatrix		/* reverse view	*/
(ncl_dsubm_r2)(const ncl_dsubmatrix* pM);

ncl_dsubmatrix		/* reverse view	*/
(ncl_dsubm_r)(const ncl_dsubmatrix* pM);

ncl_dsubmatrix	/*       transpose view	*/
(ncl_dsubm_t)(const ncl_dsubmatrix* pM);

ncl_dsubvector	/*        diagonal view	*/
(ncl_dsubm_diag)(const ncl_dsubmatrix* pM);

ncl_dsubmatrix		/* columns 0, 2, ...,   2j, ...		*/
(ncl_dsubm_even)(const ncl_dsubmatrix* pM);

ncl_dsubmatrix		/* columns 1, 3, ..., 1+2j, ...		*/
(ncl_dsubm_odd)(const ncl_dsubmatrix* pM);

ncl_dsubmatrix*		/* M[i] <-> M[k]			*/
(ncl_dsubm_vswap)(ncl_dsubmatrix* pM, ncl_offset i, ncl_offset k);

ncl_dsubmatrix*		/* M <-> N				*/
(ncl_dsubm_mswap)(ncl_dsubmatrix* pM, ncl_dsubmatrix* pN);

#endif /* NCL_INLINE */

ncl_dsubmatrix*
(ncl_dsubm_ramp)(ncl_dsubmatrix* pM);

ncl_dsubmatrix*
(ncl_dsubm_reverse)(ncl_dsubmatrix* pM);

					/* matrix-scalar operations	*/
ncl_dsubmatrix*			/* M  = x */
(ncl_dsubm_fill)(ncl_dsubmatrix* pM, ncl_dscalar x);

ncl_dsubmatrix*			/* M *= x */
(ncl_dsubm_smul)(ncl_dsubmatrix* pM, ncl_dscalar x);

ncl_dsubmatrix*			/* M /= x */
(ncl_dsubm_sdiv)(ncl_dsubmatrix* pM, ncl_dscalar x);

ncl_dsubmatrix*			/* M += x */
(ncl_dsubm_sadd)(ncl_dsubmatrix* pM, ncl_dscalar x);

ncl_dsubmatrix*			/* M -= x */
(ncl_dsubm_ssub)(ncl_dsubmatrix* pM, ncl_dscalar x);

					/* matrix-matrix operations	*/
ncl_dsubmatrix*			/* M  = N */
(ncl_dsubm_mcpy)(ncl_dsubmatrix* pM, const ncl_dsubmatrix* pN);

ncl_dsubmatrix*			/* M *= N */
(ncl_dsubm_mmul)(ncl_dsubmatrix* pM, const ncl_dsubmatrix* pN);

ncl_dsubmatrix*			/* M /= N */
(ncl_dsubm_mdiv)(ncl_dsubmatrix* pM, const ncl_dsubmatrix* pN);

ncl_dsubmatrix*			/* M += N */
(ncl_dsubm_madd)(ncl_dsubmatrix* pM, const ncl_dsubmatrix* pN);

ncl_dsubmatrix*			/* M -= N */
(ncl_dsubm_msub)(ncl_dsubmatrix* pM, const ncl_dsubmatrix* pN);

int
(ncl_dsubm_fprintf)(FILE* stream, const ncl_dsubmatrix* pM,
  int width, unsigned int precision, ncl_fmtflags flags, unsigned int columns);

#ifdef NCL_INLINE

inline static			/* define     matrix operations		*/
ncl_dmatrix		/* automatic storage constructor	*/
(ncl_dm_create)(ncl_extent m, ncl_extent n) {
  ncl_dsubmatrix	M;
  ncl_dscalar*	ps
    = (ncl_dscalar*)((0 < m*n)? malloc(m*n*sizeof(ncl_dscalar)): NULL);
  ncl_dsubm_init(&M, ncl_dh_create(ps),
    (ncl_offset)0, m, (ncl_stride)n, n, (ncl_stride)1); M._ = ncl_owner;
  return M; }

inline static
ncl_dmatrix		/* automatic storage clone constructor	*/
(ncl_dm_createclone)(const ncl_dsubmatrix* pN) {
  ncl_extent	n = ncl_dsubm_extent1(pN);
  ncl_extent	m = ncl_dsubm_extent2(pN);
  ncl_dmatrix	M = ncl_dm_create(m, n);
  ncl_dsubm_mcpy(&M, pN);
  return M; }

inline static
void				/* automatic storage  destructor	*/
(ncl_dm_destroy)(ncl_dmatrix* pM) {
  ncl_dhandle	h = ncl_dsubm_handle(pM);
  pM->_ = ~ncl_owner;
  free((void*)(ncl_dh_pointer(&h))); }

inline static
ncl_dmatrix*		/*      free storage constructor	*/
(ncl_dm_new)(ncl_extent m, ncl_extent n) {
  ncl_dsubmatrix*	pM
    = (ncl_dsubmatrix*)malloc(sizeof(ncl_dsubmatrix));
  if (pM) {
    ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < m*n)? malloc(m*n*sizeof(ncl_dscalar)): NULL);
    ncl_dsubm_init(pM, ncl_dh_create(ps),
    (ncl_offset)0, m, n, n, (ncl_stride)1); pM->_ = ncl_owner;
    }
  return pM; }

inline static
ncl_dmatrix*		/*      free storage clone constructor	*/
(ncl_dm_newclone)(const ncl_dsubmatrix* pN) {
  ncl_extent	n = ncl_dsubm_extent1(pN);
  ncl_extent	m = ncl_dsubm_extent2(pN);
  return ncl_dsubm_mcpy(ncl_dm_new(m, n), pN); }

inline static
void				/*      free storage  destructor	*/
(ncl_dm_delete)(ncl_dmatrix* pM) {
  ncl_dm_destroy(pM);
  free((void*)pM); }

inline static
ncl_dmatrix*		/*                 reconstructor	*/
(ncl_dm_resize)(ncl_dmatrix* pM, ncl_extent m, ncl_extent n) {
  ncl_dhandle	h = ncl_dsubm_handle(pM);
  if (!ncl_dh_empty(&h)) {
    free((void*)(ncl_dh_pointer(&h)));
    }
  { ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < m*n)? malloc(m*n*sizeof(ncl_dscalar)): NULL);
    ncl_dsubm_init(pM, ncl_dh_create(ps),
      (ncl_offset)0, m, n, n, (ncl_stride)1); pM->_ = ncl_owner;
    }
  return pM; }
  
inline static
ncl_dmatrix*		/*           clone reconstructor	*/
(ncl_dm_resizeclone)(ncl_dmatrix* pM,
		const ncl_dsubmatrix* pN) {
  ncl_extent	n = ncl_dsubm_extent1(pN);
  ncl_extent	m = ncl_dsubm_extent2(pN);
  return ncl_dsubm_mcpy(ncl_dm_resize(pM, m, n), pN); }

#else  /* NCL_INLINE */
				/* declare   matrix operations		*/
ncl_dmatrix		/* automatic storage constructor	*/
(ncl_dm_create)(ncl_extent m, ncl_extent n);

ncl_dmatrix		/* automatic storage clone constructor	*/
(ncl_dm_createclone)(const ncl_dsubmatrix* pN);

void				/* automatic storage  destructor	*/
(ncl_dm_destroy)(ncl_dmatrix* pM);

ncl_dmatrix*		/*      free storage constructor	*/
(ncl_dm_new)(ncl_extent m, ncl_extent n);

ncl_dmatrix*		/*      free storage clone constructor	*/
(ncl_dm_newclone)(const ncl_dsubmatrix* pN);

void				/*      free storage  destructor	*/
(ncl_dm_delete)(ncl_dmatrix* pM);

ncl_dmatrix*		/*                 reconstructor	*/
(ncl_dm_resize)(ncl_dmatrix* pM, ncl_extent m, ncl_extent n);

ncl_dmatrix*		/*           clone reconstructor	*/
(ncl_dm_resizeclone)(ncl_dmatrix* pM,
		const ncl_dsubmatrix* pN);

#endif /* NCL_INLINE */

#ifdef NCL_MACROS

#define ncl_dsubm_handle(pM)  ((pM)->H)
#define ncl_dsubm_offset(pM)  ((pM)->O)
#define ncl_dsubm_extent1(pM) ((pM)->N1)
#define ncl_dsubm_stride1(pM) ((pM)->S1)
#define ncl_dsubm_extent2(pM) ((pM)->N2)
#define ncl_dsubm_stride2(pM) ((pM)->S2)
#define ncl_dsubm_init(pM, h, o, n2, s2, n1, s1) \
do { \
  (pM)->H  = (h);  (pM)->O  = (o); \
  (pM)->N1 = (n1); (pM)->S1 = (s1); \
  (pM)->N2 = (n2); (pM)->S2 = (s2); (pM)->_ = ncl_other; \
  } while (0)
#define ncl_dsubm_initclone(pM, pN) ncl_dsubm_init(pM, \
ncl_dsubm_handle(pN), ncl_dsubm_offset(pN), \
ncl_dsubm_extent2(pN), ncl_dsubm_stride2(pN), \
ncl_dsubm_extent1(pN), ncl_dsubm_stride1(pN))
#define ncl_dsubm_createclone(pN) ncl_dsubm_create( \
ncl_dsubm_handle(pN), ncl_dsubm_offset(pN), \
ncl_dsubm_extent2(pN), ncl_dsubm_stride2(pN), \
ncl_dsubm_extent1(pN), ncl_dsubm_stride1(pN))
#define ncl_dsubm_newclone(pN) ncl_dsubm_new( \
ncl_dsubm_handle(pN), ncl_dsubm_offset(pN), \
ncl_dsubm_extent2(pN), ncl_dsubm_stride2(pN), \
ncl_dsubm_extent1(pN), ncl_dsubm_stride1(pN))
#define ncl_dsubm_destroy(pM) ((pM)->_ = ~ncl_other)
#define ncl_dsubm_delete(pM) \
do { ncl_dsubm_destroy(pM); free((void*)(pM)); } while (0)
#define ncl_dsubm_empty(pM) \
 (ncl_dh_empty(&((pM)->H)) || \
0 == ncl_dsubm_extent1(pM) || \
0 == ncl_dsubm_extent2(pM))
#define ncl_dsubm_get(pM, i, j) \
ncl_dh_get(&((pM)->H), ncl_dsubm_offset(pM) \
  + (ncl_stride)(j)*ncl_dsubm_stride1(pM) \
  + (ncl_stride)(i)*ncl_dsubm_stride2(pM))
#define ncl_dsubm_set(pM, i, j, x) \
ncl_dh_set(&((pM)->H), ncl_dsubm_offset(pM) \
  + (ncl_stride)(j)*ncl_dsubm_stride1(pM) \
  + (ncl_stride)(i)*ncl_dsubm_stride2(pM), (x))
#define ncl_dm_destroy(pM) \
do { \
  (pM)->_ = ~ncl_owner; \
  free((void*)(ncl_dh_pointer(&ncl_dsubm_handle(pM)))); \
  } while (0)
#define ncl_dm_delete(pM) \
do { ncl_dm_destroy(pM); free((void*)(pM)); } while (0)

#endif /* NCL_MACROS */

#endif /* _ncl_dmatrix_h */
