#ifndef _ncl_dvector_h
#define _ncl_dvector_h 1
/*
The ANSI C Numerical Class Library.
Copyright (C) 2000 E. Robert Tisdale

This file is part of The ANSI C Numerical Class Library.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#ifdef NCL_INLINE
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif /* NCL_REVEAL */
#endif /* NCL_INLINE */

#ifdef NCL_MACROS
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif /* NCL_REVEAL */
#endif /* NCL_MACROS */

/* Include double precision  handle type definitions.	*/
#include<ncl_dhandle.h>

#ifdef NCL_REVEAL
/*  Reveal type definitions after applications are thoroughly tested.	*/

typedef struct {		/* subvector class definition		*/
  /* private: */
  ncl_dhandle	H;
  ncl_offset	O;
  ncl_extent	N;
  ncl_stride	S;
  ncl_______	_;
  } ncl_dsubvector;
/* A subvector does not own the array storage that it references.	*/
/* It does not allocate any array storage when it is constructed	*/
/* nor does it deallocate any array storage when it is destroyed.	*/

#else  /* NCL_REVEAL */
/* Conceal type definitions until applications are thoroughly tested.	*/

#define NCL_DSUBVECTOR_SIZE NCL_SUBVECTOR_SIZE
typedef int ncl_dhidden_subvector[NCL_DSUBVECTOR_SIZE/sizeof(int)];

typedef struct {		/* subvector class definition		*/
  /* private: */
  ncl_dhidden_subvector	V;
  } ncl_dsubvector;
/* A subvector does not own the array storage that it references.	*/
/* It does not allocate any array storage when it is constructed	*/
/* nor does it deallocate any array storage when it is destroyed.	*/

#endif /* NCL_REVEAL */

typedef ncl_dsubvector ncl_dvector;
/* A vector owns the array storage that it references.			*/ 
/* It allocates array storage for itself when it is constructed		*/
/* and deallocates it's array storage when it is destroyed.		*/

#ifdef NCL_INLINE

inline static			/* define  subvector operations		*/
ncl_dhandle
(ncl_dsubv_handle)(const ncl_dsubvector* pv) { return pv->H; }

inline static
ncl_offset
(ncl_dsubv_offset)(const ncl_dsubvector* pv) { return pv->O; }

inline static
ncl_extent
(ncl_dsubv_extent)(const ncl_dsubvector* pv) { return pv->N; }

inline static
ncl_stride
(ncl_dsubv_stride)(const ncl_dsubvector* pv) { return pv->S; }

inline static
void				/* initializer				*/
(ncl_dsubv_init)(ncl_dsubvector* pv,
    ncl_dhandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  pv->H = h; pv->O = o; pv->N = n; pv->S = s; pv->_ = ncl_other; }

inline static
void				/* initializer clone			*/
(ncl_dsubv_initclone)(ncl_dsubvector* pv,
		const ncl_dsubvector* pu) {
  ncl_dhandle	h = ncl_dsubv_handle(pu);
  ncl_offset	o = ncl_dsubv_offset(pu);
  ncl_extent	n = ncl_dsubv_extent(pu);
  ncl_stride	s = ncl_dsubv_stride(pu);
  ncl_dsubv_init(pv, h, o, n, s); }

inline static
ncl_dsubvector		/* automatic storage constructor	*/
(ncl_dsubv_create)(
    ncl_dhandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  ncl_dsubvector	v;
  ncl_dsubv_init(&v, h, o, n, s);
  return v; }

inline static
ncl_dsubvector		/* automatic storage clone constructor	*/
(ncl_dsubv_createclone)(const ncl_dsubvector* pu) {
  ncl_dsubvector	v = *pu; v._ = ncl_other;
  return v; }

inline static
void				/* automatic storage  destructor	*/
(ncl_dsubv_destroy)(ncl_dsubvector* pv) {
  pv->_ = ~ncl_other; }

inline static
ncl_dsubvector*		/*      free storage constructor	*/
(ncl_dsubv_new)(
    ncl_dhandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  ncl_dsubvector*	pv
    = (ncl_dsubvector*)malloc(sizeof(ncl_dsubvector));
  if (pv)
    ncl_dsubv_init(pv, h, o, n, s);
  return pv; }

inline static
ncl_dsubvector*		/*      free storage clone constructor	*/
(ncl_dsubv_newclone)(const ncl_dsubvector* pu) {
  ncl_dhandle	h = ncl_dsubv_handle(pu);
  ncl_offset	o = ncl_dsubv_offset(pu);
  ncl_extent	n = ncl_dsubv_extent(pu);
  ncl_stride	s = ncl_dsubv_stride(pu);
  return ncl_dsubv_new(h, o, n, s); }

inline static
void				/*      free storage  destructor	*/
(ncl_dsubv_delete)(ncl_dsubvector* pv) {
  ncl_dsubv_destroy(pv);
  free((void*)pv); }

inline static
ncl_dsubvector*		/*                 reconstructor	*/
(ncl_dsubv_resize)(ncl_dsubvector* pv,
    ncl_dhandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  ncl_dsubv_init(pv, h, o, n, s);
  return pv; }

inline static
ncl_dsubvector*		/*           clone reconstructor	*/
(ncl_dsubv_resizeclone)(ncl_dsubvector* pv,
		const ncl_dsubvector* pu) {
  ncl_dhandle	h = ncl_dsubv_handle(pu);
  ncl_offset	o = ncl_dsubv_offset(pu);
  ncl_extent	n = ncl_dsubv_extent(pu);
  ncl_stride	s = ncl_dsubv_stride(pu);
  return ncl_dsubv_resize(pv, h, o, n, s); }

inline static
int
(ncl_dsubv_empty)(const ncl_dsubvector* pv) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_extent	n = ncl_dsubv_extent(pv);
  return ncl_dh_empty(&h) || 0 == n; }

inline static
ncl_dscalar
(ncl_dsubv_get)(const ncl_dsubvector* pv,
    ncl_offset j) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
  return ncl_dh_get(&h, o + (ncl_stride)j*s); }

inline static
void
(ncl_dsubv_set)(      ncl_dsubvector* pv,
    ncl_offset j, ncl_dscalar x) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
  ncl_dh_set(&h, o + (ncl_stride)j*s, x); }

inline static
ncl_dsubvector		/*  vector view	*/
(ncl_dsubv_subv)(const ncl_dsubvector* pv,
    ncl_offset j, ncl_extent n, ncl_stride s) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_stride	s_= ncl_dsubv_stride(pv);
  return ncl_dsubv_create(h, o + (ncl_stride)j*s, n, s*s_); }
  
inline static
ncl_dsubvector		/* reverse view	*/
(ncl_dsubv_r)(const ncl_dsubvector* pv) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
  o += (0 < n)? (ncl_stride)(n-1)*s: 0;
  return ncl_dsubv_create(h, o, n, -s); }

inline static
ncl_dsubvector		/* elements 0, 2, ...,   2j, ...	*/
(ncl_dsubv_even)(const ncl_dsubvector* pv) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
  return ncl_dsubv_create(h, o, (n+1) >> 1, s << 1); }

inline static
ncl_dsubvector		/* elements 1, 3, ..., 1+2j, ...	*/
(ncl_dsubv_odd)(const ncl_dsubvector* pv) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
  return ncl_dsubv_create(h, o, n >> 1, s << 1); }

inline static
ncl_dsubvector*		/* v[j] <-> v[k]			*/
(ncl_dsubv_sswap)(ncl_dsubvector* pv, ncl_offset j, ncl_offset k) {
  ncl_dscalar	t = ncl_dsubv_get(pv, j);
  ncl_dsubv_set(pv, j, ncl_dsubv_get(pv, k));
  ncl_dsubv_set(pv, k, t);
  return pv; }

#else  /* NCL_INLINE */
				/* declare subvector operations		*/
ncl_dhandle
(ncl_dsubv_handle)(const ncl_dsubvector* pv);

ncl_offset
(ncl_dsubv_offset)(const ncl_dsubvector* pv);

ncl_extent
(ncl_dsubv_extent)(const ncl_dsubvector* pv);

ncl_stride
(ncl_dsubv_stride)(const ncl_dsubvector* pv);

void				/* initializer				*/
(ncl_dsubv_init)(ncl_dsubvector* pv,
    ncl_dhandle h, ncl_offset o, ncl_extent n, ncl_stride s);

void				/* initializer				*/
(ncl_dsubv_initclone)(ncl_dsubvector* pv,
		const ncl_dsubvector* pu);

ncl_dsubvector		/* automatic storage constructor	*/
(ncl_dsubv_create)(
    ncl_dhandle h, ncl_offset o, ncl_extent n, ncl_stride s);

ncl_dsubvector		/* automatic storage clone constructor	*/
(ncl_dsubv_createclone)(const ncl_dsubvector* pu);

void				/* automatic storage  destructor	*/
(ncl_dsubv_destroy)(ncl_dsubvector* pv);

ncl_dsubvector*		/*      free storage constructor	*/
(ncl_dsubv_new)(
    ncl_dhandle h, ncl_offset o, ncl_extent n, ncl_stride s);

ncl_dsubvector*		/*      free storage clone constructor	*/
(ncl_dsubv_newclone)(const ncl_dsubvector* pu);

void				/*      free storage  destructor	*/
(ncl_dsubv_delete)(ncl_dsubvector* pv);

ncl_dsubvector*		/*                 reconstructor	*/
(ncl_dsubv_resize)(ncl_dsubvector* pv,
    ncl_dhandle h, ncl_offset o, ncl_extent n, ncl_stride s);

ncl_dsubvector*		/*           clone reconstructor	*/
(ncl_dsubv_resizeclone)(ncl_dsubvector* pv,
		const ncl_dsubvector* pu);

int
(ncl_dsubv_empty)(const ncl_dsubvector* pv);

ncl_dscalar
(ncl_dsubv_get)(const ncl_dsubvector* pv,
    ncl_offset j);

void
(ncl_dsubv_set)(      ncl_dsubvector* pv,
    ncl_offset j, ncl_dscalar x);

ncl_dsubvector		/*  vector view	*/
(ncl_dsubv_subv)(const ncl_dsubvector* pv,
    ncl_offset j, ncl_extent n, ncl_stride s);

ncl_dsubvector		/* reverse view	*/
(ncl_dsubv_r)(const ncl_dsubvector* pv);

ncl_dsubvector		/* elements 0, 2, ...,   2j, ...	*/
(ncl_dsubv_even)(const ncl_dsubvector* pv);

ncl_dsubvector		/* elements 1, 3, ..., 1+2j, ...	*/
(ncl_dsubv_odd)(const ncl_dsubvector* pv);

ncl_dsubvector*		/* v[j] <-> v[k]			*/
(ncl_dsubv_sswap)(ncl_dsubvector* pv, ncl_offset j, ncl_offset k);

#endif /* NCL_INLINE */

ncl_dsubvector*		/* v <-> u				*/
(ncl_dsubv_vswap)(ncl_dsubvector* pv, ncl_dsubvector* pu);

ncl_dsubvector*
(ncl_dsubv_ramp)(ncl_dsubvector* pv);

ncl_dsubvector*
(ncl_dsubv_reverse)(ncl_dsubvector* pv);

					/* vector-scalar operations	*/
ncl_dsubvector*			/* v  = x */
(ncl_dsubv_fill)(ncl_dsubvector* pv, ncl_dscalar x);

ncl_dsubvector*			/* v *= x */
(ncl_dsubv_smul)(ncl_dsubvector* pv, ncl_dscalar x);

ncl_dsubvector*			/* v /= x */
(ncl_dsubv_sdiv)(ncl_dsubvector* pv, ncl_dscalar x);

ncl_dsubvector*			/* v += x */
(ncl_dsubv_sadd)(ncl_dsubvector* pv, ncl_dscalar x);

ncl_dsubvector*			/* v -= x */
(ncl_dsubv_ssub)(ncl_dsubvector* pv, ncl_dscalar x);

					/* vector-vector operations	*/
ncl_dsubvector*			/* v  = w */
(ncl_dsubv_vcpy)(ncl_dsubvector* pv, const ncl_dsubvector* pw);

ncl_dsubvector*			/* v *= w */
(ncl_dsubv_vmul)(ncl_dsubvector* pv, const ncl_dsubvector* pw);

ncl_dsubvector*			/* v /= w */
(ncl_dsubv_vdiv)(ncl_dsubvector* pv, const ncl_dsubvector* pw);

ncl_dsubvector*			/* v += w */
(ncl_dsubv_vadd)(ncl_dsubvector* pv, const ncl_dsubvector* pw);

ncl_dsubvector*			/* v -= w */
(ncl_dsubv_vsub)(ncl_dsubvector* pv, const ncl_dsubvector* pw);

int
(ncl_dsubv_fprintf)(FILE* stream, const ncl_dsubvector* pv,
  int width, unsigned int precision, ncl_fmtflags flags, unsigned int columns);

#ifdef NCL_INLINE

inline static			/* define     vector operations		*/
ncl_dvector		/* automatic storage constructor	*/
(ncl_dv_create)(ncl_extent n) {
  ncl_dsubvector	v;
  ncl_dscalar*	ps
    = (ncl_dscalar*)((0 < n)? malloc(n*sizeof(ncl_dscalar)): NULL);
  ncl_dsubv_init(&v, ncl_dh_create(ps),
    (ncl_offset)0, n, (ncl_stride)1); v._ = ncl_owner;
  return v; }

inline static
ncl_dvector		/* automatic storage clone constructor	*/
(ncl_dv_createclone)(const ncl_dsubvector* pu) {
  ncl_dvector	v = ncl_dv_create(ncl_dsubv_extent(pu));
  ncl_dsubv_vcpy(&v, pu);
  return v; }

inline static
void				/* automatic storage  destructor	*/
(ncl_dv_destroy)(ncl_dvector* pv) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  pv->_ = ~ncl_owner;
  free((void*)(ncl_dh_pointer(&h)));
  }

inline static
ncl_dvector*		/*      free storage constructor	*/
(ncl_dv_new)(ncl_extent n) {
  ncl_dsubvector*	pv
    = (ncl_dsubvector*)malloc(sizeof(ncl_dsubvector));
  if (pv) {
    ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < n)? malloc(n*sizeof(ncl_dscalar)): NULL);
    ncl_dsubv_init(pv, ncl_dh_create(ps),
    (ncl_offset)0, n, (ncl_stride)1); pv->_ = ncl_owner;
    }
  return pv; }

inline static
ncl_dvector*		/*      free storage clone constructor	*/
(ncl_dv_newclone)(const ncl_dsubvector* pu) {
  return ncl_dsubv_vcpy(ncl_dv_new(ncl_dsubv_extent(pu)), pu); }

inline static
void				/*      free storage  destructor	*/
(ncl_dv_delete)(ncl_dvector* pv) {
  ncl_dv_destroy(pv);
  free((void*)pv); }

inline static
ncl_dvector*		/*                 reconstructor	*/
(ncl_dv_resize)(ncl_dvector* pv, ncl_extent n) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  if (!ncl_dh_empty(&h)) {
    free((void*)(ncl_dh_pointer(&h)));
    }
  { ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < n)? malloc(n*sizeof(ncl_dscalar)): NULL);
    ncl_dsubv_init(pv, ncl_dh_create(ps),
      (ncl_offset)0, n, (ncl_stride)1); pv->_ = ncl_owner;
    }
  return pv; }
  
inline static
ncl_dvector*		/*           clone reconstructor	*/
(ncl_dv_resizeclone)(
    ncl_dvector* pv, const ncl_dsubvector* pu) {
  return ncl_dsubv_vcpy(
    ncl_dv_resize(pv, ncl_dsubv_extent(pu)), pu); }

#else  /* NCL_INLINE */
				/* declare   vector operations		*/
ncl_dvector		/* automatic storage constructor	*/
(ncl_dv_create)(ncl_extent n);

ncl_dvector		/* automatic storage clone constructor	*/
(ncl_dv_createclone)(const ncl_dsubvector* pu);

void				/* automatic storage  destructor	*/
(ncl_dv_destroy)(ncl_dvector* pv);

ncl_dvector*		/*      free storage constructor	*/
(ncl_dv_new)(ncl_extent n);

ncl_dvector*		/*      free storage clone constructor	*/
(ncl_dv_newclone)(const ncl_dsubvector* pu);

void				/*      free storage  destructor	*/
(ncl_dv_delete)(ncl_dvector* pv);

ncl_dvector*		/*                 reconstructor	*/
(ncl_dv_resize)(ncl_dvector* pv, ncl_extent n);

ncl_dvector*		/*           clone reconstructor	*/
(ncl_dv_resizeclone)(ncl_dvector* pv,
		const ncl_dsubvector* pu);

#endif /* NCL_INLINE */

#ifdef NCL_MACROS

#define ncl_dsubv_handle(pv) ((pv)->H)
#define ncl_dsubv_offset(pv) ((pv)->O)
#define ncl_dsubv_extent(pv) ((pv)->N)
#define ncl_dsubv_stride(pv) ((pv)->S)
#define ncl_dsubv_init(pv, h, o, n, s) \
do { \
  (pv)->H = (h); (pv)->O = (o); \
  (pv)->N = (n); (pv)->S = (s); (pv)->_ = ncl_other; \
  } while (0)
#define ncl_dsubv_initclone(pv, pu) ncl_dsubv_init(pv, \
ncl_dsubv_handle(pu), ncl_dsubv_offset(pu), \
ncl_dsubv_extent(pu), ncl_dsubv_stride(pu))
#define ncl_dsubv_createclone(pu) ncl_dsubv_create( \
ncl_dsubv_handle(pu), ncl_dsubv_offset(pu), \
ncl_dsubv_extent(pu), ncl_dsubv_stride(pu))
#define ncl_dsubv_newclone(pu) ncl_dsubv_new( \
ncl_dsubv_handle(pu), ncl_dsubv_offset(pu), \
ncl_dsubv_extent(pu), ncl_dsubv_stride(pu))
#define ncl_dsubv_destroy(pv) ((pv)->_ = ~ncl_other)
#define ncl_dsubv_delete(pv) \
do { ncl_dsubv_destroy(pv); free((void*)(pv)); } while (0)
#define ncl_dsubv_empty(pv) \
(ncl_dh_empty(&((pv)->H)) || 0 == ncl_dsubv_extent(pv))
#define ncl_dsubv_get(pv, j) ncl_dh_get(&((pv)->H), \
ncl_dsubv_offset(pv) + (ncl_stride)(j)*ncl_dsubv_stride(pv))
#define ncl_dsubv_set(pv, j, x) ncl_dh_set(&((pv)->H), \
ncl_dsubv_offset(pv) + (ncl_stride)(j)*ncl_dsubv_stride(pv), (x))
#define ncl_dv_destroy(pv) \
do { \
  (pv)->_ = ~ncl_owner; \
  free((void*)(ncl_dh_pointer(&ncl_dsubv_handle(pv)))); \
  } while (0)
#define ncl_dv_delete(pv) \
do { ncl_dv_destroy(pv); free((void*)(pv)); } while (0)

#endif /* NCL_MACROS */

#endif /* _ncl_dvector_h */
