#ifndef _ncl_global_h
#define _ncl_global_h 1
/*
The ANSI C Numerical Class Library.
Copyright (C) 2000 E. Robert Tisdale

This file is part of The ANSI C Numerical Class Library.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#include<stdio.h>
#include<stdlib.h>

/* global type definitions	*/

typedef   signed int	ptrdiff_t;

typedef    size_t	ncl_offset;
typedef    size_t	ncl_extent;
typedef ptrdiff_t	ncl_stride;
typedef int		ncl_______;

typedef      float	ncl_single;
typedef      double	ncl_double;
typedef long double	ncl_extended;

/*     valid type patterns	*/
#define	ncl_owner (0x5555)
#define	ncl_other (0x3333)

#define NCL_HANDLE_SIZE		 4
#define NCL_SUBVECTOR_SIZE	20
#define NCL_SUBMATRIX_SIZE	28
#define NCL_SUBTENSOR_SIZE	36

void
ncl_message(const char* m);

#define ncl_message(m) fprintf(stderr, (m))

int
ncl_contains(ncl_extent m, ncl_offset o, ncl_extent n, ncl_stride s);

#define ncl_contains(m, o, n, s) ((o) < (m) && (0 == (n) || \
((s) < 0 && (-(s))*((n)-1) < (o)) || (0 <= (s) && (o) + (s)*((n)-1) < (m))))

typedef unsigned int ncl_fmtflags;

#define ncl_skipws	= (1 <<  0) /* skip whitespace on input		   */
				    /* field adjustment:		   */
#define ncl_left	= (1 <<  1) /* pad after value			   */
#define ncl_right	= (1 <<  2) /* pad before value			   */
#define ncl_internal	= (1 <<  3) /* pad between sign and value	   */

#define ncl_boolalpha	= (1 <<  4) /* represent true and false symbolically*/
				    /* integer base:			   */
#define ncl_dec	  	= (1 <<  5) /* base 10 output (decimal)		   */
#define ncl_hex	  	= (1 <<  6) /* base 16 output (hexadecimal)	   */
#define ncl_oct	  	= (1 <<  7) /* base  8 output (octal)		   */

#define ncl_scientific  = (1 <<  8) /* floating-point notation d.ddddddEdd */
#define ncl_fixed	= (1 <<  9) /* fixed-point    notation dddd.dd	   */

#define ncl_showbase	= (1 << 10) /* prefix oct by 0 and hex by 0x on output*/
#define ncl_showpoint	= (1 << 11) /* print trailing zeros		   */
#define ncl_showpos	= (1 << 12) /* explicit '+' for positive ints	   */
#define ncl_uppercase	= (1 << 13) /* 'E', 'X' rather than 'e', 'x'	   */
				    /* flags related to:		   */
#define ncl_adjustfield = (1 << 14) /* adjustment			   */
#define ncl_basefield	= (1 << 15) /* integer base			   */
#define ncl_floatfield	= (1 << 16) /* floating-point output		   */

#endif /* _ncl_global_h */
