/*
The ANSI C Numerical Class Library.
Copyright (C) 2000 E. Robert Tisdale

This file is part of The ANSI C Numerical Class Library.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#undef NCL_INLINE
#undef NCL_MACROS
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif  /* NCL_REVEAL */

#include<ncl_dcmatrix.h>

/* Note: The pointer ncl_dscalar* ps must be an lvalue!		*/
#define ncl_dch_pointer(ph) ((ph)->P)
#define ncl_dch_create(ps) (*((ncl_dchandle*)(&(ps))))
#define ncl_dch_destroy(ph)
#define ncl_dch_empty(ph) ((ph)->P == NULL)
#define ncl_dch_get(ph, o) (((ncl_dcscalar*)((ph)->P))[o])
#define ncl_dch_set(ph, o, x) (((ncl_dcscalar*)((ph)->P))[o] = (x))

/* Both ncl_dch_get and ncl_dch_set assume that			*/
/* sizeof(ncl_dcscalar) == 2*sizeof(ncl_dscalar))			*/

#define ncl_dcsubv_handle(pv) ((pv)->H);
#define ncl_dcsubv_offset(pv) ((pv)->O);
#define ncl_dcsubv_extent(pv) ((pv)->N);
#define ncl_dcsubv_stride(pv) ((pv)->S);
#define ncl_dcsubv_init(pv, h, o, n, s) \
do { \
  (pv)->H = (h); (pv)->O = (o); \
  (pv)->N = (n); (pv)->S = (s); (pv)->_ = ncl_other; \
  } while (0)
#define ncl_dcsubv_initclone(pv, pu) ncl_dcsubv_init(pv, \
ncl_dcsubv_handle(pu), ncl_dcsubv_offset(pu), \
ncl_dcsubv_extent(pu), ncl_dcsubv_stride(pu))
#define ncl_dcsubv_destroy(pv) ((pv)->_ = ~ncl_other)
#define ncl_dcsubv_empty(pv) \
(ncl_dch_empty(&((pv)->H)) || 0 == ncl_dcsubv_extent(pv))
#define ncl_dcsubv_get(pv, j) ncl_dch_get(&((pv)->H), \
ncl_dcsubv_offset(pv) + (ncl_stride)(j)*ncl_dcsubv_stride(pv))
#define ncl_dcsubv_set(pv, j, x) ncl_dch_set(&((pv)->H), \
ncl_dcsubv_offset(pv) + (ncl_stride)(j)*ncl_dcsubv_stride(pv), (x))
#define ncl_dcv_destroy(pv) \
do { \
  (pv)->_ = ~ncl_owner; \
  free((void*)(ncl_dch_pointer(&ncl_dcsubv_handle(pv)))); \
  } while (0)
#define ncl_dcv_delete(pv) \
do { ncl_dcv_destroy(pv); free((void*)(pv)); } while (0)

				/* define  submatrix operations		*/
ncl_dchandle
(ncl_dcsubm_handle)(const ncl_dcsubmatrix* pM) { return pM->H; }

#define ncl_dcsubm_handle(pM) ((pM)->H)

ncl_offset
(ncl_dcsubm_offset)(const ncl_dcsubmatrix* pM) { return pM->O; }

#define ncl_dcsubm_offset(pM) ((pM)->O)

ncl_extent
(ncl_dcsubm_extent1)(const ncl_dcsubmatrix* pM) { return pM->N1; }

#define ncl_dcsubm_extent1(pM) ((pM)->N1)

ncl_stride
(ncl_dcsubm_stride1)(const ncl_dcsubmatrix* pM) { return pM->S1; }

#define ncl_dcsubm_stride1(pM) ((pM)->S1)

ncl_extent
(ncl_dcsubm_extent2)(const ncl_dcsubmatrix* pM) { return pM->N2; }

#define ncl_dcsubm_extent2(pM) ((pM)->N2)

ncl_stride
(ncl_dcsubm_stride2)(const ncl_dcsubmatrix* pM) { return pM->S2; }

#define ncl_dcsubm_stride2(pM) ((pM)->S2)

void				/* initializer				*/
(ncl_dcsubm_init)(ncl_dcsubmatrix* pM,
    ncl_dchandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  pM->H  = h; pM->O  = o;
  pM->N1 = n; pM->S1 = s1;
  pM->N2 = m; pM->S2 = s2; pM->_ = ncl_other; }

#define ncl_dcsubm_init(pM, h, o, m, s2, n, s1) \
do { \
  (pM)->H  = (h); (pM)->O  = (o); \
  (pM)->N1 = (n); (pM)->S1 = (s1); \
  (pM)->N2 = (m); (pM)->S2 = (s2); (pM)->_ = ncl_other; \
  } while (0)

void				/* initializer clone			*/
(ncl_dcsubm_initclone)(ncl_dcsubmatrix* pM,
		const ncl_dcsubmatrix* pN) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pN);
  ncl_offset	o  = ncl_dcsubm_offset(pN);
  ncl_extent	n  = ncl_dcsubm_extent1(pN);
  ncl_stride	s1 = ncl_dcsubm_stride1(pN);
  ncl_extent	m  = ncl_dcsubm_extent2(pN);
  ncl_stride	s2 = ncl_dcsubm_stride2(pN);
  ncl_dcsubm_init(pM, h, o, m, s2, n, s1); }

#define ncl_dcsubm_initclone(pM, pN) ncl_dcsubm_init( \
ncl_dcsubm_handle(pN), ncl_dcsubm_offset(pN), \
ncl_dcsubm_extent2(pN), ncl_dcsubm_stride2(pN), \
ncl_dcsubm_extent1(pN), ncl_dcsubm_stride1(pN))

ncl_dcsubmatrix		/* automatic storage constructor	*/
(ncl_dcsubm_create)(
    ncl_dchandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  ncl_dcsubmatrix	M;
  ncl_dcsubm_init(&M, h, o, m, s2, n, s1);
  return M; }

ncl_dcsubmatrix		/* automatic storage clone constructor	*/
(ncl_dcsubm_createclone)(const ncl_dcsubmatrix* pN) {
  ncl_dcsubmatrix	M = *pN; M._ = ncl_other;
  return M; }

void				/* automatic storage  destructor	*/
(ncl_dcsubm_destroy)(ncl_dcsubmatrix* pM) {
#ifdef NCL_ERRANT
  if (ncl_other != pM->_)
    ncl_message("In function ncl_dcsubm_destroy(ncl_dcsubmatrix*):\n"
      "  argument 1 is not a valid ncl_dcsubmatrix*\n");
#endif /* NCL_ERRANT */
  pM->_ = ~ncl_other; }

#define ncl_dcsubm_destroy(pM) (pM->_ = ~ncl_other)

ncl_dcsubmatrix*		/*      free storage constructor	*/
(ncl_dcsubm_new)(
    ncl_dchandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  ncl_dcsubmatrix*	pM
    = (ncl_dcsubmatrix*)malloc(sizeof(ncl_dcsubmatrix));
  if (pM)
    ncl_dcsubm_init(pM, h, o, m, s2, n, s1);
#ifdef NCL_ERRANT
  else
    ncl_message(
     "In function ncl_dcsubm_new(ncl_dchandle, ncl_offset,\n"
   "  ncl_extent, ncl_stride, ncl_extent, ncl_stride):\n"
   "  memory allocation failed.\n");
#endif /* NCL_ERRANT */
  return pM; }

ncl_dcsubmatrix*		/*      free storage clone constructor	*/
(ncl_dcsubm_newclone)(const ncl_dcsubmatrix* pN) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pN);
  ncl_offset	o  = ncl_dcsubm_offset(pN);
  ncl_extent	n  = ncl_dcsubm_extent1(pN);
  ncl_stride	s1 = ncl_dcsubm_stride1(pN);
  ncl_extent	m  = ncl_dcsubm_extent2(pN);
  ncl_stride	s2 = ncl_dcsubm_stride2(pN);
  return ncl_dcsubm_new(h, o, m, s2, n, s1); }

void				/*      free storage  destructor	*/
(ncl_dcsubm_delete)(ncl_dcsubmatrix* pM) {
#ifdef NCL_ERRANT
  if (ncl_other != pM->_)
    ncl_message("In function ncl_dcsubm_delete(ncl_dcsubmatrix*):\n"
      "  argument 1 is not a valid ncl_dcsubmatrix*\n");
#endif /* NCL_ERRANT */
  ncl_dcsubm_destroy(pM);
  free((void*)pM); }

ncl_dcsubmatrix*		/*                 reconstructor	*/
(ncl_dcsubm_resize)(ncl_dcsubmatrix* pM,
    ncl_dchandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
#ifdef NCL_ERRANT
  if (ncl_other != pM->_)
    ncl_message(
  "In function ncl_dcsubm_resize(ncl_dcsubmatrix*, ncl_dchandle,\n"
  "  ncl_offset, ncl_extent, ncl_stride, ncl_extent, ncl_stride):\n"
  "  argument 1 is not a valid ncl_dcsubmatrix*\n");
#endif /* NCL_ERRANT */
  ncl_dcsubm_init(pM, h, o, m, s2, n, s1);
  return pM; }

ncl_dcsubmatrix*		/*           clone reconstructor	*/
(ncl_dcsubm_resizeclone)(ncl_dcsubmatrix* pM,
		const ncl_dcsubmatrix* pN) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pN);
  ncl_offset	o  = ncl_dcsubm_offset(pN);
  ncl_extent	n  = ncl_dcsubm_extent1(pN);
  ncl_stride	s1 = ncl_dcsubm_stride1(pN);
  ncl_extent	m  = ncl_dcsubm_extent2(pN);
  ncl_stride	s2 = ncl_dcsubm_stride2(pN);
  return ncl_dcsubm_resize(pM, h, o, m, s2, n, s1); }

int
(ncl_dcsubm_empty)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h = ncl_dcsubm_handle(pM);
  ncl_extent	n = ncl_dcsubm_extent1(pM);
  ncl_extent	m = ncl_dcsubm_extent2(pM);
  return ncl_dch_empty(&h) || 0 == n || 0 == m; }

ncl_dcscalar
(ncl_dcsubm_get)(const ncl_dcsubmatrix* pM,
    ncl_offset i, ncl_offset j) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  if (m <= i)
    ncl_message(
      "In function ncl_dcsubm_get(const ncl_dcsubmatrix*,\n"
      "  ncl_offset, ncl_offset): 1st offset exceeds extent.\n");
  if (n <= j)
    ncl_message(
      "In function ncl_dcsubm_get(const ncl_dcsubmatrix*,\n"
      "  ncl_offset, ncl_offset): 2nd offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  return ncl_dch_get(&h, o + (ncl_stride)j*s1 + (ncl_stride)i*s2); }

#define ncl_dcsubm_get_(pM, i, j) \
ncl_dch_get(&((pM)->H), ncl_dcsubm_offset(pM) \
+ (ncl_stride)(j)*ncl_dcsubm_stride1(pM) \
+ (ncl_stride)(i)*ncl_dcsubm_stride2(pM))

void
(ncl_dcsubm_set)(      ncl_dcsubmatrix* pM,
    ncl_offset i, ncl_offset j, ncl_dcscalar x) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  if (m <= i)
    ncl_message(
      "In function ncl_dcsubm_set(ncl_dcsubmatrix*, ncl_offset,\n"
      "  ncl_offset, ncl_dcscalar): 1st offset exceeds extent.\n");
  if (n <= j)
    ncl_message(
      "In function ncl_dcsubm_set(ncl_dcsubmatrix*, ncl_offset,\n"
      "  ncl_offset, ncl_dcscalar): 2nd offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  ncl_dch_set(&h, o + (ncl_stride)j*s1 + (ncl_stride)i*s2, x); }

#define ncl_dcsubm_set_(pM, i, j, x) \
ncl_dch_set(&((pM)->H), ncl_dcsubm_offset(pM) \
+ (ncl_stride)(j)*ncl_dcsubm_stride1(pM) \
+ (ncl_stride)(i)*ncl_dcsubm_stride2(pM), (x))

ncl_dcsubvector		/*  vector view	*/
(ncl_dcsubm_subv1)(const ncl_dcsubmatrix* pM, ncl_offset j) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  if (n <= j)
    ncl_message(
  "In function ncl_dcsubm_subv1(const ncl_dcsubmatrix*, ncl_offset):\n"
  "  offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  return ncl_dcsubv_create(h, o + (ncl_stride)j*s1, m, s2);
  }

ncl_dcsubvector		/*  vector view	*/
(ncl_dcsubm_subv2)(const ncl_dcsubmatrix* pM, ncl_offset i) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  if (m <= i)
    ncl_message(
    "In function ncl_dcsubm_subv2(const ncl_dcsubmatrix*, ncl_offset):\n"
    "  offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  return ncl_dcsubv_create(h, o + (ncl_stride)i*s2, n, s1);
  }

ncl_dcsubmatrix		/*  matrix view	*/
(ncl_dcsubm_subm1)(const ncl_dcsubmatrix* pM,
    ncl_offset j, ncl_extent n, ncl_stride s) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	n_ = ncl_dcsubm_extent1(pM);
  if (!ncl_contains(n_, j, n, s))
    ncl_message("In function ncl_dcsubm_subm1(const ncl_dcsubmatrix*,\n"
   "  ncl_offset, ncl_extent, ncl_stride): matrix cannot contain submatrix.\n");
#endif /* NCL_ERRANT */
  return ncl_dcsubm_create(h, o + (ncl_stride)j*s1, m, s2, n, s*s1); }

ncl_dcsubmatrix		/*  matrix view	*/
(ncl_dcsubm_subm2)(const ncl_dcsubmatrix* pM,
    ncl_offset i, ncl_extent m, ncl_stride s) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	m_ = ncl_dcsubm_extent2(pM);
  if (!ncl_contains(m_, i, m, s))
    ncl_message("In function ncl_dcsubm_subm2(const ncl_dcsubmatrix*,\n"
   "  ncl_offset, ncl_extent, ncl_stride): matrix cannot contain submatrix.\n");
#endif /* NCL_ERRANT */
  return ncl_dcsubm_create(h, o + (ncl_stride)i*s2, m, s*s2, n, s1); }

ncl_dcsubmatrix		/*  matrix view	*/
(ncl_dcsubm_subm)(const ncl_dcsubmatrix* pM,
    ncl_offset i, ncl_extent m, ncl_stride s2,
    ncl_offset j, ncl_extent n, ncl_stride s1) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_stride	s1_= ncl_dcsubm_stride1(pM);
  ncl_stride	s2_= ncl_dcsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	n_= ncl_dcsubm_extent1(pM);
  ncl_extent	m_= ncl_dcsubm_extent2(pM);
  if (!(ncl_contains(n_, j, n, s1) && ncl_contains(m_, i, m, s2)))
    ncl_message("In function ncl_dcsubm_subm(const ncl_dcsubmatrix*,\n"
"  ncl_offset, ncl_extent, ncl_stride, ncl_offset, ncl_extent, ncl_stride):\n"
"  matrix cannot contain submatrix.\n");
#endif /* NCL_ERRANT */
  return ncl_dcsubm_create(h,
    o + (ncl_stride)j*s1_ + (ncl_stride)i*s2_, m, s2*s2_, n, s1*s1_); }

ncl_dcsubmatrix		/* reverse view	*/
(ncl_dcsubm_r1)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  o += (0 < n)? (ncl_stride)(n-1)*s1: 0;
  return ncl_dcsubm_create(h, o, m, +s2, n, -s1); }

ncl_dcsubmatrix		/* reverse view	*/
(ncl_dcsubm_r2)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  o += (0 < m)? (ncl_stride)(m-1)*s2: 0;
  return ncl_dcsubm_create(h, o, m, -s2, n, +s1); }

ncl_dcsubmatrix		/* reverse view	*/
(ncl_dcsubm_r)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  o += (0 < n)? (ncl_stride)(n-1)*s1: 0;
  o += (0 < m)? (ncl_stride)(m-1)*s2: 0;
  return ncl_dcsubm_create(h, o, m, -s2, n, -s1); }

ncl_dcsubmatrix	/*       transpose view	*/
(ncl_dcsubm_t)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  return ncl_dcsubm_create(h, o, n, s1, m, s2); }

ncl_dcsubvector	/*        diagonal view	*/
(ncl_dcsubm_diag)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  return ncl_dcsubv_create(h, o, ((m < n)? m: n), s1 + s2); }

ncl_dcsubmatrix		/* columns 0, 2, ...,   2j, ...		*/
(ncl_dcsubm_even)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  return ncl_dcsubm_create(h, o, m, s2, (n+1) >> 1, s1 << 1); }

ncl_dcsubmatrix		/* columns 1, 3, ..., 1+2j, ...		*/
(ncl_dcsubm_odd)(const ncl_dcsubmatrix* pM) {
  ncl_dchandle	h  = ncl_dcsubm_handle(pM);
  ncl_offset	o  = ncl_dcsubm_offset(pM);
  ncl_extent	n  = ncl_dcsubm_extent1(pM);
  ncl_stride	s1 = ncl_dcsubm_stride1(pM);
  ncl_extent	m  = ncl_dcsubm_extent2(pM);
  ncl_stride	s2 = ncl_dcsubm_stride2(pM);
  return ncl_dcsubm_create(h, o, m, s2, n >> 1, s1 << 1); }

ncl_dcsubmatrix*		/* M[i] <-> M[k]			*/
(ncl_dcsubm_vswap)(ncl_dcsubmatrix* pM, ncl_offset i, ncl_offset k) {
#ifdef NCL_ERRANT
  ncl_extent	m = ncl_dcsubm_extent2(pM);
  if (m <= i)
    ncl_message(
      "In function ncl_dcsubm_swap(ncl_dcsubmatrix*,\n"
      "  ncl_offset, ncl_offset): 1st offset exceeds extent.\n");
  if (m <= k)
    ncl_message(
      "In function ncl_dcsubm_swap(ncl_dcsubmatrix*,\n"
      "  ncl_offset, ncl_offset): 2nd offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  { ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    ncl_dcsubvector	u = ncl_dcsubm_subv2(pM, k);
    ncl_dcsubv_vswap(&v, &u);
    ncl_dcsubv_destroy(&u);
    ncl_dcsubv_destroy(&v);
    }
  return pM; }

ncl_dcsubmatrix*		/* M <-> N				*/
(ncl_dcsubm_mswap)(ncl_dcsubmatrix* pM, ncl_dcsubmatrix* pN) {
  ncl_extent	mM = ncl_dcsubm_extent2(pM);
  ncl_offset	i  = 0;
#ifdef NCL_ERRANT
  ncl_extent	mN = ncl_dcsubm_extent2(pN);
  if (mM != mN) {
    ncl_message("In function ncl_dcsubm_mswap(ncl_dcsubmatrix*,\n"
      "  ncl_dcsubmatrix*): submatrices differ in 2nd extent.\n");
    mM = (mM < mN)? mM: mN;
    }
  { ncl_extent	nM = ncl_dcsubm_extent1(pM);
    ncl_extent	nN = ncl_dcsubm_extent1(pN);
    if (nM != nN) {
      ncl_message("In function ncl_dcsubm_mswap(ncl_dcsubmatrix*,\n"
        "  ncl_dcsubmatrix*): submatrices differ in 1st extent.\n");
      }
    }
#endif /* NCL_ERRANT */
  for (i = 0; i < mM; ++i) {
    ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    ncl_dcsubvector	u = ncl_dcsubm_subv2(pN, i);
    ncl_dcsubv_vswap(&v, &u);
    ncl_dcsubv_destroy(&u);
    ncl_dcsubv_destroy(&v);
    }
  return pM; }

ncl_dcsubmatrix*
(ncl_dcsubm_reverse)(ncl_dcsubmatrix* pM) {
  ncl_extent	m = ncl_dcsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    ncl_dcsubv_reverse(&v);
    ncl_dcsubv_destroy(&v);
    }
  return pM; }
					/* matrix-scalar operations	*/
ncl_dcsubmatrix*			/* M  = x */
(ncl_dcsubm_fill)(ncl_dcsubmatrix* pM, ncl_dcscalar x) {
  ncl_extent	m = ncl_dcsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    ncl_dcsubv_fill(&v, x);
    ncl_dcsubv_destroy(&v);
    }
  return pM; }

ncl_dcsubmatrix*			/* M *= x */
(ncl_dcsubm_smul)(ncl_dcsubmatrix* pM, ncl_dcscalar x) {
  ncl_extent	m = ncl_dcsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    ncl_dcsubv_smul(&v, x);
    ncl_dcsubv_destroy(&v);
    }
  return pM; }

ncl_dcsubmatrix*			/* M *= x */
(ncl_dcsubm_sdiv)(ncl_dcsubmatrix* pM, ncl_dcscalar x) {
  ncl_extent	m = ncl_dcsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    ncl_dcsubv_sdiv(&v, x);
    ncl_dcsubv_destroy(&v);
    }
  return pM; }

ncl_dcsubmatrix*			/* M *= x */
(ncl_dcsubm_sadd)(ncl_dcsubmatrix* pM, ncl_dcscalar x) {
  ncl_extent	m = ncl_dcsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    ncl_dcsubv_sadd(&v, x);
    ncl_dcsubv_destroy(&v);
    }
  return pM; }

ncl_dcsubmatrix*			/* M *= x */
(ncl_dcsubm_ssub)(ncl_dcsubmatrix* pM, ncl_dcscalar x) {
  ncl_extent	m = ncl_dcsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    ncl_dcsubv_ssub(&v, x);
    ncl_dcsubv_destroy(&v);
    }
  return pM; }
					/* matrix-matrix operations	*/
ncl_dcsubmatrix*			/* M  = N */
(ncl_dcsubm_mcpy)(ncl_dcsubmatrix* pM, const ncl_dcsubmatrix* pN) {
  ncl_extent	mM = ncl_dcsubm_extent2(pM);
  ncl_offset	i = 0;
#ifdef NCL_ERRANT
  ncl_extent	mN = ncl_dcsubm_extent2(pN);
  if (mM != mN) {
    ncl_message("In function ncl_dcsubm_mcpy(ncl_dcsubmatrix*,\n"
      "  const ncl_dcsubmatrix*): submatrix differ in second extent.\n");
    mM = (mM < mN)? mM: mN;
    }
#endif /* NCL_ERRANT */
  for (i = 0; i < mM; ++i) {
    ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    ncl_dcsubvector	u = ncl_dcsubm_subv2(pN, i);
    ncl_dcsubv_vcpy(&v, &u);
    ncl_dcsubv_destroy(&u);
    ncl_dcsubv_destroy(&v);
    }
  return pM; }

ncl_dcsubmatrix*			/* M *= N */
(ncl_dcsubm_mmul)(ncl_dcsubmatrix* pM, const ncl_dcsubmatrix* pN) {
  ncl_extent	mM = ncl_dcsubm_extent2(pM);
  ncl_offset	i = 0;
#ifdef NCL_ERRANT
  ncl_extent	mN = ncl_dcsubm_extent2(pN);
  if (mM != mN) {
    ncl_message("In function ncl_dcsubm_mmul(ncl_dcsubmatrix*,\n"
      "  const ncl_dcsubmatrix*): submatrix differ in second extent.\n");
    mM = (mM < mN)? mM: mN;
    }
#endif /* NCL_ERRANT */
  for (i = 0; i < mM; ++i) {
    ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    ncl_dcsubvector	u = ncl_dcsubm_subv2(pN, i);
    ncl_dcsubv_vmul(&v, &u);
    ncl_dcsubv_destroy(&u);
    ncl_dcsubv_destroy(&v);
    }
  return pM; }

ncl_dcsubmatrix*			/* M /= N */
(ncl_dcsubm_mdiv)(ncl_dcsubmatrix* pM, const ncl_dcsubmatrix* pN) {
  ncl_extent	mM = ncl_dcsubm_extent2(pM);
  ncl_offset	i = 0;
#ifdef NCL_ERRANT
  ncl_extent	mN = ncl_dcsubm_extent2(pN);
  if (mM != mN) {
    ncl_message("In function ncl_dcsubm_mdiv(ncl_dcsubmatrix*,\n"
      "  const ncl_dcsubmatrix*): submatrix differ in second extent.\n");
    mM = (mM < mN)? mM: mN;
    }
#endif /* NCL_ERRANT */
  for (i = 0; i < mM; ++i) {
    ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    ncl_dcsubvector	u = ncl_dcsubm_subv2(pN, i);
    ncl_dcsubv_vdiv(&v, &u);
    ncl_dcsubv_destroy(&u);
    ncl_dcsubv_destroy(&v);
    }
  return pM; }

ncl_dcsubmatrix*			/* M += N */
(ncl_dcsubm_madd)(ncl_dcsubmatrix* pM, const ncl_dcsubmatrix* pN) {
  ncl_extent	mM = ncl_dcsubm_extent2(pM);
  ncl_offset	i = 0;
#ifdef NCL_ERRANT
  ncl_extent	mN = ncl_dcsubm_extent2(pN);
  if (mM != mN) {
    ncl_message("In function ncl_dcsubm_madd(ncl_dcsubmatrix*,\n"
      "  const ncl_dcsubmatrix*): submatrix differ in second extent.\n");
    mM = (mM < mN)? mM: mN;
    }
#endif /* NCL_ERRANT */
  for (i = 0; i < mM; ++i) {
    ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    ncl_dcsubvector	u = ncl_dcsubm_subv2(pN, i);
    ncl_dcsubv_vadd(&v, &u);
    ncl_dcsubv_destroy(&u);
    ncl_dcsubv_destroy(&v);
    }
  return pM; }

ncl_dcsubmatrix*			/* M -= N */
(ncl_dcsubm_msub)(ncl_dcsubmatrix* pM, const ncl_dcsubmatrix* pN) {
  ncl_extent	mM = ncl_dcsubm_extent2(pM);
  ncl_offset	i = 0;
#ifdef NCL_ERRANT
  ncl_extent	mN = ncl_dcsubm_extent2(pN);
  if (mM != mN) {
    ncl_message("In function ncl_dcsubm_msub(ncl_dcsubmatrix*,\n"
      "  const ncl_dcsubmatrix*): submatrix differ in second extent.\n");
    mM = (mM < mN)? mM: mN;
    }
#endif /* NCL_ERRANT */
  for (i = 0; i < mM; ++i) {
    ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    ncl_dcsubvector	u = ncl_dcsubm_subv2(pN, i);
    ncl_dcsubv_vsub(&v, &u);
    ncl_dcsubv_destroy(&u);
    ncl_dcsubv_destroy(&v);
    }
  return pM; }
				/* define     matrix operations		*/
ncl_dcmatrix		/* automatic storage constructor	*/
(ncl_dcm_create)(ncl_extent m, ncl_extent n) {
  ncl_dcsubmatrix	M;
  ncl_dscalar*	ps
    = (ncl_dscalar*)((0 < m*n)? malloc(m*n*sizeof(ncl_dcscalar)): NULL);
#ifdef NCL_ERRANT
  if (!ps)
    ncl_message("In function ncl_dcm_create(ncl_extent, ncl_extent):\n"
      "  memory allocation failed.\n");
#endif /* NCL_ERRANT */
  ncl_dcsubm_init(&M, ncl_dch_create(ps),
    (ncl_offset)0, m, n, n, (ncl_stride)1); M._ = ncl_owner;
  return M; }

ncl_dcmatrix		/* automatic storage clone constructor	*/
(ncl_dcm_createclone)(const ncl_dcsubmatrix* pN) {
  ncl_extent	n = ncl_dcsubm_extent1(pN);
  ncl_extent	m = ncl_dcsubm_extent2(pN);
  ncl_dcmatrix	M = ncl_dcm_create(m, n);
  ncl_dcsubm_mcpy(&M, pN);
  return M; }

void				/* automatic storage  destructor	*/
(ncl_dcm_destroy)(ncl_dcmatrix* pM) {
  ncl_dchandle	h = ncl_dcsubm_handle(pM);
#ifdef NCL_ERRANT
  if (ncl_owner != pM->_)
    ncl_message("In function ncl_dcm_destroy(ncl_dcmatrix*):\n"
      "  argument 1 is not a valid ncl_dcmatrix*\n");
#endif /* NCL_ERRANT */
  free((void*)(ncl_dch_pointer(&h)));
  }

ncl_dcmatrix*		/*      free storage constructor	*/
(ncl_dcm_new)(ncl_extent m, ncl_extent n) {
  ncl_dcsubmatrix*	pM
    = (ncl_dcsubmatrix*)malloc(sizeof(ncl_dcsubmatrix));
  if (pM) {
    ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < m*n)? malloc(m*n*sizeof(ncl_dcscalar)): NULL);
#ifdef NCL_ERRANT
    if (!ps)
      ncl_message("In function ncl_dcm_new(ncl_extent, ncl_extent):\n"
      "  memory allocation failed.\n");
#endif /* NCL_ERRANT */
    ncl_dcsubm_init(pM, ncl_dch_create(ps),
    (ncl_offset)0, m, n, n, (ncl_stride)1); pM->_ = ncl_owner;
    }
#ifdef NCL_ERRANT
  else
    ncl_message("In function ncl_dcm_new(ncl_extent, ncl_extent):\n"
      "  memory allocation failed.\n");
#endif /* NCL_ERRANT */
  return pM; }

ncl_dcmatrix*		/*      free storage clone constructor	*/
(ncl_dcm_newclone)(const ncl_dcsubmatrix* pN) {
  ncl_extent	n = ncl_dcsubm_extent1(pN);
  ncl_extent	m = ncl_dcsubm_extent2(pN);
  return ncl_dcsubm_mcpy(ncl_dcm_new(m, n), pN); }

void				/*      free storage  destructor	*/
(ncl_dcm_delete)(ncl_dcmatrix* pM) {
  ncl_dchandle	h = ncl_dcsubm_handle(pM);
#ifdef NCL_ERRANT
  if (ncl_owner != pM->_)
    ncl_message("In function ncl_dcm_delete(ncl_dcmatrix*):\n"
      "  argument 1 is not a valid ncl_dcmatrix*\n");
#endif /* NCL_ERRANT */
  free((void*)(ncl_dch_pointer(&h)));
  free((void*)pM); }

ncl_dcmatrix*		/*                 reconstructor	*/
(ncl_dcm_resize)(ncl_dcmatrix* pM, ncl_extent m, ncl_extent n) {
  ncl_dchandle	h = ncl_dcsubm_handle(pM);
#ifdef NCL_ERRANT
  if (ncl_owner != pM->_) ncl_message(
 "In function ncl_dcm_resize(ncl_dcmatrix*, ncl_extent, ncl_extent):\n"
 "  argument 1 is not a valid ncl_dcmatrix*\n");
#endif /* NCL_ERRANT */
  if (!ncl_dch_empty(&h)) {
    free((void*)(ncl_dch_pointer(&h)));
    }
  { ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < m*n)? malloc(m*n*sizeof(ncl_dcscalar)): NULL);
#ifdef NCL_ERRANT
    if (!ps) ncl_message(
 "In function ncl_dcm_resize(ncl_dcmatrix*, ncl_extent, ncl_extent):\n"
 "  memory allocation failed.\n");
#endif /* NCL_ERRANT */
    ncl_dcsubm_init(pM, ncl_dch_create(ps),
      (ncl_offset)0, m, n, n, (ncl_stride)1);
    }
  return pM; }
  
ncl_dcmatrix*		/*           clone reconstructor	*/
(ncl_dcm_resizeclone)(ncl_dcmatrix* pM,
		const ncl_dcsubmatrix* pN) {
  ncl_extent	n = ncl_dcsubm_extent1(pN);
  ncl_extent	m = ncl_dcsubm_extent2(pN);
  return ncl_dcsubm_mcpy(ncl_dcm_resize(pM, m, n), pN); }
  
int
(ncl_dcsubm_fprintf)(FILE* stream, const ncl_dcsubmatrix* pM, 
  int width, unsigned int precision, ncl_fmtflags flags, unsigned int columns) {
  int           total = 0;
  const
  ncl_extent	m = ncl_dcsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dcsubvector	v = ncl_dcsubm_subv2(pM, i);
    int characters
      = ncl_dcsubv_fprintf(stream, &v, width, precision, flags, columns);
    ncl_dcsubv_destroy(&v);
    if (0 <= characters) { total += characters; } else { return -1; }
    }
  return total;
  }

