/*
The ANSI C Numerical Class Library.
Copyright (C) 2000 E. Robert Tisdale

This file is part of The ANSI C Numerical Class Library.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#undef NCL_INLINE
#undef NCL_MACROS
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif  /* NCL_REVEAL */

#include<ncl_dmatrix.h>

/* Note: The pointer ncl_dscalar* ps must be an lvalue!		*/
#define ncl_dh_pointer(ph) ((ph)->P)
#define ncl_dh_create(ps) (*((ncl_dhandle*)(&(ps))))
#define ncl_dh_destroy(ph)
#define ncl_dh_empty(ph) ((ph)->P == NULL)
#define ncl_dh_get(ph, o) (((ncl_dscalar*)((ph)->P))[o])
#define ncl_dh_set(ph, o, x) (((ncl_dscalar*)((ph)->P))[o] = (x))

#define ncl_dsubv_handle(pv) ((pv)->H);
#define ncl_dsubv_offset(pv) ((pv)->O);
#define ncl_dsubv_extent(pv) ((pv)->N);
#define ncl_dsubv_stride(pv) ((pv)->S);
#define ncl_dsubv_init(pv, h, o, n, s) \
do { \
  (pv)->H = (h); (pv)->O = (o); \
  (pv)->N = (n); (pv)->S = (s); (pv)->_ = ncl_other; \
  } while (0)
#define ncl_dsubv_initclone(pv, pu) ncl_dsubv_init(pv, \
ncl_dsubv_handle(pu), ncl_dsubv_offset(pu), \
ncl_dsubv_extent(pu), ncl_dsubv_stride(pu))
#define ncl_dsubv_destroy(pv) ((pv)->_ = ~ncl_other)
#define ncl_dsubv_empty(pv) \
(ncl_dh_empty(&((pv)->H)) || 0 == ncl_dsubv_extent(pv))
#define ncl_dsubv_get(pv, j) ncl_dh_get(&((pv)->H), \
ncl_dsubv_offset(pv) + (ncl_stride)(j)*ncl_dsubv_stride(pv))
#define ncl_dsubv_set(pv, j, x) ncl_dh_set(&((pv)->H), \
ncl_dsubv_offset(pv) + (ncl_stride)(j)*ncl_dsubv_stride(pv), (x))
#define ncl_dv_destroy(pv) \
do { \
  (pv)->_ = ~ncl_owner; \
  free((void*)(ncl_dh_pointer(&ncl_dsubv_handle(pv)))); \
  } while (0)
#define ncl_dv_delete(pv) \
do { ncl_dv_destroy(pv); free((void*)(pv)); } while (0)

				/* define  submatrix operations		*/
ncl_dhandle
(ncl_dsubm_handle)(const ncl_dsubmatrix* pM) { return pM->H; }

#define ncl_dsubm_handle(pM) ((pM)->H)

ncl_offset
(ncl_dsubm_offset)(const ncl_dsubmatrix* pM) { return pM->O; }

#define ncl_dsubm_offset(pM) ((pM)->O)

ncl_extent
(ncl_dsubm_extent1)(const ncl_dsubmatrix* pM) { return pM->N1; }

#define ncl_dsubm_extent1(pM) ((pM)->N1)

ncl_stride
(ncl_dsubm_stride1)(const ncl_dsubmatrix* pM) { return pM->S1; }

#define ncl_dsubm_stride1(pM) ((pM)->S1)

ncl_extent
(ncl_dsubm_extent2)(const ncl_dsubmatrix* pM) { return pM->N2; }

#define ncl_dsubm_extent2(pM) ((pM)->N2)

ncl_stride
(ncl_dsubm_stride2)(const ncl_dsubmatrix* pM) { return pM->S2; }

#define ncl_dsubm_stride2(pM) ((pM)->S2)

void				/* initializer				*/
(ncl_dsubm_init)(ncl_dsubmatrix* pM,
    ncl_dhandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  pM->H  = h; pM->O  = o;
  pM->N1 = n; pM->S1 = s1;
  pM->N2 = m; pM->S2 = s2; pM->_ = ncl_other; }

#define ncl_dsubm_init(pM, h, o, m, s2, n, s1) \
do { \
  (pM)->H  = (h); (pM)->O  = (o); \
  (pM)->N1 = (n); (pM)->S1 = (s1); \
  (pM)->N2 = (m); (pM)->S2 = (s2); (pM)->_ = ncl_other; \
  } while (0)

void				/* initializer clone			*/
(ncl_dsubm_initclone)(ncl_dsubmatrix* pM,
		const ncl_dsubmatrix* pN) {
  ncl_dhandle	h  = ncl_dsubm_handle(pN);
  ncl_offset	o  = ncl_dsubm_offset(pN);
  ncl_extent	n  = ncl_dsubm_extent1(pN);
  ncl_stride	s1 = ncl_dsubm_stride1(pN);
  ncl_extent	m  = ncl_dsubm_extent2(pN);
  ncl_stride	s2 = ncl_dsubm_stride2(pN);
  ncl_dsubm_init(pM, h, o, m, s2, n, s1); }

#define ncl_dsubm_initclone(pM, pN) ncl_dsubm_init( \
ncl_dsubm_handle(pN), ncl_dsubm_offset(pN), \
ncl_dsubm_extent2(pN), ncl_dsubm_stride2(pN), \
ncl_dsubm_extent1(pN), ncl_dsubm_stride1(pN))

ncl_dsubmatrix		/* automatic storage constructor	*/
(ncl_dsubm_create)(
    ncl_dhandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  ncl_dsubmatrix	M;
  ncl_dsubm_init(&M, h, o, m, s2, n, s1);
  return M; }

ncl_dsubmatrix		/* automatic storage clone constructor	*/
(ncl_dsubm_createclone)(const ncl_dsubmatrix* pN) {
  ncl_dsubmatrix	M = *pN; M._ = ncl_other;
  return M; }

void				/* automatic storage  destructor	*/
(ncl_dsubm_destroy)(ncl_dsubmatrix* pM) {
#ifdef NCL_ERRANT
  if (ncl_other != pM->_)
    ncl_message("In function ncl_dsubm_destroy(ncl_dsubmatrix*):\n"
      "  argument 1 is not a valid ncl_dsubmatrix*\n");
#endif /* NCL_ERRANT */
  pM->_ = ~ncl_other; }

#define ncl_dsubm_destroy(pM) (pM->_ = ~ncl_other)

ncl_dsubmatrix*		/*      free storage constructor	*/
(ncl_dsubm_new)(
    ncl_dhandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
  ncl_dsubmatrix*	pM
    = (ncl_dsubmatrix*)malloc(sizeof(ncl_dsubmatrix));
  if (pM)
    ncl_dsubm_init(pM, h, o, m, s2, n, s1);
#ifdef NCL_ERRANT
  else
    ncl_message(
     "In function ncl_dsubm_new(ncl_dhandle, ncl_offset,\n"
   "  ncl_extent, ncl_stride, ncl_extent, ncl_stride):\n"
   "  memory allocation failed.\n");
#endif /* NCL_ERRANT */
  return pM; }

ncl_dsubmatrix*		/*      free storage clone constructor	*/
(ncl_dsubm_newclone)(const ncl_dsubmatrix* pN) {
  ncl_dhandle	h  = ncl_dsubm_handle(pN);
  ncl_offset	o  = ncl_dsubm_offset(pN);
  ncl_extent	n  = ncl_dsubm_extent1(pN);
  ncl_stride	s1 = ncl_dsubm_stride1(pN);
  ncl_extent	m  = ncl_dsubm_extent2(pN);
  ncl_stride	s2 = ncl_dsubm_stride2(pN);
  return ncl_dsubm_new(h, o, m, s2, n, s1); }

void				/*      free storage  destructor	*/
(ncl_dsubm_delete)(ncl_dsubmatrix* pM) {
#ifdef NCL_ERRANT
  if (ncl_other != pM->_)
    ncl_message("In function ncl_dsubm_delete(ncl_dsubmatrix*):\n"
      "  argument 1 is not a valid ncl_dsubmatrix*\n");
#endif /* NCL_ERRANT */
  ncl_dsubm_destroy(pM);
  free((void*)pM); }

ncl_dsubmatrix*		/*                 reconstructor	*/
(ncl_dsubm_resize)(ncl_dsubmatrix* pM,
    ncl_dhandle h, ncl_offset o,
    ncl_extent m, ncl_stride s2,
    ncl_extent n, ncl_stride s1) {
#ifdef NCL_ERRANT
  if (ncl_other != pM->_)
    ncl_message(
  "In function ncl_dsubm_resize(ncl_dsubmatrix*, ncl_dhandle,\n"
  "  ncl_offset, ncl_extent, ncl_stride, ncl_extent, ncl_stride):\n"
  "  argument 1 is not a valid ncl_dsubmatrix*\n");
#endif /* NCL_ERRANT */
  ncl_dsubm_init(pM, h, o, m, s2, n, s1);
  return pM; }

ncl_dsubmatrix*		/*           clone reconstructor	*/
(ncl_dsubm_resizeclone)(ncl_dsubmatrix* pM,
		const ncl_dsubmatrix* pN) {
  ncl_dhandle	h  = ncl_dsubm_handle(pN);
  ncl_offset	o  = ncl_dsubm_offset(pN);
  ncl_extent	n  = ncl_dsubm_extent1(pN);
  ncl_stride	s1 = ncl_dsubm_stride1(pN);
  ncl_extent	m  = ncl_dsubm_extent2(pN);
  ncl_stride	s2 = ncl_dsubm_stride2(pN);
  return ncl_dsubm_resize(pM, h, o, m, s2, n, s1); }

int
(ncl_dsubm_empty)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h = ncl_dsubm_handle(pM);
  ncl_extent	n = ncl_dsubm_extent1(pM);
  ncl_extent	m = ncl_dsubm_extent2(pM);
  return ncl_dh_empty(&h) || 0 == n || 0 == m; }

ncl_dscalar
(ncl_dsubm_get)(const ncl_dsubmatrix* pM,
    ncl_offset i, ncl_offset j) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  if (m <= i)
    ncl_message(
      "In function ncl_dsubm_get(const ncl_dsubmatrix*,\n"
      "  ncl_offset, ncl_offset): 1st offset exceeds extent.\n");
  if (n <= j)
    ncl_message(
      "In function ncl_dsubm_get(const ncl_dsubmatrix*,\n"
      "  ncl_offset, ncl_offset): 2nd offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  return ncl_dh_get(&h, o + (ncl_stride)j*s1 + (ncl_stride)i*s2); }

#define ncl_dsubm_get_(pM, i, j) \
ncl_dh_get(&((pM)->H), ncl_dsubm_offset(pM) \
+ (ncl_stride)(j)*ncl_dsubm_stride1(pM) \
+ (ncl_stride)(i)*ncl_dsubm_stride2(pM))

void
(ncl_dsubm_set)(      ncl_dsubmatrix* pM,
    ncl_offset i, ncl_offset j, ncl_dscalar x) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  if (m <= i)
    ncl_message(
      "In function ncl_dsubm_set(ncl_dsubmatrix*, ncl_offset,\n"
      "  ncl_offset, ncl_dscalar): 1st offset exceeds extent.\n");
  if (n <= j)
    ncl_message(
      "In function ncl_dsubm_set(ncl_dsubmatrix*, ncl_offset,\n"
      "  ncl_offset, ncl_dscalar): 2nd offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  ncl_dh_set(&h, o + (ncl_stride)j*s1 + (ncl_stride)i*s2, x); }

#define ncl_dsubm_set_(pM, i, j, x) \
ncl_dh_set(&((pM)->H), ncl_dsubm_offset(pM) \
+ (ncl_stride)(j)*ncl_dsubm_stride1(pM) \
+ (ncl_stride)(i)*ncl_dsubm_stride2(pM), (x))

ncl_dsubvector		/*  vector view	*/
(ncl_dsubm_subv1)(const ncl_dsubmatrix* pM, ncl_offset j) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  if (n <= j)
    ncl_message(
  "In function ncl_dsubm_subv1(const ncl_dsubmatrix*, ncl_offset):\n"
  "  offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  return ncl_dsubv_create(h, o + (ncl_stride)j*s1, m, s2);
  }

ncl_dsubvector		/*  vector view	*/
(ncl_dsubm_subv2)(const ncl_dsubmatrix* pM, ncl_offset i) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  if (m <= i)
    ncl_message(
    "In function ncl_dsubm_subv2(const ncl_dsubmatrix*, ncl_offset):\n"
    "  offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  return ncl_dsubv_create(h, o + (ncl_stride)i*s2, n, s1);
  }

ncl_dsubmatrix		/*  matrix view	*/
(ncl_dsubm_subm1)(const ncl_dsubmatrix* pM,
    ncl_offset j, ncl_extent n, ncl_stride s) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	n_ = ncl_dsubm_extent1(pM);
  if (!ncl_contains(n_, j, n, s))
    ncl_message("In function ncl_dsubm_subm1(const ncl_dsubmatrix*,\n"
   "  ncl_offset, ncl_extent, ncl_stride): matrix cannot contain submatrix.\n");
#endif /* NCL_ERRANT */
  return ncl_dsubm_create(h, o + (ncl_stride)j*s1, m, s2, n, s*s1); }

ncl_dsubmatrix		/*  matrix view	*/
(ncl_dsubm_subm2)(const ncl_dsubmatrix* pM,
    ncl_offset i, ncl_extent m, ncl_stride s) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	m_ = ncl_dsubm_extent2(pM);
  if (!ncl_contains(m_, i, m, s))
    ncl_message("In function ncl_dsubm_subm2(const ncl_dsubmatrix*,\n"
   "  ncl_offset, ncl_extent, ncl_stride): matrix cannot contain submatrix.\n");
#endif /* NCL_ERRANT */
  return ncl_dsubm_create(h, o + (ncl_stride)i*s2, m, s*s2, n, s1); }

ncl_dsubmatrix		/*  matrix view	*/
(ncl_dsubm_subm)(const ncl_dsubmatrix* pM,
    ncl_offset i, ncl_extent m, ncl_stride s2,
    ncl_offset j, ncl_extent n, ncl_stride s1) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_stride	s1_= ncl_dsubm_stride1(pM);
  ncl_stride	s2_= ncl_dsubm_stride2(pM);
#ifdef NCL_ERRANT
  ncl_extent	n_= ncl_dsubm_extent1(pM);
  ncl_extent	m_= ncl_dsubm_extent2(pM);
  if (!(ncl_contains(n_, j, n, s1) && ncl_contains(m_, i, m, s2)))
    ncl_message("In function ncl_dsubm_subm(const ncl_dsubmatrix*,\n"
"  ncl_offset, ncl_extent, ncl_stride, ncl_offset, ncl_extent, ncl_stride):\n"
"  matrix cannot contain submatrix.\n");
#endif /* NCL_ERRANT */
  return ncl_dsubm_create(h,
    o + (ncl_stride)j*s1_ + (ncl_stride)i*s2_, m, s2*s2_, n, s1*s1_); }

ncl_dsubmatrix		/* reverse view	*/
(ncl_dsubm_r1)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  o += (0 < n)? (ncl_stride)(n-1)*s1: 0;
  return ncl_dsubm_create(h, o, m, +s2, n, -s1); }

ncl_dsubmatrix		/* reverse view	*/
(ncl_dsubm_r2)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  o += (0 < m)? (ncl_stride)(m-1)*s2: 0;
  return ncl_dsubm_create(h, o, m, -s2, n, +s1); }

ncl_dsubmatrix		/* reverse view	*/
(ncl_dsubm_r)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  o += (0 < n)? (ncl_stride)(n-1)*s1: 0;
  o += (0 < m)? (ncl_stride)(m-1)*s2: 0;
  return ncl_dsubm_create(h, o, m, -s2, n, -s1); }

ncl_dsubmatrix	/*       transpose view	*/
(ncl_dsubm_t)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  return ncl_dsubm_create(h, o, n, s1, m, s2); }

ncl_dsubvector	/*        diagonal view	*/
(ncl_dsubm_diag)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  return ncl_dsubv_create(h, o, ((m < n)? m: n), s1 + s2); }

ncl_dsubmatrix		/* columns 0, 2, ...,   2j, ...		*/
(ncl_dsubm_even)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  return ncl_dsubm_create(h, o, m, s2, (n+1) >> 1, s1 << 1); }

ncl_dsubmatrix		/* columns 1, 3, ..., 1+2j, ...		*/
(ncl_dsubm_odd)(const ncl_dsubmatrix* pM) {
  ncl_dhandle	h  = ncl_dsubm_handle(pM);
  ncl_offset	o  = ncl_dsubm_offset(pM);
  ncl_extent	n  = ncl_dsubm_extent1(pM);
  ncl_stride	s1 = ncl_dsubm_stride1(pM);
  ncl_extent	m  = ncl_dsubm_extent2(pM);
  ncl_stride	s2 = ncl_dsubm_stride2(pM);
  return ncl_dsubm_create(h, o, m, s2, n >> 1, s1 << 1); }

ncl_dsubmatrix*		/* M[i] <-> M[k]			*/
(ncl_dsubm_vswap)(ncl_dsubmatrix* pM, ncl_offset i, ncl_offset k) {
#ifdef NCL_ERRANT
  ncl_extent	m = ncl_dsubm_extent2(pM);
  if (m <= i)
    ncl_message(
      "In function ncl_dsubm_swap(ncl_dsubmatrix*,\n"
      "  ncl_offset, ncl_offset): 1st offset exceeds extent.\n");
  if (m <= k)
    ncl_message(
      "In function ncl_dsubm_swap(ncl_dsubmatrix*,\n"
      "  ncl_offset, ncl_offset): 2nd offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  { ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubvector	u = ncl_dsubm_subv2(pM, k);
    ncl_dsubv_vswap(&v, &u);
    ncl_dsubv_destroy(&u);
    ncl_dsubv_destroy(&v);
    }
  return pM; }

ncl_dsubmatrix*		/* M <-> N				*/
(ncl_dsubm_mswap)(ncl_dsubmatrix* pM, ncl_dsubmatrix* pN) {
  ncl_extent	mM = ncl_dsubm_extent2(pM);
  ncl_offset	i  = 0;
#ifdef NCL_ERRANT
  ncl_extent	mN = ncl_dsubm_extent2(pN);
  if (mM != mN) {
    ncl_message("In function ncl_dsubm_mswap(ncl_dsubmatrix*,\n"
      "  ncl_dsubmatrix*): submatrices differ in 2nd extent.\n");
    mM = (mM < mN)? mM: mN;
    }
  { ncl_extent	nM = ncl_dsubm_extent1(pM);
    ncl_extent	nN = ncl_dsubm_extent1(pN);
    if (nM != nN) {
      ncl_message("In function ncl_dsubm_mswap(ncl_dsubmatrix*,\n"
        "  ncl_dsubmatrix*): submatrices differ in 1st extent.\n");
      }
    }
#endif /* NCL_ERRANT */
  for (i = 0; i < mM; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubvector	u = ncl_dsubm_subv2(pN, i);
    ncl_dsubv_vswap(&v, &u);
    ncl_dsubv_destroy(&u);
    ncl_dsubv_destroy(&v);
    }
  return pM; }

ncl_dsubmatrix*
(ncl_dsubm_ramp)(ncl_dsubmatrix* pM) {
  ncl_extent	m = ncl_dsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubv_ramp(&v);
    ncl_dsubv_destroy(&v);
    }
  return pM; }

ncl_dsubmatrix*
(ncl_dsubm_reverse)(ncl_dsubmatrix* pM) {
  ncl_extent	m = ncl_dsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubv_reverse(&v);
    ncl_dsubv_destroy(&v);
    }
  return pM; }
					/* matrix-scalar operations	*/
ncl_dsubmatrix*			/* M  = x */
(ncl_dsubm_fill)(ncl_dsubmatrix* pM, ncl_dscalar x) {
  ncl_extent	m = ncl_dsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubv_fill(&v, x);
    ncl_dsubv_destroy(&v);
    }
  return pM; }

ncl_dsubmatrix*			/* M *= x */
(ncl_dsubm_smul)(ncl_dsubmatrix* pM, ncl_dscalar x) {
  ncl_extent	m = ncl_dsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubv_smul(&v, x);
    ncl_dsubv_destroy(&v);
    }
  return pM; }

ncl_dsubmatrix*			/* M *= x */
(ncl_dsubm_sdiv)(ncl_dsubmatrix* pM, ncl_dscalar x) {
  ncl_extent	m = ncl_dsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubv_sdiv(&v, x);
    ncl_dsubv_destroy(&v);
    }
  return pM; }

ncl_dsubmatrix*			/* M *= x */
(ncl_dsubm_sadd)(ncl_dsubmatrix* pM, ncl_dscalar x) {
  ncl_extent	m = ncl_dsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubv_sadd(&v, x);
    ncl_dsubv_destroy(&v);
    }
  return pM; }

ncl_dsubmatrix*			/* M *= x */
(ncl_dsubm_ssub)(ncl_dsubmatrix* pM, ncl_dscalar x) {
  ncl_extent	m = ncl_dsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubv_ssub(&v, x);
    ncl_dsubv_destroy(&v);
    }
  return pM; }
					/* matrix-matrix operations	*/
ncl_dsubmatrix*			/* M  = N */
(ncl_dsubm_mcpy)(ncl_dsubmatrix* pM, const ncl_dsubmatrix* pN) {
  ncl_extent	mM = ncl_dsubm_extent2(pM);
  ncl_offset	i = 0;
#ifdef NCL_ERRANT
  ncl_extent	mN = ncl_dsubm_extent2(pN);
  if (mM != mN) {
    ncl_message("In function ncl_dsubm_mcpy(ncl_dsubmatrix*,\n"
      "  const ncl_dsubmatrix*): submatrix differ in second extent.\n");
    mM = (mM < mN)? mM: mN;
    }
#endif /* NCL_ERRANT */
  for (i = 0; i < mM; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubvector	u = ncl_dsubm_subv2(pN, i);
    ncl_dsubv_vcpy(&v, &u);
    ncl_dsubv_destroy(&u);
    ncl_dsubv_destroy(&v);
    }
  return pM; }

ncl_dsubmatrix*			/* M *= N */
(ncl_dsubm_mmul)(ncl_dsubmatrix* pM, const ncl_dsubmatrix* pN) {
  ncl_extent	mM = ncl_dsubm_extent2(pM);
  ncl_offset	i = 0;
#ifdef NCL_ERRANT
  ncl_extent	mN = ncl_dsubm_extent2(pN);
  if (mM != mN) {
    ncl_message("In function ncl_dsubm_mmul(ncl_dsubmatrix*,\n"
      "  const ncl_dsubmatrix*): submatrix differ in second extent.\n");
    mM = (mM < mN)? mM: mN;
    }
#endif /* NCL_ERRANT */
  for (i = 0; i < mM; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubvector	u = ncl_dsubm_subv2(pN, i);
    ncl_dsubv_vmul(&v, &u);
    ncl_dsubv_destroy(&u);
    ncl_dsubv_destroy(&v);
    }
  return pM; }

ncl_dsubmatrix*			/* M /= N */
(ncl_dsubm_mdiv)(ncl_dsubmatrix* pM, const ncl_dsubmatrix* pN) {
  ncl_extent	mM = ncl_dsubm_extent2(pM);
  ncl_offset	i = 0;
#ifdef NCL_ERRANT
  ncl_extent	mN = ncl_dsubm_extent2(pN);
  if (mM != mN) {
    ncl_message("In function ncl_dsubm_mdiv(ncl_dsubmatrix*,\n"
      "  const ncl_dsubmatrix*): submatrix differ in second extent.\n");
    mM = (mM < mN)? mM: mN;
    }
#endif /* NCL_ERRANT */
  for (i = 0; i < mM; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubvector	u = ncl_dsubm_subv2(pN, i);
    ncl_dsubv_vdiv(&v, &u);
    ncl_dsubv_destroy(&u);
    ncl_dsubv_destroy(&v);
    }
  return pM; }

ncl_dsubmatrix*			/* M += N */
(ncl_dsubm_madd)(ncl_dsubmatrix* pM, const ncl_dsubmatrix* pN) {
  ncl_extent	mM = ncl_dsubm_extent2(pM);
  ncl_offset	i = 0;
#ifdef NCL_ERRANT
  ncl_extent	mN = ncl_dsubm_extent2(pN);
  if (mM != mN) {
    ncl_message("In function ncl_dsubm_madd(ncl_dsubmatrix*,\n"
      "  const ncl_dsubmatrix*): submatrix differ in second extent.\n");
    mM = (mM < mN)? mM: mN;
    }
#endif /* NCL_ERRANT */
  for (i = 0; i < mM; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubvector	u = ncl_dsubm_subv2(pN, i);
    ncl_dsubv_vadd(&v, &u);
    ncl_dsubv_destroy(&u);
    ncl_dsubv_destroy(&v);
    }
  return pM; }

ncl_dsubmatrix*			/* M -= N */
(ncl_dsubm_msub)(ncl_dsubmatrix* pM, const ncl_dsubmatrix* pN) {
  ncl_extent	mM = ncl_dsubm_extent2(pM);
  ncl_offset	i = 0;
#ifdef NCL_ERRANT
  ncl_extent	mN = ncl_dsubm_extent2(pN);
  if (mM != mN) {
    ncl_message("In function ncl_dsubm_msub(ncl_dsubmatrix*,\n"
      "  const ncl_dsubmatrix*): submatrix differ in second extent.\n");
    mM = (mM < mN)? mM: mN;
    }
#endif /* NCL_ERRANT */
  for (i = 0; i < mM; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    ncl_dsubvector	u = ncl_dsubm_subv2(pN, i);
    ncl_dsubv_vsub(&v, &u);
    ncl_dsubv_destroy(&u);
    ncl_dsubv_destroy(&v);
    }
  return pM; }
				/* define     matrix operations		*/
ncl_dmatrix		/* automatic storage constructor	*/
(ncl_dm_create)(ncl_extent m, ncl_extent n) {
  ncl_dsubmatrix	M;
  ncl_dscalar*	ps
    = (ncl_dscalar*)((0 < m*n)? malloc(m*n*sizeof(ncl_dscalar)): NULL);
#ifdef NCL_ERRANT
  if (!ps)
    ncl_message("In function ncl_dm_create(ncl_extent, ncl_extent):\n"
      "  memory allocation failed.\n");
#endif /* NCL_ERRANT */
  ncl_dsubm_init(&M, ncl_dh_create(ps),
    (ncl_offset)0, m, n, n, (ncl_stride)1); M._ = ncl_owner;
  return M; }

ncl_dmatrix		/* automatic storage clone constructor	*/
(ncl_dm_createclone)(const ncl_dsubmatrix* pN) {
  ncl_extent	n = ncl_dsubm_extent1(pN);
  ncl_extent	m = ncl_dsubm_extent2(pN);
  ncl_dmatrix	M = ncl_dm_create(m, n);
  ncl_dsubm_mcpy(&M, pN);
  return M; }

void				/* automatic storage  destructor	*/
(ncl_dm_destroy)(ncl_dmatrix* pM) {
  ncl_dhandle	h = ncl_dsubm_handle(pM);
#ifdef NCL_ERRANT
  if (ncl_owner != pM->_)
    ncl_message("In function ncl_dm_destroy(ncl_dmatrix*):\n"
      "  argument 1 is not a valid ncl_dmatrix*\n");
#endif /* NCL_ERRANT */
  free((void*)(ncl_dh_pointer(&h)));
  }

ncl_dmatrix*		/*      free storage constructor	*/
(ncl_dm_new)(ncl_extent m, ncl_extent n) {
  ncl_dsubmatrix*	pM
    = (ncl_dsubmatrix*)malloc(sizeof(ncl_dsubmatrix));
  if (pM) {
    ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < m*n)? malloc(m*n*sizeof(ncl_dscalar)): NULL);
#ifdef NCL_ERRANT
    if (!ps)
      ncl_message("In function ncl_dm_new(ncl_extent, ncl_extent):\n"
      "  memory allocation failed.\n");
#endif /* NCL_ERRANT */
    ncl_dsubm_init(pM, ncl_dh_create(ps),
    (ncl_offset)0, m, n, n, (ncl_stride)1); pM->_ = ncl_owner;
    }
#ifdef NCL_ERRANT
  else
    ncl_message("In function ncl_dm_new(ncl_extent, ncl_extent):\n"
      "  memory allocation failed.\n");
#endif /* NCL_ERRANT */
  return pM; }

ncl_dmatrix*		/*      free storage clone constructor	*/
(ncl_dm_newclone)(const ncl_dsubmatrix* pN) {
  ncl_extent	n = ncl_dsubm_extent1(pN);
  ncl_extent	m = ncl_dsubm_extent2(pN);
  return ncl_dsubm_mcpy(ncl_dm_new(m, n), pN); }

void				/*      free storage  destructor	*/
(ncl_dm_delete)(ncl_dmatrix* pM) {
  ncl_dhandle	h = ncl_dsubm_handle(pM);
#ifdef NCL_ERRANT
  if (ncl_owner != pM->_)
    ncl_message("In function ncl_dm_delete(ncl_dmatrix*):\n"
      "  argument 1 is not a valid ncl_dmatrix*\n");
#endif /* NCL_ERRANT */
  free((void*)(ncl_dh_pointer(&h)));
  free((void*)pM); }

ncl_dmatrix*		/*                 reconstructor	*/
(ncl_dm_resize)(ncl_dmatrix* pM, ncl_extent m, ncl_extent n) {
  ncl_dhandle	h = ncl_dsubm_handle(pM);
#ifdef NCL_ERRANT
  if (ncl_owner != pM->_) ncl_message(
 "In function ncl_dm_resize(ncl_dmatrix*, ncl_extent, ncl_extent):\n"
 "  argument 1 is not a valid ncl_dmatrix*\n");
#endif /* NCL_ERRANT */
  if (!ncl_dh_empty(&h)) {
    free((void*)(ncl_dh_pointer(&h)));
    }
  { ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < m*n)? malloc(m*n*sizeof(ncl_dscalar)): NULL);
#ifdef NCL_ERRANT
    if (!ps) ncl_message(
 "In function ncl_dm_resize(ncl_dmatrix*, ncl_extent, ncl_extent):\n"
 "  memory allocation failed.\n");
#endif /* NCL_ERRANT */
    ncl_dsubm_init(pM, ncl_dh_create(ps),
      (ncl_offset)0, m, n, n, (ncl_stride)1);
    }
  return pM; }
  
ncl_dmatrix*		/*           clone reconstructor	*/
(ncl_dm_resizeclone)(ncl_dmatrix* pM,
		const ncl_dsubmatrix* pN) {
  ncl_extent	n = ncl_dsubm_extent1(pN);
  ncl_extent	m = ncl_dsubm_extent2(pN);
  return ncl_dsubm_mcpy(ncl_dm_resize(pM, m, n), pN); }
  
int
(ncl_dsubm_fprintf)(FILE* stream, const ncl_dsubmatrix* pM, 
  int width, unsigned int precision, ncl_fmtflags flags, unsigned int columns) {
  int           total = 0;
  const
  ncl_extent	m = ncl_dsubm_extent2(pM);
  ncl_offset	i = 0;
  for (i = 0; i < m; ++i) {
    ncl_dsubvector	v = ncl_dsubm_subv2(pM, i);
    int characters
      = ncl_dsubv_fprintf(stream, &v, width, precision, flags, columns);
    ncl_dsubv_destroy(&v);
    if (0 <= characters) { total += characters; } else { return -1; }
    }
  return total;
  }

