/*
The ANSI C Numerical Class Library.
Copyright (C) 2000 E. Robert Tisdale

This file is part of The ANSI C Numerical Class Library.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#undef NCL_INLINE
#undef NCL_MACROS
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif  /* NCL_REVEAL */

#include<ncl_dcvector.h>

/* Note: The pointer ncl_dscalar* ps must be an lvalue!		*/
#define ncl_dch_pointer(ph) ((ph)->P)
#define ncl_dch_create(ps) (*((ncl_dchandle*)(&(ps))))
#define ncl_dch_destroy(ph)
#define ncl_dch_empty(ph) ((ph)->P == NULL)
#define ncl_dch_get(ph, o) (((ncl_dcscalar*)((ph)->P))[o])
#define ncl_dch_set(ph, o, x) (((ncl_dcscalar*)((ph)->P))[o] = (x))

/* Both ncl_dch_get and ncl_dch_set assume that			*/
/* sizeof(ncl_dcscalar) == 2*sizeof(ncl_dscalar))			*/

				/* define  subvector operations		*/
ncl_dchandle
(ncl_dcsubv_handle)(const ncl_dcsubvector* pv) { return pv->H; }

#define ncl_dcsubv_handle(pv) ((pv)->H)

ncl_offset
(ncl_dcsubv_offset)(const ncl_dcsubvector* pv) { return pv->O; }

#define ncl_dcsubv_offset(pv) ((pv)->O)

ncl_extent
(ncl_dcsubv_extent)(const ncl_dcsubvector* pv) { return pv->N; }

#define ncl_dcsubv_extent(pv) ((pv)->N)

ncl_stride
(ncl_dcsubv_stride)(const ncl_dcsubvector* pv) { return pv->S; }

#define ncl_dcsubv_stride(pv) ((pv)->S)

void				/* initializer				*/
(ncl_dcsubv_init)(ncl_dcsubvector* pv,
    ncl_dchandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  pv->H = h; pv->O = o; pv->N = n; pv->S = s; pv->_ = ncl_other; }

#define ncl_dcsubv_init(pv, h, o, n, s) \
do { \
  (pv)->H = (h); \
  (pv)->O = (o); (pv)->N = (n); (pv)->S = (s); (pv)->_ = ncl_other; \
  } while (0)

void				/* initializer clone			*/
(ncl_dcsubv_initclone)(ncl_dcsubvector* pv,
		const ncl_dcsubvector* pu) {
  ncl_dchandle	h = ncl_dcsubv_handle(pu);
  ncl_offset	o = ncl_dcsubv_offset(pu);
  ncl_extent	n = ncl_dcsubv_extent(pu);
  ncl_stride	s = ncl_dcsubv_stride(pu);
  ncl_dcsubv_init(pv, h, o, n, s); }

#define ncl_dcsubv_initclone(pv, pu) ncl_dcsubv_init(pv, \
ncl_dcsubv_handle(pu), ncl_dcsubv_offset(pu), \
ncl_dcsubv_extent(pu), ncl_dcsubv_stride(pu))

ncl_dcsubvector		/* automatic storage constructor	*/
(ncl_dcsubv_create)(
    ncl_dchandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  ncl_dcsubvector	v;
  ncl_dcsubv_init(&v, h, o, n, s);
  return v; }

ncl_dcsubvector		/* automatic storage clone constructor	*/
(ncl_dcsubv_createclone)(const ncl_dcsubvector* pu) {
  ncl_dcsubvector	v = *pu; v._ = ncl_other;
  return v; }

void				/* automatic storage  destructor	*/
(ncl_dcsubv_destroy)(ncl_dcsubvector* pv) {
#ifdef NCL_ERRANT
  if (ncl_other != pv->_)
    ncl_message("In function ncl_dcsubv_destroy(ncl_dcsubvector*):\n"
      "  argument 1 is not a valid ncl_dcsubvector*\n");
#endif /* NCL_ERRANT */
  pv->_ = ~ncl_other; }

#define ncl_dcsubv_destroy(pv) (pv->_ = ~ncl_other)

ncl_dcsubvector*		/*      free storage constructor	*/
(ncl_dcsubv_new)(
    ncl_dchandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  ncl_dcsubvector*	pv
    = (ncl_dcsubvector*)malloc(sizeof(ncl_dcsubvector));
  if (pv)
    ncl_dcsubv_init(pv, h, o, n, s);
#ifdef NCL_ERRANT
  else
    ncl_message("In function ncl_dcsubv_new(ncl_dchandle,\n"
      "  ncl_offset, ncl_extent, ncl_stride): memory allocation failed.\n");
#endif /* NCL_ERRANT */
  return pv; }

ncl_dcsubvector*		/*      free storage clone constructor	*/
(ncl_dcsubv_newclone)(const ncl_dcsubvector* pu) {
  ncl_dchandle	h = ncl_dcsubv_handle(pu);
  ncl_offset	o = ncl_dcsubv_offset(pu);
  ncl_extent	n = ncl_dcsubv_extent(pu);
  ncl_stride	s = ncl_dcsubv_stride(pu);
  return ncl_dcsubv_new(h, o, n, s); }

void				/*      free storage  destructor	*/
(ncl_dcsubv_delete)(ncl_dcsubvector* pv) {
#ifdef NCL_ERRANT
  if (ncl_other != pv->_)
    ncl_message("In function ncl_dcsubv_delete(ncl_dcsubvector*):\n"
      "  argument 1 is not a valid ncl_dcsubvector*\n");
#endif /* NCL_ERRANT */
  ncl_dcsubv_destroy(pv);
  free((void*)pv); }

ncl_dcsubvector*		/*                 reconstructor	*/
(ncl_dcsubv_resize)(ncl_dcsubvector* pv,
    ncl_dchandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
#ifdef NCL_ERRANT
  if (ncl_other != pv->_)
    ncl_message("In function ncl_dcsubv_resize(ncl_dcsubvector*,\n"
      "ncl_dchandle, ncl_offset, ncl_extent, ncl_stride):\n"
      "  argument 1 is not a valid ncl_dcsubvector*\n");
#endif /* NCL_ERRANT */
  ncl_dcsubv_init(pv, h, o, n, s);
  return pv; }

ncl_dcsubvector*		/*           clone reconstructor	*/
(ncl_dcsubv_resizeclone)(ncl_dcsubvector* pv,
		const ncl_dcsubvector* pu) {
  ncl_dchandle	h = ncl_dcsubv_handle(pu);
  ncl_offset	o = ncl_dcsubv_offset(pu);
  ncl_extent	n = ncl_dcsubv_extent(pu);
  ncl_stride	s = ncl_dcsubv_stride(pu);
  return ncl_dcsubv_resize(pv, h, o, n, s); }

int
(ncl_dcsubv_empty)(const ncl_dcsubvector* pv) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_extent	n = ncl_dcsubv_extent(pv);
  return ncl_dch_empty(&h) || 0 == n; }

ncl_dcscalar
(ncl_dcsubv_get)(const ncl_dcsubvector* pv,
    ncl_offset j) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
#ifdef NCL_ERRANT
  ncl_extent	n = ncl_dcsubv_extent(pv);
  if (n <= j)
    ncl_message(
      "In function ncl_dcsubv_get(const ncl_dcsubvector*,\n"
        "  ncl_offset): offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  return ncl_dch_get(&h, o + (ncl_stride)j*s); }

#define ncl_dcsubv_get_(pv, j) ncl_dch_get(&((pv)->H), \
ncl_dcsubv_offset(pv) + (ncl_stride)(j)*ncl_dcsubv_stride(pv))

void
(ncl_dcsubv_set)(      ncl_dcsubvector* pv,
    ncl_offset j, ncl_dcscalar x) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
#ifdef NCL_ERRANT
  ncl_extent	n = ncl_dcsubv_extent(pv);
  if (n <= j)
    ncl_message(
      "In function ncl_dcsubv_set(const ncl_dcsubvector*,\n"
        "  ncl_offset, ncl_dcscalar): offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  ncl_dch_set(&h, o + (ncl_stride)j*s, x); }

#define ncl_dcsubv_set_(pv, j, x) ncl_dch_set(&((pv)->H), \
ncl_dcsubv_offset(pv) + (ncl_stride)(j)*ncl_dcsubv_stride(pv), (x))

ncl_dcsubvector		/*  vector view	*/
(ncl_dcsubv_subv)(const ncl_dcsubvector* pv,
    ncl_offset j, ncl_extent n, ncl_stride s) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_stride	s_= ncl_dcsubv_stride(pv);
#ifdef NCL_ERRANT
  ncl_extent	n_= ncl_dcsubv_extent(pv);
  if (!ncl_contains(n_, j, n, s))
   ncl_message("In function ncl_dcsubv_subv(const ncl_dcsubvector*,\n"
   "  ncl_offset, ncl_extent, ncl_stride): vector cannot contain subvector.\n");
#endif /* NCL_ERRANT */
  return ncl_dcsubv_create(h, o + (ncl_stride)j*s, n, s*s_); }
  
ncl_dcsubvector		/* reverse view	*/
(ncl_dcsubv_r)(const ncl_dcsubvector* pv) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_extent	n = ncl_dcsubv_extent(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
  o += (0 < n)? (ncl_stride)(n-1)*s: 0;
  return ncl_dcsubv_create(h, o, n, -s); }

ncl_dcsubvector		/* elements 0, 2, ...,   2j, ...	*/
(ncl_dcsubv_even)(const ncl_dcsubvector* pv) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_extent	n = ncl_dcsubv_extent(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
  return ncl_dcsubv_create(h, o, (n+1) >> 1, s << 1); }

ncl_dcsubvector		/* elements 1, 3, ..., 1+2j, ...	*/
(ncl_dcsubv_odd)(const ncl_dcsubvector* pv) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_extent	n = ncl_dcsubv_extent(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
  return ncl_dcsubv_create(h, o, n >> 1, s << 1); }

ncl_dcsubvector*		/* v[j] <-> v[k]			*/
(ncl_dcsubv_sswap)(ncl_dcsubvector* pv, ncl_offset j, ncl_offset k) {
  ncl_dcscalar	t = ncl_dcsubv_get_(pv, j);
#ifdef NCL_ERRANT
  ncl_extent	n = ncl_dcsubv_extent(pv);
  if (n <= j)
    ncl_message(
      "In function ncl_dcsubv_sswap(const ncl_dcsubvector*,\n"
      "  ncl_offset, ncl_offset): first offset exceeds extent.\n");
  if (n <= k)
    ncl_message(
      "In function ncl_dcsubv_sswap(const ncl_dcsubvector*,\n"
      "  ncl_offset, ncl_offset): second offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  ncl_dcsubv_set_(pv, j, ncl_dcsubv_get_(pv, k));
  ncl_dcsubv_set_(pv, k, t);
  return pv; }

ncl_dcsubvector*		/* v <-> u				*/
(ncl_dcsubv_vswap)(ncl_dcsubvector* pv, ncl_dcsubvector* pu) {
  ncl_extent	nv = ncl_dcsubv_extent(pv);
  ncl_offset	j  = 0;
#ifdef NCL_ERRANT
  ncl_extent	nu = ncl_dcsubv_extent(pu);
  if (nv != nu) {
    ncl_message("In function ncl_dcsubv_vswap(ncl_dcsubvector*,\n"
      "  ncl_dcsubvector*): subvectors differ in extent.\n");
    nv = (nv < nu)? nv: nu;
    }
#endif /* NCL_ERRANT */
  for (j = 0; j < nv; ++j) {
    ncl_dcscalar	t = ncl_dcsubv_get(pv, j);
    ncl_dcsubv_set(pv, j, ncl_dcsubv_get(pu, j));
    ncl_dcsubv_set(pu, j, t);
    }
  return pv; }
					/* vector-scalar operations	*/
ncl_dcsubvector*
(ncl_dcsubv_reverse)(ncl_dcsubvector* pv) {
  ncl_extent	n = ncl_dcsubv_extent(pv);
  ncl_extent	m = n >> 1;
  ncl_offset	j = 0;
  for (j = 0; j < m; ++j)
    ncl_dcsubv_sswap(pv, j, n-j);
  return pv; }
					/* vector-scalar operations	*/
ncl_dcsubvector*			/* v  = x */
(ncl_dcsubv_fill)(ncl_dcsubvector* pv, ncl_dcscalar x) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_extent	n = ncl_dcsubv_extent(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
  ncl_offset	j = 0;
  for (j = 0; j < n; ++j) {
    ncl_dch_set(&h, o, x);
    o += s; }
  return pv; }

ncl_dcsubvector*			/* v *= x */
(ncl_dcsubv_smul)(ncl_dcsubvector* pv, ncl_dcscalar x) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_extent	n = ncl_dcsubv_extent(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
  ncl_offset	j = 0;
  for (j = 0; j < n; ++j) {
    ncl_dcscalar	t;
    NCL_DCMUL(t, ncl_dch_get(&h, o), x);
    ncl_dch_set(&h, o, t);
    o += s; }
  return pv; }

ncl_dcsubvector*			/* v /= x */
(ncl_dcsubv_sdiv)(ncl_dcsubvector* pv, ncl_dcscalar x) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_extent	n = ncl_dcsubv_extent(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
  ncl_offset	j = 0;
  for (j = 0; j < n; ++j) {
    ncl_dcscalar	t;
    NCL_DCDIV(t, ncl_dch_get(&h, o), x);
    ncl_dch_set(&h, o, t);
    o += s; }
  return pv; }

ncl_dcsubvector*			/* v += x */
(ncl_dcsubv_sadd)(ncl_dcsubvector* pv, ncl_dcscalar x) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_extent	n = ncl_dcsubv_extent(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
  ncl_offset	j = 0;
  for (j = 0; j < n; ++j) {
    ncl_dcscalar	t;
    NCL_DCADD(t, ncl_dch_get(&h, o), x);
    ncl_dch_set(&h, o, t);
    o += s; }
  return pv; }

ncl_dcsubvector*			/* v -= x */
(ncl_dcsubv_ssub)(ncl_dcsubvector* pv, ncl_dcscalar x) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
  ncl_offset	o = ncl_dcsubv_offset(pv);
  ncl_extent	n = ncl_dcsubv_extent(pv);
  ncl_stride	s = ncl_dcsubv_stride(pv);
  ncl_offset	j = 0;
  for (j = 0; j < n; ++j) {
    ncl_dcscalar	t;
    NCL_DCSUB(t, ncl_dch_get(&h, o), x);
    ncl_dch_set(&h, o, t);
    o += s; }
  return pv; }

					/* vector-vector operations	*/
ncl_dcsubvector*			/* v  = w */
(ncl_dcsubv_vcpy)(ncl_dcsubvector* pv, const ncl_dcsubvector* pw) {
  ncl_dchandle	hv = ncl_dcsubv_handle(pv);
  ncl_offset	ov = ncl_dcsubv_offset(pv);
  ncl_extent	nv = ncl_dcsubv_extent(pv);
  ncl_stride	sv = ncl_dcsubv_stride(pv);
  ncl_dchandle	hw = ncl_dcsubv_handle(pw);
  ncl_offset	ow = ncl_dcsubv_offset(pw);
  ncl_stride	sw = ncl_dcsubv_stride(pw);
  ncl_offset	j = 0;
#ifdef NCL_ERRANT
  ncl_extent	nw = ncl_dcsubv_extent(pw);
  if (nv != nw) {
    ncl_message("In function ncl_dcsubv_vcpy(ncl_dcsubvector*,\n"
      "  const ncl_dcsubvector*): subvectors differ in extent.\n");
    nv = (nv < nw)? nv: nw;
    }
#endif /* NCL_ERRANT */
  for (j = 0; j < nv; ++j) {
    ncl_dch_set(&hv, ov, ncl_dch_get(&hw, ow));
    ov += sv;
    ow += sw; }
  return pv; }

ncl_dcsubvector*			/* v *= w */
(ncl_dcsubv_vmul)(ncl_dcsubvector* pv, const ncl_dcsubvector* pw) {
  ncl_dchandle	hv = ncl_dcsubv_handle(pv);
  ncl_offset	ov = ncl_dcsubv_offset(pv);
  ncl_extent	nv = ncl_dcsubv_extent(pv);
  ncl_stride	sv = ncl_dcsubv_stride(pv);
  ncl_dchandle	hw = ncl_dcsubv_handle(pw);
  ncl_offset	ow = ncl_dcsubv_offset(pw);
  ncl_stride	sw = ncl_dcsubv_stride(pw);
  ncl_offset	j = 0;
#ifdef NCL_ERRANT
  ncl_extent	nw = ncl_dcsubv_extent(pw);
  if (nv != nw) {
    ncl_message("In function ncl_dcsubv_vmul(ncl_dcsubvector*,\n"
      "  const ncl_dcsubvector*): subvectors differ in extent.\n");
    nv = (nv < nw)? nv: nw;
    }
#endif /* NCL_ERRANT */
  for (j = 0; j < nv; ++j) {
    ncl_dcscalar	t;
    NCL_DCMUL(t,
      ncl_dch_get(&hv, ov),
      ncl_dch_get(&hw, ow));
    ncl_dch_set(&hv, ov, t);
    ov += sv;
    ow += sw; }
  return pv; }

ncl_dcsubvector*			/* v /= w */
(ncl_dcsubv_vdiv)(ncl_dcsubvector* pv, const ncl_dcsubvector* pw) {
  ncl_dchandle	hv = ncl_dcsubv_handle(pv);
  ncl_offset	ov = ncl_dcsubv_offset(pv);
  ncl_extent	nv = ncl_dcsubv_extent(pv);
  ncl_stride	sv = ncl_dcsubv_stride(pv);
  ncl_dchandle	hw = ncl_dcsubv_handle(pw);
  ncl_offset	ow = ncl_dcsubv_offset(pw);
  ncl_stride	sw = ncl_dcsubv_stride(pw);
  ncl_offset	j = 0;
#ifdef NCL_ERRANT
  ncl_extent	nw = ncl_dcsubv_extent(pw);
  if (nv != nw) {
    ncl_message("In function ncl_dcsubv_vdiv(ncl_dcsubvector*,\n"
      "  const ncl_dcsubvector*): subvectors differ in extent.\n");
    nv = (nv < nw)? nv: nw;
    }
#endif /* NCL_ERRANT */
  for (j = 0; j < nv; ++j) {
    ncl_dcscalar	t;
    NCL_DCDIV(t,
      ncl_dch_get(&hv, ov),
      ncl_dch_get(&hw, ow));
    ncl_dch_set(&hv, ov, t);
    ov += sv;
    ow += sw; }
  return pv; }

ncl_dcsubvector*			/* v += w */
(ncl_dcsubv_vadd)(ncl_dcsubvector* pv, const ncl_dcsubvector* pw) {
  ncl_dchandle	hv = ncl_dcsubv_handle(pv);
  ncl_offset	ov = ncl_dcsubv_offset(pv);
  ncl_extent	nv = ncl_dcsubv_extent(pv);
  ncl_stride	sv = ncl_dcsubv_stride(pv);
  ncl_dchandle	hw = ncl_dcsubv_handle(pw);
  ncl_offset	ow = ncl_dcsubv_offset(pw);
  ncl_stride	sw = ncl_dcsubv_stride(pw);
  ncl_offset	j = 0;
#ifdef NCL_ERRANT
  ncl_extent	nw = ncl_dcsubv_extent(pw);
  if (nv != nw) {
    ncl_message("In function ncl_dcsubv_vadd(ncl_dcsubvector*,\n"
      "  const ncl_dcsubvector*): subvectors differ in extent.\n");
    nv = (nv < nw)? nv: nw;
    }
#endif /* NCL_ERRANT */
  for (j = 0; j < nv; ++j) {
    ncl_dcscalar	t;
    NCL_DCADD(t,
      ncl_dch_get(&hv, ov),
      ncl_dch_get(&hw, ow));
    ncl_dch_set(&hv, ov, t);
    ov += sv;
    ow += sw; }
  return pv; }

ncl_dcsubvector*			/* v -= w */
(ncl_dcsubv_vsub)(ncl_dcsubvector* pv, const ncl_dcsubvector* pw) {
  ncl_dchandle	hv = ncl_dcsubv_handle(pv);
  ncl_offset	ov = ncl_dcsubv_offset(pv);
  ncl_extent	nv = ncl_dcsubv_extent(pv);
  ncl_stride	sv = ncl_dcsubv_stride(pv);
  ncl_dchandle	hw = ncl_dcsubv_handle(pw);
  ncl_offset	ow = ncl_dcsubv_offset(pw);
  ncl_stride	sw = ncl_dcsubv_stride(pw);
  ncl_offset	j = 0;
#ifdef NCL_ERRANT
  ncl_extent	nw = ncl_dcsubv_extent(pw);
  if (nv != nw) {
    ncl_message("In function ncl_dcsubv_vsub(ncl_dcsubvector*,\n"
      "  const ncl_dcsubvector*): subvectors differ in extent.\n");
    nv = (nv < nw)? nv: nw;
    }
#endif /* NCL_ERRANT */
  for (j = 0; j < nv; ++j) {
    ncl_dcscalar	t;
    NCL_DCSUB(t,
      ncl_dch_get(&hv, ov),
      ncl_dch_get(&hw, ow));
    ncl_dch_set(&hv, ov, t);
    ov += sv;
    ow += sw; }
  return pv; }
				/* define     vector operations		*/
ncl_dcvector		/* automatic storage constructor	*/
(ncl_dcv_create)(ncl_extent n) {
  ncl_dcsubvector	v;
  ncl_dscalar*	ps
    = (ncl_dscalar*)((0 < n)? malloc(n*sizeof(ncl_dcscalar)): NULL);
#ifdef NCL_ERRANT
  if (!ps)
    ncl_message(
"In function ncl_dcv_create(ncl_extent): memory allocation failed.\n");
#endif /* NCL_ERRANT */
  ncl_dcsubv_init(&v, ncl_dch_create(ps),
    (ncl_offset)0, n, (ncl_stride)1); v._ = ncl_owner;
  return v; }

ncl_dcvector		/* automatic storage clone constructor	*/
(ncl_dcv_createclone)(const ncl_dcsubvector* pu) {
  ncl_dcvector	v = ncl_dcv_create(ncl_dcsubv_extent(pu));
  ncl_dcsubv_vcpy(&v, pu);
  return v; }

void				/* automatic storage  destructor	*/
(ncl_dcv_destroy)(ncl_dcvector* pv) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
#ifdef NCL_ERRANT
  if (ncl_owner != pv->_)
    ncl_message("In function ncl_dcv_destroy(ncl_dcvector*):\n"
      "  argument 1 is not a valid ncl_dcvector*\n");
#endif /* NCL_ERRANT */
  free((void*)(ncl_dch_pointer(&h)));
  }

ncl_dcvector*		/*      free storage constructor	*/
(ncl_dcv_new)(ncl_extent n) {
  ncl_dcsubvector*	pv
    = (ncl_dcsubvector*)malloc(sizeof(ncl_dcsubvector));
  if (pv) {
    ncl_dscalar*	ps
       = (ncl_dscalar*)((0 < n)? malloc(n*sizeof(ncl_dcscalar)): NULL);
#ifdef NCL_ERRANT
    if (!ps)
      ncl_message(
        "In function ncl_dcv_new(ncl_extent): memory allocation failed.\n");
#endif /* NCL_ERRANT */
    ncl_dcsubv_init(pv, ncl_dch_create(ps),
    (ncl_offset)0, n, (ncl_stride)1); pv->_ = ncl_owner;
    }
#ifdef NCL_ERRANT
  else
    ncl_message(
      "In function ncl_dcv_new(ncl_extent): memory allocation failed.\n");
#endif /* NCL_ERRANT */
  return pv; }

ncl_dcvector*		/*      free storage clone constructor	*/
(ncl_dcv_newclone)(const ncl_dcsubvector* pu) {
  return ncl_dcsubv_vcpy(ncl_dcv_new(ncl_dcsubv_extent(pu)), pu); }

void				/*      free storage  destructor	*/
(ncl_dcv_delete)(ncl_dcvector* pv) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
#ifdef NCL_ERRANT
  if (ncl_owner != pv->_)
    ncl_message("In function ncl_dcv_delete(ncl_dcvector*):\n"
      "  argument 1 is not a valid ncl_dcvector*\n");
#endif /* NCL_ERRANT */
  free((void*)(ncl_dch_pointer(&h)));
  free((void*)pv); }

ncl_dcvector*		/*                 reconstructor	*/
(ncl_dcv_resize)(ncl_dcvector* pv, ncl_extent n) {
  ncl_dchandle	h = ncl_dcsubv_handle(pv);
#ifdef NCL_ERRANT
  if (ncl_owner != pv->_)
    ncl_message(
      "In function ncl_dcv_resize(ncl_dcvector*, ncl_extent):\n"
      "  argument 1 is not a valid ncl_dcvector*\n");
#endif /* NCL_ERRANT */
  if (!ncl_dch_empty(&h)) {
    free((void*)(ncl_dch_pointer(&h)));
    }
  { ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < n)? malloc(n*sizeof(ncl_dcscalar)): NULL);
#ifdef NCL_ERRANT
    if (!ps)
      ncl_message(
        "In function ncl_dcv_resize(ncl_dcvector*, ncl_extent):\n"
        "  memory allocation failed.\n");
#endif /* NCL_ERRANT */
    ncl_dcsubv_init(pv, ncl_dch_create(ps),
      (ncl_offset)0, n, (ncl_stride)1);
    }
  return pv; }
  
ncl_dcvector*		/*           clone reconstructor	*/
(ncl_dcv_resizeclone)(
    ncl_dcvector* pv, const ncl_dcsubvector* pu) {
  return ncl_dcsubv_vcpy(
    ncl_dcv_resize(pv, ncl_dcsubv_extent(pu)), pu); }
  
int
(ncl_dcsubv_fprintf)(FILE* stream, const ncl_dcsubvector* pv, 
  int width, unsigned int precision, ncl_fmtflags flags, unsigned int columns) {
  int           characters;
  int           total = 0;
  const
  ncl_extent	n = ncl_dcsubv_extent(pv);
  ncl_offset	j = 0;
  columns = (0 < columns)? columns: 4;
  for (j = 0; j < n; ++j) {
    if (0 < j) {
      characters
         = (0 == j%columns)? fprintf(stream, "\n"): fprintf(stream, " ");
      if (0 <= characters) { total += characters; } else { return -1; }
      }
    characters = ncl_dcfprintf(
      stream, ncl_dcsubv_get_(pv, j), width, precision, flags);
    if (0 <= characters) { total += characters; } else { return -1; }
    }
  characters = fprintf(stream, "\n");
  if (0 <= characters) { total += characters; } else { return -1; }
  return total;
  }

