/*
The ANSI C Numerical Class Library.
Copyright (C) 2000 E. Robert Tisdale

This file is part of The ANSI C Numerical Class Library.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#undef NCL_INLINE
#undef NCL_MACROS
#ifndef NCL_REVEAL
#define NCL_REVEAL
#endif  /* NCL_REVEAL */

#include<ncl_dvector.h>

/* Note: The pointer ncl_dscalar* ps must be an lvalue!		*/
#define ncl_dh_pointer(ph) ((ph)->P)
#define ncl_dh_create(ps) (*((ncl_dhandle*)(&(ps))))
#define ncl_dh_destroy(ph)
#define ncl_dh_empty(ph) ((ph)->P == NULL)
#define ncl_dh_get(ph, o) (((ncl_dscalar*)((ph)->P))[o])
#define ncl_dh_set(ph, o, x) (((ncl_dscalar*)((ph)->P))[o] = (x))

				/* define  subvector operations		*/
ncl_dhandle
(ncl_dsubv_handle)(const ncl_dsubvector* pv) { return pv->H; }

#define ncl_dsubv_handle(pv) ((pv)->H)

ncl_offset
(ncl_dsubv_offset)(const ncl_dsubvector* pv) { return pv->O; }

#define ncl_dsubv_offset(pv) ((pv)->O)

ncl_extent
(ncl_dsubv_extent)(const ncl_dsubvector* pv) { return pv->N; }

#define ncl_dsubv_extent(pv) ((pv)->N)

ncl_stride
(ncl_dsubv_stride)(const ncl_dsubvector* pv) { return pv->S; }

#define ncl_dsubv_stride(pv) ((pv)->S)

void				/* initializer				*/
(ncl_dsubv_init)(ncl_dsubvector* pv,
    ncl_dhandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  pv->H = h; pv->O = o; pv->N = n; pv->S = s; pv->_ = ncl_other; }

#define ncl_dsubv_init(pv, h, o, n, s) \
do { \
  (pv)->H = (h); \
  (pv)->O = (o); (pv)->N = (n); (pv)->S = (s); (pv)->_ = ncl_other; \
  } while (0)

void				/* initializer clone			*/
(ncl_dsubv_initclone)(ncl_dsubvector* pv,
		const ncl_dsubvector* pu) {
  ncl_dhandle	h = ncl_dsubv_handle(pu);
  ncl_offset	o = ncl_dsubv_offset(pu);
  ncl_extent	n = ncl_dsubv_extent(pu);
  ncl_stride	s = ncl_dsubv_stride(pu);
  ncl_dsubv_init(pv, h, o, n, s); }

#define ncl_dsubv_initclone(pv, pu) ncl_dsubv_init(pv, \
ncl_dsubv_handle(pu), ncl_dsubv_offset(pu), \
ncl_dsubv_extent(pu), ncl_dsubv_stride(pu))

ncl_dsubvector		/* automatic storage constructor	*/
(ncl_dsubv_create)(
    ncl_dhandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  ncl_dsubvector	v;
  ncl_dsubv_init(&v, h, o, n, s);
  return v; }

ncl_dsubvector		/* automatic storage clone constructor	*/
(ncl_dsubv_createclone)(const ncl_dsubvector* pu) {
  ncl_dsubvector	v = *pu; v._ = ncl_other;
  return v; }

void				/* automatic storage  destructor	*/
(ncl_dsubv_destroy)(ncl_dsubvector* pv) {
#ifdef NCL_ERRANT
  if (ncl_other != pv->_)
    ncl_message("In function ncl_dsubv_destroy(ncl_dsubvector*):\n"
      "  argument 1 is not a valid ncl_dsubvector*\n");
#endif /* NCL_ERRANT */
  pv->_ = ~ncl_other; }

#define ncl_dsubv_destroy(pv) (pv->_ = ~ncl_other)

ncl_dsubvector*		/*      free storage constructor	*/
(ncl_dsubv_new)(
    ncl_dhandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
  ncl_dsubvector*	pv
    = (ncl_dsubvector*)malloc(sizeof(ncl_dsubvector));
  if (pv)
    ncl_dsubv_init(pv, h, o, n, s);
#ifdef NCL_ERRANT
  else
    ncl_message("In function ncl_dsubv_new(ncl_dhandle,\n"
      "  ncl_offset, ncl_extent, ncl_stride): memory allocation failed.\n");
#endif /* NCL_ERRANT */
  return pv; }

ncl_dsubvector*		/*      free storage clone constructor	*/
(ncl_dsubv_newclone)(const ncl_dsubvector* pu) {
  ncl_dhandle	h = ncl_dsubv_handle(pu);
  ncl_offset	o = ncl_dsubv_offset(pu);
  ncl_extent	n = ncl_dsubv_extent(pu);
  ncl_stride	s = ncl_dsubv_stride(pu);
  return ncl_dsubv_new(h, o, n, s); }

void				/*      free storage  destructor	*/
(ncl_dsubv_delete)(ncl_dsubvector* pv) {
#ifdef NCL_ERRANT
  if (ncl_other != pv->_)
    ncl_message("In function ncl_dsubv_delete(ncl_dsubvector*):\n"
      "  argument 1 is not a valid ncl_dsubvector*\n");
#endif /* NCL_ERRANT */
  ncl_dsubv_destroy(pv);
  free((void*)pv); }

ncl_dsubvector*		/*                 reconstructor	*/
(ncl_dsubv_resize)(ncl_dsubvector* pv,
    ncl_dhandle h, ncl_offset o, ncl_extent n, ncl_stride s) {
#ifdef NCL_ERRANT
  if (ncl_other != pv->_)
    ncl_message("In function ncl_dsubv_resize(ncl_dsubvector*,\n"
      "ncl_dhandle, ncl_offset, ncl_extent, ncl_stride):\n"
      "  argument 1 is not a valid ncl_dsubvector*\n");
#endif /* NCL_ERRANT */
  ncl_dsubv_init(pv, h, o, n, s);
  return pv; }

ncl_dsubvector*		/*           clone reconstructor	*/
(ncl_dsubv_resizeclone)(ncl_dsubvector* pv,
		const ncl_dsubvector* pu) {
  ncl_dhandle	h = ncl_dsubv_handle(pu);
  ncl_offset	o = ncl_dsubv_offset(pu);
  ncl_extent	n = ncl_dsubv_extent(pu);
  ncl_stride	s = ncl_dsubv_stride(pu);
  return ncl_dsubv_resize(pv, h, o, n, s); }

int
(ncl_dsubv_empty)(const ncl_dsubvector* pv) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_extent	n = ncl_dsubv_extent(pv);
  return ncl_dh_empty(&h) || 0 == n; }

ncl_dscalar
(ncl_dsubv_get)(const ncl_dsubvector* pv,
    ncl_offset j) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
#ifdef NCL_ERRANT
  ncl_extent	n = ncl_dsubv_extent(pv);
  if (n <= j)
    ncl_message(
      "In function ncl_dsubv_get(const ncl_dsubvector*,\n"
        "  ncl_offset): offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  return ncl_dh_get(&h, o + (ncl_stride)j*s); }

#define ncl_dsubv_get_(pv, j) ncl_dh_get(&((pv)->H), \
ncl_dsubv_offset(pv) + (ncl_stride)(j)*ncl_dsubv_stride(pv))

void
(ncl_dsubv_set)(      ncl_dsubvector* pv,
    ncl_offset j, ncl_dscalar x) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
#ifdef NCL_ERRANT
  ncl_extent	n = ncl_dsubv_extent(pv);
  if (n <= j)
    ncl_message(
      "In function ncl_dsubv_set(const ncl_dsubvector*,\n"
        "  ncl_offset, ncl_dscalar): offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  ncl_dh_set(&h, o + (ncl_stride)j*s, x); }

#define ncl_dsubv_set_(pv, j, x) ncl_dh_set(&((pv)->H), \
ncl_dsubv_offset(pv) + (ncl_stride)(j)*ncl_dsubv_stride(pv), (x))

ncl_dsubvector		/*  vector view	*/
(ncl_dsubv_subv)(const ncl_dsubvector* pv,
    ncl_offset j, ncl_extent n, ncl_stride s) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_stride	s_= ncl_dsubv_stride(pv);
#ifdef NCL_ERRANT
  ncl_extent	n_= ncl_dsubv_extent(pv);
  if (!ncl_contains(n_, j, n, s))
   ncl_message("In function ncl_dsubv_subv(const ncl_dsubvector*,\n"
   "  ncl_offset, ncl_extent, ncl_stride): vector cannot contain subvector.\n");
#endif /* NCL_ERRANT */
  return ncl_dsubv_create(h, o + (ncl_stride)j*s, n, s*s_); }
  
ncl_dsubvector		/* reverse view	*/
(ncl_dsubv_r)(const ncl_dsubvector* pv) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
  o += (0 < n)? (ncl_stride)(n-1)*s: 0;
  return ncl_dsubv_create(h, o, n, -s); }

ncl_dsubvector		/* elements 0, 2, ...,   2j, ...	*/
(ncl_dsubv_even)(const ncl_dsubvector* pv) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
  return ncl_dsubv_create(h, o, (n+1) >> 1, s << 1); }

ncl_dsubvector		/* elements 1, 3, ..., 1+2j, ...	*/
(ncl_dsubv_odd)(const ncl_dsubvector* pv) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
  return ncl_dsubv_create(h, o, n >> 1, s << 1); }

ncl_dsubvector*		/* v[j] <-> v[k]			*/
(ncl_dsubv_sswap)(ncl_dsubvector* pv, ncl_offset j, ncl_offset k) {
  ncl_dscalar	t = ncl_dsubv_get_(pv, j);
#ifdef NCL_ERRANT
  ncl_extent	n = ncl_dsubv_extent(pv);
  if (n <= j)
    ncl_message(
      "In function ncl_dsubv_sswap(const ncl_dsubvector*,\n"
      "  ncl_offset, ncl_offset): first offset exceeds extent.\n");
  if (n <= k)
    ncl_message(
      "In function ncl_dsubv_sswap(const ncl_dsubvector*,\n"
      "  ncl_offset, ncl_offset): second offset exceeds extent.\n");
#endif /* NCL_ERRANT */
  ncl_dsubv_set_(pv, j, ncl_dsubv_get_(pv, k));
  ncl_dsubv_set_(pv, k, t);
  return pv; }

ncl_dsubvector*		/* v <-> u				*/
(ncl_dsubv_vswap)(ncl_dsubvector* pv, ncl_dsubvector* pu) {
  ncl_extent	nv = ncl_dsubv_extent(pv);
  ncl_offset	j  = 0;
#ifdef NCL_ERRANT
  ncl_extent	nu = ncl_dsubv_extent(pu);
  if (nv != nu) {
    ncl_message("In function ncl_dsubv_vswap(ncl_dsubvector*,\n"
      "  ncl_dsubvector*): subvectors differ in extent.\n");
    nv = (nv < nu)? nv: nu;
    }
#endif /* NCL_ERRANT */
  for (j = 0; j < nv; ++j) {
    ncl_dscalar	t = ncl_dsubv_get(pv, j);
    ncl_dsubv_set(pv, j, ncl_dsubv_get(pu, j));
    ncl_dsubv_set(pu, j, t);
    }
  return pv; }

ncl_dsubvector*
(ncl_dsubv_ramp)(ncl_dsubvector* pv) {
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_offset	j = 0;
  for (j = 0; j < n; ++j)
    ncl_dsubv_set_(pv, j, (ncl_dscalar)j);
  return pv; }
					/* vector-scalar operations	*/
ncl_dsubvector*
(ncl_dsubv_reverse)(ncl_dsubvector* pv) {
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_extent	m = n >> 1;
  ncl_offset	j = 0;
  for (j = 0; j < m; ++j)
    ncl_dsubv_sswap(pv, j, n-j);
  return pv; }
					/* vector-scalar operations	*/
ncl_dsubvector*			/* v  = x */
(ncl_dsubv_fill)(ncl_dsubvector* pv, ncl_dscalar x) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
  ncl_offset	j = 0;
  for (j = 0; j < n; ++j) {
    ncl_dh_set(&h, o, x);
    o += s; }
  return pv; }

ncl_dsubvector*			/* v *= x */
(ncl_dsubv_smul)(ncl_dsubvector* pv, ncl_dscalar x) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
  ncl_offset	j = 0;
  for (j = 0; j < n; ++j) {
    ncl_dscalar	t;
    NCL_DMUL(t, ncl_dh_get(&h, o), x);
    ncl_dh_set(&h, o, t);
    o += s; }
  return pv; }

ncl_dsubvector*			/* v /= x */
(ncl_dsubv_sdiv)(ncl_dsubvector* pv, ncl_dscalar x) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
  ncl_offset	j = 0;
  for (j = 0; j < n; ++j) {
    ncl_dscalar	t;
    NCL_DDIV(t, ncl_dh_get(&h, o), x);
    ncl_dh_set(&h, o, t);
    o += s; }
  return pv; }

ncl_dsubvector*			/* v += x */
(ncl_dsubv_sadd)(ncl_dsubvector* pv, ncl_dscalar x) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
  ncl_offset	j = 0;
  for (j = 0; j < n; ++j) {
    ncl_dscalar	t;
    NCL_DADD(t, ncl_dh_get(&h, o), x);
    ncl_dh_set(&h, o, t);
    o += s; }
  return pv; }

ncl_dsubvector*			/* v -= x */
(ncl_dsubv_ssub)(ncl_dsubvector* pv, ncl_dscalar x) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
  ncl_offset	o = ncl_dsubv_offset(pv);
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_stride	s = ncl_dsubv_stride(pv);
  ncl_offset	j = 0;
  for (j = 0; j < n; ++j) {
    ncl_dscalar	t;
    NCL_DSUB(t, ncl_dh_get(&h, o), x);
    ncl_dh_set(&h, o, t);
    o += s; }
  return pv; }

					/* vector-vector operations	*/
ncl_dsubvector*			/* v  = w */
(ncl_dsubv_vcpy)(ncl_dsubvector* pv, const ncl_dsubvector* pw) {
  ncl_dhandle	hv = ncl_dsubv_handle(pv);
  ncl_offset	ov = ncl_dsubv_offset(pv);
  ncl_extent	nv = ncl_dsubv_extent(pv);
  ncl_stride	sv = ncl_dsubv_stride(pv);
  ncl_dhandle	hw = ncl_dsubv_handle(pw);
  ncl_offset	ow = ncl_dsubv_offset(pw);
  ncl_stride	sw = ncl_dsubv_stride(pw);
  ncl_offset	j = 0;
#ifdef NCL_ERRANT
  ncl_extent	nw = ncl_dsubv_extent(pw);
  if (nv != nw) {
    ncl_message("In function ncl_dsubv_vcpy(ncl_dsubvector*,\n"
      "  const ncl_dsubvector*): subvectors differ in extent.\n");
    nv = (nv < nw)? nv: nw;
    }
#endif /* NCL_ERRANT */
  for (j = 0; j < nv; ++j) {
    ncl_dh_set(&hv, ov, ncl_dh_get(&hw, ow));
    ov += sv;
    ow += sw; }
  return pv; }

ncl_dsubvector*			/* v *= w */
(ncl_dsubv_vmul)(ncl_dsubvector* pv, const ncl_dsubvector* pw) {
  ncl_dhandle	hv = ncl_dsubv_handle(pv);
  ncl_offset	ov = ncl_dsubv_offset(pv);
  ncl_extent	nv = ncl_dsubv_extent(pv);
  ncl_stride	sv = ncl_dsubv_stride(pv);
  ncl_dhandle	hw = ncl_dsubv_handle(pw);
  ncl_offset	ow = ncl_dsubv_offset(pw);
  ncl_stride	sw = ncl_dsubv_stride(pw);
  ncl_offset	j = 0;
#ifdef NCL_ERRANT
  ncl_extent	nw = ncl_dsubv_extent(pw);
  if (nv != nw) {
    ncl_message("In function ncl_dsubv_vmul(ncl_dsubvector*,\n"
      "  const ncl_dsubvector*): subvectors differ in extent.\n");
    nv = (nv < nw)? nv: nw;
    }
#endif /* NCL_ERRANT */
  for (j = 0; j < nv; ++j) {
    ncl_dscalar	t;
    NCL_DMUL(t,
      ncl_dh_get(&hv, ov),
      ncl_dh_get(&hw, ow));
    ncl_dh_set(&hv, ov, t);
    ov += sv;
    ow += sw; }
  return pv; }

ncl_dsubvector*			/* v /= w */
(ncl_dsubv_vdiv)(ncl_dsubvector* pv, const ncl_dsubvector* pw) {
  ncl_dhandle	hv = ncl_dsubv_handle(pv);
  ncl_offset	ov = ncl_dsubv_offset(pv);
  ncl_extent	nv = ncl_dsubv_extent(pv);
  ncl_stride	sv = ncl_dsubv_stride(pv);
  ncl_dhandle	hw = ncl_dsubv_handle(pw);
  ncl_offset	ow = ncl_dsubv_offset(pw);
  ncl_stride	sw = ncl_dsubv_stride(pw);
  ncl_offset	j = 0;
#ifdef NCL_ERRANT
  ncl_extent	nw = ncl_dsubv_extent(pw);
  if (nv != nw) {
    ncl_message("In function ncl_dsubv_vdiv(ncl_dsubvector*,\n"
      "  const ncl_dsubvector*): subvectors differ in extent.\n");
    nv = (nv < nw)? nv: nw;
    }
#endif /* NCL_ERRANT */
  for (j = 0; j < nv; ++j) {
    ncl_dscalar	t;
    NCL_DDIV(t,
      ncl_dh_get(&hv, ov),
      ncl_dh_get(&hw, ow));
    ncl_dh_set(&hv, ov, t);
    ov += sv;
    ow += sw; }
  return pv; }

ncl_dsubvector*			/* v += w */
(ncl_dsubv_vadd)(ncl_dsubvector* pv, const ncl_dsubvector* pw) {
  ncl_dhandle	hv = ncl_dsubv_handle(pv);
  ncl_offset	ov = ncl_dsubv_offset(pv);
  ncl_extent	nv = ncl_dsubv_extent(pv);
  ncl_stride	sv = ncl_dsubv_stride(pv);
  ncl_dhandle	hw = ncl_dsubv_handle(pw);
  ncl_offset	ow = ncl_dsubv_offset(pw);
  ncl_stride	sw = ncl_dsubv_stride(pw);
  ncl_offset	j = 0;
#ifdef NCL_ERRANT
  ncl_extent	nw = ncl_dsubv_extent(pw);
  if (nv != nw) {
    ncl_message("In function ncl_dsubv_vadd(ncl_dsubvector*,\n"
      "  const ncl_dsubvector*): subvectors differ in extent.\n");
    nv = (nv < nw)? nv: nw;
    }
#endif /* NCL_ERRANT */
  for (j = 0; j < nv; ++j) {
    ncl_dscalar	t;
    NCL_DADD(t,
      ncl_dh_get(&hv, ov),
      ncl_dh_get(&hw, ow));
    ncl_dh_set(&hv, ov, t);
    ov += sv;
    ow += sw; }
  return pv; }

ncl_dsubvector*			/* v -= w */
(ncl_dsubv_vsub)(ncl_dsubvector* pv, const ncl_dsubvector* pw) {
  ncl_dhandle	hv = ncl_dsubv_handle(pv);
  ncl_offset	ov = ncl_dsubv_offset(pv);
  ncl_extent	nv = ncl_dsubv_extent(pv);
  ncl_stride	sv = ncl_dsubv_stride(pv);
  ncl_dhandle	hw = ncl_dsubv_handle(pw);
  ncl_offset	ow = ncl_dsubv_offset(pw);
  ncl_stride	sw = ncl_dsubv_stride(pw);
  ncl_offset	j = 0;
#ifdef NCL_ERRANT
  ncl_extent	nw = ncl_dsubv_extent(pw);
  if (nv != nw) {
    ncl_message("In function ncl_dsubv_vsub(ncl_dsubvector*,\n"
      "  const ncl_dsubvector*): subvectors differ in extent.\n");
    nv = (nv < nw)? nv: nw;
    }
#endif /* NCL_ERRANT */
  for (j = 0; j < nv; ++j) {
    ncl_dscalar	t;
    NCL_DSUB(t,
      ncl_dh_get(&hv, ov),
      ncl_dh_get(&hw, ow));
    ncl_dh_set(&hv, ov, t);
    ov += sv;
    ow += sw; }
  return pv; }
				/* define     vector operations		*/
ncl_dvector		/* automatic storage constructor	*/
(ncl_dv_create)(ncl_extent n) {
  ncl_dsubvector	v;
  ncl_dscalar*	ps
    = (ncl_dscalar*)((0 < n)? malloc(n*sizeof(ncl_dscalar)): NULL);
#ifdef NCL_ERRANT
  if (!ps)
    ncl_message(
"In function ncl_dv_create(ncl_extent): memory allocation failed.\n");
#endif /* NCL_ERRANT */
  ncl_dsubv_init(&v, ncl_dh_create(ps),
    (ncl_offset)0, n, (ncl_stride)1); v._ = ncl_owner;
  return v; }

ncl_dvector		/* automatic storage clone constructor	*/
(ncl_dv_createclone)(const ncl_dsubvector* pu) {
  ncl_dvector	v = ncl_dv_create(ncl_dsubv_extent(pu));
  ncl_dsubv_vcpy(&v, pu);
  return v; }

void				/* automatic storage  destructor	*/
(ncl_dv_destroy)(ncl_dvector* pv) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
#ifdef NCL_ERRANT
  if (ncl_owner != pv->_)
    ncl_message("In function ncl_dv_destroy(ncl_dvector*):\n"
      "  argument 1 is not a valid ncl_dvector*\n");
#endif /* NCL_ERRANT */
  free((void*)(ncl_dh_pointer(&h)));
  }

ncl_dvector*		/*      free storage constructor	*/
(ncl_dv_new)(ncl_extent n) {
  ncl_dsubvector*	pv
    = (ncl_dsubvector*)malloc(sizeof(ncl_dsubvector));
  if (pv) {
    ncl_dscalar*	ps
       = (ncl_dscalar*)((0 < n)? malloc(n*sizeof(ncl_dscalar)): NULL);
#ifdef NCL_ERRANT
    if (!ps)
      ncl_message(
        "In function ncl_dv_new(ncl_extent): memory allocation failed.\n");
#endif /* NCL_ERRANT */
    ncl_dsubv_init(pv, ncl_dh_create(ps),
    (ncl_offset)0, n, (ncl_stride)1); pv->_ = ncl_owner;
    }
#ifdef NCL_ERRANT
  else
    ncl_message(
      "In function ncl_dv_new(ncl_extent): memory allocation failed.\n");
#endif /* NCL_ERRANT */
  return pv; }

ncl_dvector*		/*      free storage clone constructor	*/
(ncl_dv_newclone)(const ncl_dsubvector* pu) {
  return ncl_dsubv_vcpy(ncl_dv_new(ncl_dsubv_extent(pu)), pu); }

void				/*      free storage  destructor	*/
(ncl_dv_delete)(ncl_dvector* pv) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
#ifdef NCL_ERRANT
  if (ncl_owner != pv->_)
    ncl_message("In function ncl_dv_delete(ncl_dvector*):\n"
      "  argument 1 is not a valid ncl_dvector*\n");
#endif /* NCL_ERRANT */
  free((void*)(ncl_dh_pointer(&h)));
  free((void*)pv); }

ncl_dvector*		/*                 reconstructor	*/
(ncl_dv_resize)(ncl_dvector* pv, ncl_extent n) {
  ncl_dhandle	h = ncl_dsubv_handle(pv);
#ifdef NCL_ERRANT
  if (ncl_owner != pv->_)
    ncl_message(
      "In function ncl_dv_resize(ncl_dvector*, ncl_extent):\n"
      "  argument 1 is not a valid ncl_dvector*\n");
#endif /* NCL_ERRANT */
  if (!ncl_dh_empty(&h)) {
    free((void*)(ncl_dh_pointer(&h)));
    }
  { ncl_dscalar*	ps
      = (ncl_dscalar*)((0 < n)? malloc(n*sizeof(ncl_dscalar)): NULL);
#ifdef NCL_ERRANT
    if (!ps)
      ncl_message(
        "In function ncl_dv_resize(ncl_dvector*, ncl_extent):\n"
        "  memory allocation failed.\n");
#endif /* NCL_ERRANT */
    ncl_dsubv_init(pv, ncl_dh_create(ps),
      (ncl_offset)0, n, (ncl_stride)1);
    }
  return pv; }
  
ncl_dvector*		/*           clone reconstructor	*/
(ncl_dv_resizeclone)(
    ncl_dvector* pv, const ncl_dsubvector* pu) {
  return ncl_dsubv_vcpy(
    ncl_dv_resize(pv, ncl_dsubv_extent(pu)), pu); }
  
int
(ncl_dsubv_fprintf)(FILE* stream, const ncl_dsubvector* pv, 
  int width, unsigned int precision, ncl_fmtflags flags, unsigned int columns) {
  int           characters;
  int           total = 0;
  const
  ncl_extent	n = ncl_dsubv_extent(pv);
  ncl_offset	j = 0;
  columns = (0 < columns)? columns: 4;
  for (j = 0; j < n; ++j) {
    if (0 < j) {
      characters
         = (0 == j%columns)? fprintf(stream, "\n"): fprintf(stream, " ");
      if (0 <= characters) { total += characters; } else { return -1; }
      }
    characters = ncl_dfprintf(
      stream, ncl_dsubv_get_(pv, j), width, precision, flags);
    if (0 <= characters) { total += characters; } else { return -1; }
    }
  characters = fprintf(stream, "\n");
  if (0 <= characters) { total += characters; } else { return -1; }
  return total;
  }

