dnl This is a class prototype input file for .../svmt/bin/genclass.
pushdef(`index', `ifelse($#, 0, ``index'', ``index($*)'')')dnl
pushdef(`shift', `ifelse($#, 0, ``shift'', ``shift($*)'')')dnl
/*
The C++ Scalar, Vector, Matrix and Tensor classes.
Copyright (C) 1998 E. Robert Tisdale

This file is part of The C++ Scalar, Vector, Matrix and Tensor classes.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#include<<Type><System>Matrix.h>

define(_point_,
ifelse(_type_, long double, floating,
ifelse(_type_,      double, floating,
ifelse(_type_,       float, floating, fixed))))dnl
define(TYPE, ifelse(_system_, complex, `<Type><System>', `<type>'))dnl
define(ZERO, ifelse(_system_, complex,
  `<Type><System>((<type>)0, (<type>)0)', `(<type>)0'))dnl

// Functions
<Type><System>SubMatrix&
  <Type><System>SubMatrix::reverse1(void) {
  <Type><System>SubMatrix&	M = *this;
  Extent			m = extent2();
  for (Offset i = 0; i < m; i++)
    M[i].reverse();
  return *this;
  }
<Type><System>SubMatrix&
  <Type><System>SubMatrix::reverse2(void) {
  Extent			m = extent2();
  for (Offset i = 0; i < m/2; i++)
    swap(i, m-1 - i);
  return *this;
  }

ifelse(_system_, complex,
`ifelse(floating, _point_,
`<Type>SubMatrix&
  <Type>SubMatrix::cdft(int sign) {	// real to complex dft
  <Type>SubMatrix&
		M = *this;
  Extent	n = M.extent1();
  if (1 < n) {
    <Type><System>Vector
		w(n);
		w.twiddle(sign);
    Extent	m = M.extent2();
    for (Offset i = 0; i < m; i++)
      M[i].transpose(false).rcfdft(w);
    }
  return M;
  }

<Type>SubMatrix&
  <Type>SubMatrix::rdft(int sign) {	// complex to real dft
  <Type>SubMatrix&
		M = *this;
  Extent	n = M.extent1();
  if (1 < n) {
    <Type><System>Vector
		w(n);
		w.twiddle(sign);
    Extent	m = M.extent2();
    for (Offset i = 0; i < m; i++)
      M[i].crfdft(w).transpose(true);
    }
  return M;
  }

<Type><System>SubMatrix&
  <Type><System>SubMatrix::dft(int sign) {	// complex to complex dft
  <Type><System>SubMatrix&
		M = *this;
  Extent	n = M.extent1();
  if (1 < n) {
    <Type><System>Vector
		w(n);
		w.twiddle(sign);
    Extent	m = M.extent2();
    for (Offset i = 0; i < m; i++)
      M[i].transpose().fdft(w);
    }
  return M;
  }
')dnl

', `
// M_{i,j} = j
<Type><System>SubMatrix&
  <Type><System>SubMatrix::ramp(void) {
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i].ramp();
    }
  return *this;
  }
')dnl

<Type><System>SubMatrix&
  <Type><System>SubMatrix::swap(<Type><System>SubMatrix& N) {
  <Type><System>SubMatrix	M = (*this);
  for (Offset i = 0; i < extent2(); i++) {
    <Type><System>SubVector	v = M[i];
    <Type><System>SubVector	w = N[i];
    v.swap(w);
    }
  return *this;
  }

<Type><System>SubMatrix&
  <Type><System>SubMatrix::rotate(Stride n) {
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    <Type><System>SubVector	u = M[i];
    u.rotate(n);
    }
  return *this;
  }

<Type><System>SubMatrix&
  <Type><System>SubMatrix::shift(Stride n, const TYPE& s) {
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    <Type><System>SubVector	u = M[i];
    u.shift(n, s);
    }
  return *this;
  }

ifelse(_system_, complex, `', `
const
<Type><System>Vector
  min(const <Type><System>SubMatrix& M) {
  <Type><System>Vector	result(M.extent2());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = min(M[i]);
    }
  return result;
  }

const
<Type><System>Vector
  max(const <Type><System>SubMatrix& M) {
  <Type><System>Vector	result(M.extent2());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = max(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
  min(const <Type><System>SubMatrix& M, const <Type><System>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "min(const <Type><System>SubMatrix&, const <Type><System>SubMatrix&)",
    M.extent2(), N.extent2(), M.extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>Matrix	result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = min(M[i], N[i]);
    }
  return result;
  }

const
<Type><System>Matrix
  max(const <Type><System>SubMatrix& M, const <Type><System>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "max(const <Type><System>SubMatrix&, const <Type><System>SubMatrix&)",
    M.extent2(), N.extent2(), M.extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>Matrix	result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = max(M[i], N[i]);
    }
  return result;
  }
')dnl

// M.sum() = M_{i,0} + M_{i,1} + ... + M_{i,n-1}
const
<Type><System>Vector
  <Type><System>SubMatrix::sum(void) const {
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Vector		result(extent2(), ZERO);
  for (Offset i = 0; i < extent2(); i++) {
    result[i] += M[i].sum();
    }
  return result;
  }

ifelse(_system_, complex, `
// v.dot(M) = v_{0}*M_{j,0} + v_{1}*M_{j,1} + ... + v_{n-1}*M_{j,n-1}
const
<Type><System>Vector
  <Type>SubVector::dot(const <Type><System>SubMatrix& M) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type>SubVector::dot(const <Type><System>SubMatrix&) const",
    extent(), M.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type>SubVector&		v = *this;
  <Type><System>Vector		result(M.extent2());
  for (Offset j = 0; j < M.extent2(); j++) {
    result[j] = v.dot(M[j]);
    }
  return result;
  }

// v.dot(M) = v_{0}*M_{j,0} + v_{1}*M_{j,1} + ... + v_{n-1}*M_{j,n-1}
const
<Type><System>Vector
  <Type><System>SubVector::dot(const <Type>SubMatrix& M) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::dot(const <Type>SubMatrix&) const",
    extent(), M.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Vector		result(M.extent2());
  for (Offset j = 0; j < M.extent2(); j++) {
    result[j] = v.dot(M[j]);
    }
  return result;
  }

// M.dot(N) = M_{i,0}*N_{j,0} + M_{i,1}*N_{j,1} + ... + M_{i,n-1}*N_{j,n-1}
const
<Type><System>Matrix
  <Type>SubMatrix::dot(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type>SubMatrix::dot(const <Type><System>SubMatrix&) const",
    extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type>SubMatrix&		M = *this;
  <Type><System>Matrix		result(extent2(), N.extent2());
  for (Offset i = 0; i < extent2(); i++) {
    for (Offset j = 0; j < N.extent2(); j++) {
      result[i][j] = M[i].dot(N[j]);
      }
    }
  return result;
  }

// M.dot(N) = M_{i,0}*N_{j,0} + M_{i,1}*N_{j,1} + ... + M_{i,n-1}*N_{j,n-1}
const
<Type><System>Matrix
  <Type><System>SubMatrix::dot(const <Type>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubMatrix::dot(const <Type>SubMatrix&) const",
    extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), N.extent2());
  for (Offset i = 0; i < extent2(); i++) {
    for (Offset j = 0; j < N.extent2(); j++) {
      result[i][j] = M[i].dot(N[j]);
      }
    }
  return result;
  }

// M.dot(v) = M_{i,0}*v_{0} + M_{i,1}*v_{1} + ... + M_{i,n-1}*v_{n-1}
const
<Type><System>Matrix
  <Type>SubMatrix::dot(const <Type><System>SubVector& v) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type>SubMatrix::dot(const <Type><System>SubVector&) const",
    extent1(), v.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type>SubMatrix&		M = *this;
  <Type><System>Matrix		result(extent2());
  for (Offset i = 0; i < extent2(); i++) {
    result[i][0] = M[i].dot(v);
    }
  return result;
  }

// M.dot(v) = M_{i,0}*v_{0} + M_{i,1}*v_{1} + ... + M_{i,n-1}*v_{n-1}
const
<Type><System>Matrix
  <Type><System>SubMatrix::dot(const <Type>SubVector& v) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubMatrix::dot(const <Type>SubVector&) const",
    extent1(), v.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2());
  for (Offset i = 0; i < extent2(); i++) {
    result[i][0] = M[i].dot(v);
    }
  return result;
  }
')dnl

// v.dot(M) = v_{0}*M_{j,0} + v_{1}*M_{j,1} + ... + v_{n-1}*M_{j,n-1}
const
<Type><System>Vector
  <Type><System>SubVector::dot(const <Type><System>SubMatrix& M) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::dot(const <Type><System>SubMatrix&) const",
    extent(), M.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Vector		result(M.extent2());
  for (Offset j = 0; j < M.extent2(); j++) {
    result[j] = v.dot(M[j]);
    }
  return result;
  }

// M.dot(N) = M_{i,0}*N_{j,0} + M_{i,1}*N_{j,1} + ... + M_{i,n-1}*N_{j,n-1}
const
<Type><System>Matrix
  <Type><System>SubMatrix::dot(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubMatrix::dot(const <Type><System>SubMatrix&) const",
    extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), N.extent2());
  for (Offset i = 0; i < extent2(); i++) {
    for (Offset j = 0; j < N.extent2(); j++) {
      result[i][j] = M[i].dot(N[j]);
      }
    }
  return result;
  }

// M.dot(v) = M_{i,0}*v_{0} + M_{i,1}*v_{1} + ... + M_{i,n-1}*v_{n-1}
const
<Type><System>Matrix
  <Type><System>SubMatrix::dot(const <Type><System>SubVector& v) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubMatrix::dot(const <Type><System>SubVector&) const",
    extent1(), v.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2());
  for (Offset i = 0; i < extent2(); i++) {
    result[i][0] = M[i].dot(v);
    }
  return result;
  }

// pack the diagonal and subdiagonal elements with scalar s.
<Type><System>SubMatrix&
  <Type><System>SubMatrix::pack(const TYPE& s) {
  <Type><System>SubMatrix&	M = *this;
  Extent		m = extent2();
  Extent		n = extent1();
  if (n < m) {
    for (Offset i = 0; i < n; i++)
      M[i].sub(0, 1+i, 1) = s;
    M.sub(n, m-n, 1) = s;
    }
  else {
    for (Offset i = 0; i < m; i++)
      M[i].sub(0, 1+i, 1) = s;
    }
  return M; }
// pack the diagonal and subdiagonal elements
// with the corresponding elements of submatrix N.
<Type><System>SubMatrix&
  <Type><System>SubMatrix::pack(const <Type><System>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::pack(const <Type><System>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  Extent		m = extent2();
  Extent		n = extent1();
  if (n < m) {
    for (Offset i = 0; i < n; i++)
      M[i].sub(0, 1+i, 1) = N[i].sub(0, i+1, 1);
    M.sub(n, m-n, 1) = N.sub(n, m-n, 1);
    }
  else {
    for (Offset i = 0; i < m; i++)
      M[i].sub(0, 1+i, 1) = N[i].sub(0, i+1, 1);
    }
  return M; }

// LU decomposition using Crout's method with partial pivoting.
const
offsetVector
  <Type><System>SubMatrix::lud(void) {
  <Type><System>SubMatrix&	A = *this;
  <Type><System>SubMatrix	U = A.t();
  Extent			m = A.extent2();
  Extent			n = A.extent1();
  offsetVector			p(m);
  p.ramp();				// Initialize the permutation vector.
  
  for (Offset j = 0; j < n; j++) {	// Crout's method.
    <Type><System>SubVector	u = U[j];
    if (0 < j) {
      for (Offset i = 1; i < j && i < m; i++) {
	u[i] -= u.sub(0, i, 1).dot(A[i].sub(0, i, 1));
	}
      if (j < m) {
	u.sub(j, m-j, 1) -= u.sub(0, j, 1).dot(A.sub(j, m-j, 1, 0, j, 1));
	}
      }

    if (j+1 < m) {			// Find the pivot element.
      <type>			x = (<type>)0;
      Offset			k = m;
      for (Offset i = j; i < m; i++) {
	<type>			s = abs(u[i]);
	if (x < s) {
	  x = s;
	  k = i;
	  }
	}
      if (j < k && k < m) {		// There is a non-zero pivot element.
        p.swap(j, k);
	A.swap(j, k);
	}
      if (ZERO != u[j]) {
	u.sub(j+1, m-(j+1), 1) /= u[j];
	}
      }
    }
  return p;
  }

inline
<Type><System>Matrix
  outer(const <Type><System>SubVector& v, const <Type><System>SubVector& w) {
  return v.submatrix().t().dot(w.submatrix().t()); }

// Householder transformation AQ -- post multiplication update.
<Type><System>SubMatrix&
  <Type><System>SubMatrix::hht(const <Type><System>SubVector& v) {
  <Type><System>SubMatrix&	A = *this;
  TYPE				beta = 2/v.dot(v);
  <Type><System>Vector		w = beta*v.dot(A);
  A -= outer(w, v);
  return A;
  }

ifelse(_system_, complex, `',
`// QR decomposition using Householder transformations and partial pivoting.
const
offsetVector
  <Type><System>SubMatrix::qrd(void) {
  <Type><System>SubMatrix&	A = *this;
  <Type><System>SubMatrix	T = A.t();
  Extent			m = A.extent2();
  Extent			n = A.extent1();
  offsetVector			p(m);
  p.ramp();				// Initialize the permutation vector.
  
  for (Offset j = 0; j < m && j < n; j++) {	// Householders method.
    <Type><System>SubVector	x = T[j].sub(j, m-j, 1);
    if (j < m-1) {			// Find the pivot element.
      Offset	k = j + abs(x).max();
      if (j < k) {			// There is a non-zero pivot element.
        p.swap(j, k);
	A.swap(j, k);
	}
      }
    <type>			mu = sqrt(x.dot(x));
    if (0 < mu) {
      <type>			beta = x[0] + ((x[0] < 0)? -mu: +mu);
      <Type><System>Vector	v = (1/beta)*x;
      v[0] = 1;				// Householder vector v
      T.sub(j, n-j, 1, j, m-j, 1).hht(v);
      if (j+1 < m) {
	// Pack the essential part of each Householder vector
	// into the subdiagonal elements of matrix A.
	x.sub(1, m-j-1, 1) = v.sub(1, m-j-1, 1);
	}
      }
    else
      break;
    }
  return p;
  }
')dnl

// solve v.dot(P) = w.dot(L)
const
<Type><System>Vector
  <Type><System>SubVector::pl(const offsetSubVector& p,
			      const <Type><System>SubMatrix& L) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::pl(const <Type><System>SubMatrix&) const",
    extent(), L.extent2());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Vector		w(L.extent1(), ZERO);
  Extent	n = (L.extent1() < L.extent2())? L.extent1(): L.extent2();
  for (Offset i = 0; i < n; i++) {
    w[i] = v[p[i]] - w.dot(L[i]);
    }
  return w;
  }

// solve v = w.dot(L)
const
<Type><System>Vector
  <Type><System>SubVector::l(const <Type><System>SubMatrix& L) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::l(const <Type><System>SubMatrix&) const",
    extent(), L.extent2());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Vector		w(L.extent1(), ZERO);
  Extent	n = (L.extent1() < L.extent2())? L.extent1(): L.extent2();
  for (Offset i = 0; i < n; i++) {
    w[i] = v[i] - w.dot(L[i]);
    }
  return w;
  }

// solve v = w.dot(LD)
const
<Type><System>Vector
  <Type><System>SubVector::pld(const offsetSubVector& p,
			       const <Type><System>SubMatrix& LD) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::pld(const <Type><System>SubMatrix&) const",
    extent(), LD.extent2());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Vector		w(LD.extent1(), ZERO);
  Extent	n = (LD.extent1() < LD.extent2())? LD.extent1(): LD.extent2();
  for (Offset i = 0; i < n; i++) {
    w[i] = (v[p[i]] - w.dot(LD[i]))/LD[i][i];
    }
  return w;
  }

// solve v = w.dot(LD)
const
<Type><System>Vector
  <Type><System>SubVector::ld(const <Type><System>SubMatrix& LD) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::ld(const <Type><System>SubMatrix&) const",
    extent(), LD.extent2());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Vector		w(LD.extent1(), ZERO);
  Extent	n = (LD.extent1() < LD.extent2())? LD.extent1(): LD.extent2();
  for (Offset i = 0; i < n; i++) {
    w[i] = (v[i] - w.dot(LD[i]))/LD[i][i];
    }
  return w;
  }

// solve v = w.dot(U)
const
<Type><System>Vector
  <Type><System>SubVector::u(const <Type><System>SubMatrix& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::u(const <Type><System>SubMatrix&) const",
    extent(), U.extent2());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Vector		w(U.extent1(), ZERO);
  Extent	n = (U.extent1() < U.extent2())? U.extent1(): U.extent2();
  for (Offset j = 0; j < n; j++) {
    Offset	i = n-1 - j;
    w[i] = v[i] - w.dot(U[i]);
    }
  return w;
  }

// solve v = w.dot(U)
const
<Type><System>Vector
  <Type><System>SubVector::up(const <Type><System>SubMatrix& U,
			      const offsetSubVector& p) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::up(const <Type><System>SubMatrix&) const",
    extent(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Vector		w(U.extent1(), ZERO);
  Extent	n = (U.extent1() < U.extent2())? U.extent1(): U.extent2();
  for (Offset j = 0; j < n; j++) {
    Offset	i = n-1 - j;
    w[i] = v[i] - w.dot(U[i]);
    }
  return w.permutation(p);
  }

// solve v = w.dot(DU)
const
<Type><System>Vector
  <Type><System>SubVector::du(const <Type><System>SubMatrix& DU) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::du(const <Type><System>SubMatrix&) const",
    extent(), DU.extent2());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Vector		w(DU.extent1(), ZERO);
  Extent	n = (DU.extent1() < DU.extent2())? DU.extent1(): DU.extent2();
  for (Offset j = 0; j < n; j++) {
    Offset	i = n-1 - j;
    w[i] = (v[i] - w.dot(DU[i]))/DU[i][i];
    }
  return w;
  }

// solve v = w.dot(DU)
const
<Type><System>Vector
  <Type><System>SubVector::dup(const <Type><System>SubMatrix& DU,
			      const offsetSubVector& p) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::dup(const <Type><System>SubMatrix&) const",
    extent(), DU.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Vector		w(DU.extent1(), ZERO);
  Extent	n = (DU.extent1() < DU.extent2())? DU.extent1(): DU.extent2();
  for (Offset j = 0; j < n; j++) {
    Offset	i = n-1 - j;
    w[i] = (v[i] - w.dot(DU[i]))/DU[i][i];
    }
  return w.permutation(p);
  }

// solve B = Y.dot(Q)
const
<Type><System>Matrix
  <Type><System>SubMatrix::q(const <Type><System>SubMatrix& L) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubMatrix::q(const <Type><System>SubMatrix&) const",
    extent1(), L.extent2());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix	V = L.t();
  const
  <Type><System>SubMatrix&	B = *this;
  <Type><System>Matrix		Y = B;
  Extent			l = B.extent2();
  Extent			m = L.extent2();
  Extent			n = L.extent1();
  for (Offset i = 0; i < m && i < n; i++) {
    <Type><System>Vector	v = V[i].sub(i, m-i, 1);
    v[0] = 1;
    Y.sub(0, l, 1, i, m-i, 1).hht(v);
    }
  return Y;
  }

// solve B.dot(P) = Y.dot(Q)
const
<Type><System>Matrix
  <Type><System>SubMatrix::pq(const offsetSubVector& p,
			      const <Type><System>SubMatrix& L) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubMatrix::pq(const <Type><System>SubMatrix&) const",
    extent1(), L.extent2());
#endif // SVMT_DEBUG_MODE
  return this->permutation(p).q(L);
  }

// solve b = y.dot(Q)
const
<Type><System>Vector
  <Type><System>SubVector::q(const <Type><System>SubMatrix& L) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::q(const <Type><System>SubMatrix&) const",
    extent(), L.extent2());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix	B = submatrix();
  <Type><System>Matrix		Y = B.q(L);
  return Y[0];
  }

// solve b.dot(P) = y.dot(Q)
const
<Type><System>Vector
  <Type><System>SubVector::pq(const offsetSubVector& p,
			      const <Type><System>SubMatrix& L) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::pq(const <Type><System>SubMatrix&) const",
    extent(), L.extent2());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix	B = submatrix();
  <Type><System>Matrix		Y = B.pq(p, L);
  return Y[0];
  }
const
<Type><System>Matrix
  <Type><System>SubMatrix::permutation(const offsetSubVector& p) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubMatrix::permutation(const offsetSubVector&) const",
    extent1(), p.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  Offset			m = extent2();
  Offset			n = extent1();
  <Type><System>Matrix		N(m, n);
  for (Offset i = 0; i < m; i++) {
    const
    <Type><System>SubVector	v = M[i];
    <Type><System>SubVector	w = N[i];
    for (Offset j = 0; j < n; j++) {
      w[j] = v[p[j]];
      }
    }
  return N;
  }

ifelse(_system_, complex, `
const
boolMatrix
  <Type><System>SubMatrix::eq(const <Type>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::eq(const <Type>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  boolMatrix			result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i]	= M[i].eq(N[i]);
    }
  return result;
  }

const
boolMatrix
  <Type><System>SubMatrix::ne(const <Type>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::ne(const <Type>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  boolMatrix			result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i].ne(N[i]);
    }
  return result;
  }
',

`const
boolMatrix
  <Type><System>SubMatrix::lt(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::lt(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  boolMatrix			result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i].lt(N[i]);
    }
  return result;
  }

const
boolMatrix
  <Type><System>SubMatrix::le(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::le(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  boolMatrix			result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i].le(N[i]);
    }
  return result;
  }
')dnl

const
boolMatrix
  <Type><System>SubMatrix::eq(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::eq(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  boolMatrix			result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); ++i) {
    result[i] = M[i].eq(N[i]);
    }
  return result;
  }

const
boolMatrix
  <Type><System>SubMatrix::ne(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::ne(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix	M = *this;
  boolMatrix			result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i].ne(N[i]);
    }
  return result;
  }

Extent
  <Type><System>SubMatrix::zeros(void) const {
  const
  <Type><System>SubMatrix&	M = *this;
  Extent			result = 0;
  for (Offset i = 0; i < extent2(); i++) {
    result += M[i].zeros();
    }
  return result;
  }

const
offsetVector
  <Type><System>SubMatrix::index(void) const {
  const
  <Type><System>SubMatrix&	M = *this;
  offsetVector		result(extent2()*extent1() - zeros());
  Offset		k = 0;
  for (Offset i = 0; i < M.extent2(); i++) {
    <Type><System>SubVector	v = M[i];
    for (Offset j = 0; j < M.extent1(); j++)
      if (v[j] != ZERO)
	result[k++] = i*extent1() + j;
    }
  return result;
  }

const
<Type><System>Vector
  <Type><System>SubMatrix::gather(const offsetSubVector& x) const {
  <Type><System>Vector	result(x.extent());
  for (Offset i = 0; i < extent2(); i++) {
#ifdef SVMT_DEBUG_MODE
    svmt_check_vector_index_range(
      "<Type><System>SubMatrix::gather(const offsetSubVector&) const",
      i, extent2());
#endif // SVMT_DEBUG_MODE
    }
  return result;
  }

<Type><System>SubMatrix&
  <Type><System>SubMatrix::scatter(const offsetSubVector& x,
    const <Type><System>SubVector& v) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubMatrix::scatter(\n"
    "const offsetSubVector&, const <Type><System>SubVector&)",
    x.extent(), v.extent());
#endif // SVMT_DEBUG_MODE
  return *this;
  }

const
<Type><System>Matrix
  <Type><System>SubMatrix::aside(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubMatrix::aside(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2());
#endif // SVMT_DEBUG_MODE
  <Type><System>Matrix		result(extent2(), extent1()
					      + N.extent1());
  result.sub((Offset)0, extent2(), (Stride)1,
	     (Offset)0, extent1(), (Stride)1) = *this;
  result.sub((Offset)0, extent2(), (Stride)1,
	     extent1(), N.extent1(), (Stride)1) = N;
  return result;
  }

const
<Type><System>Matrix
  <Type><System>SubMatrix::above(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubMatrix::above(const <Type><System>SubMatrix&) const",
    extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>Matrix		result(extent2()
				   + N.extent2(), extent1());
  result.sub((Offset)0, extent2(), (Stride)1) = *this;
  result.sub(extent2(), N.extent2(), (Stride)1) = N;
  return result;
  }

const
<Type><System>Matrix
  <Type><System>SubVector::kron(const <Type><System>SubMatrix& M) const {
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Matrix		result(M.extent2(), extent()*M.extent1());

  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = v.kron(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
  <Type><System>SubMatrix::kron(const <Type><System>SubMatrix& N) const {
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix	result(extent2()*N.extent2(), extent1()*N.extent1());
  for (Offset i = 0; i < extent2(); i++) {
    for (Offset j = 0; j < extent1(); j++) {
      result.sub(i*N.extent2(), N.extent2(), (Stride)1,
		 j*N.extent1(), N.extent1(), (Stride)1)
	= M[i][j]*N;
      }
    }
  return result;
  }

const
<Type><System>Matrix
  <Type><System>SubMatrix::kron(const <Type><System>SubVector& v) const {
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1()*v.extent());

  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i].kron(v);
    }
  return result;
  }

ifelse(_system_, complex, `
const
<Type><System>Matrix
  <Type>SubVector::kron(const <Type><System>SubMatrix& M) const {
  const
  <Type>SubVector&	v = *this;
  <Type><System>Matrix	result(M.extent2(), extent()*M.extent1());

  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = v.kron(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
  <Type>SubMatrix::kron(const <Type><System>SubMatrix& N) const {
  const
  <Type>SubMatrix&	M = *this;
  <Type><System>Matrix	result(extent2()*N.extent2(), extent1()*N.extent1());
  for (Offset i = 0; i < extent2(); i++) {
    for (Offset j = 0; j < extent1(); j++) {
      result.sub(i*N.extent2(), N.extent2(), (Stride)1,
		 j*N.extent1(), N.extent1(), (Stride)1)
	= M[i][j]*N;
      }
    }
  return result;
  }

const
<Type><System>Matrix
  <Type>SubMatrix::kron(const <Type><System>SubVector& v) const {
  const
  <Type>SubMatrix&	M = *this;
  <Type><System>Matrix	result(extent2(), extent1()*v.extent());

  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i].kron(v);
    }
  return result;
  }

const
<Type><System>Matrix
  <Type><System>SubVector::kron(const <Type>SubMatrix& M) const {
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Matrix		result(M.extent2(), extent()*M.extent1());

  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = v.kron(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
  <Type><System>SubMatrix::kron(const <Type>SubMatrix& N) const {
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix	result(extent2()*N.extent2(), extent1()*N.extent1());
  for (Offset i = 0; i < extent2(); i++) {
    for (Offset j = 0; j < extent1(); j++) {
      result.sub(i*N.extent2(), N.extent2(), (Stride)1,
		 j*N.extent1(), N.extent1(), (Stride)1)
	= M[i][j]*N;
      }
    }
  return result;
  }

const
<Type><System>Matrix
  <Type><System>SubMatrix::kron(const <Type>SubVector& v) const {
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1()*v.extent());

  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i].kron(v);
    }
  return result;
  }
')dnl


const
<Type><System>Matrix
  <Type><System>SubMatrix::apply(const TYPE (*f)(const TYPE&)) const {
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i].apply(f);
    }
  return result;
  }

const
<Type><System>Matrix
  <Type><System>SubMatrix::apply(      TYPE (*f)(const TYPE&)) const {
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i].apply(f);
    }
  return result;
  }

const
<Type><System>Matrix
  <Type><System>SubMatrix::apply(      TYPE (*f)(      TYPE )) const {
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i].apply(f);
    }
  return result;
  }

const
<Type><System>Matrix
    exp(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = exp(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
    log(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = log(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
   sqrt(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = sqrt(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
    cos(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = cos(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
    sin(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = sin(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
    tan(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = tan(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
   acos(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = acos(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
   asin(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = asin(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
   atan(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = atan(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
   cosh(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = cosh(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
   sinh(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = sinh(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
   tanh(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = tanh(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
  acosh(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = acosh(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
  asinh(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = asinh(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
  atanh(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = atanh(M[i]);
    }
  return result;
  }

ifelse(_system_, complex,
`// Complex Functions
const
<Type><System>Matrix
   conj(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = conj(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
  iconj(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = iconj(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
  polar(const <Type>SubMatrix& M, const <Type>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "polar(const <Type>SubMatrix&, const <Type>SubMatrix&)",
    M.extent2(), N.extent2(), M.extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = polar(M[i], N[i]);
    }
  return result;
  }

const
<Type>Matrix
   norm(const <Type><System>SubMatrix& M) {
  <Type>Matrix			result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = norm(M[i]);
    }
  return result;
  }
', `
const
<Type><System>Matrix
    sgn(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = sgn(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
    abs(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = abs(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
  floor(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix	result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = floor(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
   ceil(const <Type><System>SubMatrix& M) {
  <Type><System>Matrix	result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = ceil(M[i]);
    }
  return result;
  }

const
<Type><System>Matrix
  hypot(const <Type><System>SubMatrix& M, const <Type><System>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "hypot(const <Type><System>SubMatrix&, const <Type><System>SubMatrix&)",
    M.extent2(), N.extent2(), M.extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>Matrix		result(M.extent2(), M.extent1());
  for (Offset i = 0; i < M.extent2(); i++) {
    result[i] = hypot(M[i], N[i]);
    }
  return result;
  }

const
<Type><System>Matrix
  atan2(const <Type><System>SubMatrix& N, const <Type><System>SubMatrix& M) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "atan2(const <Type><System>SubMatrix&, const <Type><System>SubMatrix&)",
    N.extent2(), M.extent2(), N.extent1(), M.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>Matrix		result(N.extent2(), N.extent1());
  for (Offset i = 0; i < N.extent2(); i++) {
    result[i] = atan2(N[i], M[i]);
    }
  return result;
  }
')dnl

// Operators
ifelse(floating, _point_, `',
`// complement
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator ~(void) const {
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = ~M[i];
    }
  return result;
  }
')dnl

// minus
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator -(void) const {
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = -M[i];
    }
  return result;
  }

ifelse(_system_, complex, `
// multiply
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator *(const <Type>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator *(const <Type>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i]*N[i];
    }
  return result;
  }

// divide
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator /(const <Type>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator /(const <Type>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i]/N[i];
    }
  return result;
  }

// add
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator +(const <Type>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator +(const <Type>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i] + N[i];
    }
  return result;
  }

// subtract
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator -(const <Type>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator -(const <Type>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i] - N[i];
    }
  return result;
  }

// divide
const
<Type><System>Matrix
  <Type>SubMatrix::operator /(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type>SubMatrix::operator /(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type>SubMatrix&		M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i]/N[i];
    }
  return result;
  }

// subtract
const
<Type><System>Matrix
  <Type>SubMatrix::operator -(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type>SubMatrix::operator -(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type>SubMatrix&		M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i] - N[i];
    }
  return result;
  }
')dnl

// multiply
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator *(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator *(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i]*N[i];
    }
  return result;
  }

// divide
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator /(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator /(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i]/N[i];
    }
  return result;
  }

ifelse(floating, _point_, `',
`// modulo (remainder)
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator %(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator %(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i]%N[i];
    }
  return result;
  }
')dnl

// add
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator +(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator +(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i] + N[i];
    }
  return result;
  }

// subtract
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator -(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator -(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i] - N[i];
    }
  return result;
  }

ifelse(floating, _point_, `',
`// shift left
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator <<(int n) const {
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i] << n;
    }
  return result;
  }

// shift right
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator >>(int n) const {
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i] >> n;
    }
  return result;
  }
')dnl

std::istream&	operator >>(std::istream& is,       <Type><System>SubMatrix& M) {
  for (Offset i = 0; i < M.extent2(); i++) {
    <Type><System>SubVector	v = M[i];
    if (!(is >> v)) break;
    }
  return is;
  }

std::ostream&	operator <<(std::ostream& os, const <Type><System>SubMatrix& M) {
  int			width = os.width();
  for (Offset i = 0; i < M.extent2(); i++) {
    os << std::setw(width) << M[i];
    }
  return os;
  }

ifelse(_system_, complex, `
// equal
bool	<Type><System>SubMatrix::operator ==(const <Type>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator ==(const <Type>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  Offset			i = (Offset)0;
  while (i < extent2() && M[i] == N[i]) {
   ++i;
   }
  return i == extent2();
  }

// not equal
bool	<Type><System>SubMatrix::operator !=(const <Type>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator !=(const <Type>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  Offset			i = (Offset)0;
  while (i < extent2() && M[i] != N[i]) {
   ++i;
   }
  return i == extent2();
  }
',

`// less than
bool	<Type><System>SubMatrix::operator < (
  const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator < (const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  Offset			i = (Offset)0;
  while (i < extent2() && M[i] <  N[i]) {
   ++i;
   }
  return i == extent2();
  }

// less than or equal
bool	<Type><System>SubMatrix::operator <=(
  const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator <=(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  Offset			i = (Offset)0;
  while (i < extent2() && M[i] <= N[i]) {
   ++i;
   }
  return i == extent2();
  }
')dnl

// equal
bool	<Type><System>SubMatrix::operator ==(
  const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator ==(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  Offset			i = (Offset)0;
  while (i < extent2() && M[i] == N[i]) {
   ++i;
   }
  return i == extent2();
  }

// not equal
bool	<Type><System>SubMatrix::operator !=(
  const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator !=(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  Offset			i = (Offset)0;
  while (i < extent2() && M[i] != N[i]) {
   ++i;
   }
  return i == extent2();
  }

ifelse(floating, _point_, `',
`// bitwise AND
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator &(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator &(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i]&N[i];
    }
  return result;
  }

// bitwise exclusive OR
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator ^(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator ^(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i]^N[i];
    }
  return result;
  }

// bitwise inclusive OR
const
<Type><System>Matrix
  <Type><System>SubMatrix::operator |(const <Type><System>SubMatrix& N) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator |(const <Type><System>SubMatrix&) const",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubMatrix&	M = *this;
  <Type><System>Matrix		result(extent2(), extent1());
  for (Offset i = 0; i < extent2(); i++) {
    result[i] = M[i]|N[i];
    }
  return result;
  }
')dnl

ifelse(_system_, complex, `
// simple assignment
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator  =(const <Type>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator  =(const <Type>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] = N[i];
    }
  return *this;
  }

// multiply and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator *=(const <Type>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator *=(const <Type>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] *= N[i];
    }
  return *this;
  }

// divide and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator /=(const <Type>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator /=(const <Type>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] /= N[i];
    }
  return *this;
  }

// add and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator +=(const <Type>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator +=(const <Type>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] += N[i];
    }
  return *this;
  }

// subtract and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator -=(const <Type>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator -=(const <Type>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] -= N[i];
    }
  return *this;
  }
')dnl

// simple assignment
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator  =(const <Type><System>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator  =(const <Type><System>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] = N[i];
    }
  return *this;
  }

// multiply and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator *=(const <Type><System>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator *=(const <Type><System>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] *= N[i];
    }
  return *this;
  }

// divide and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator /=(const <Type><System>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator /=(const <Type><System>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] /= N[i];
    }
  return *this;
  }

ifelse(floating, _point_, `',
`// modulo and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator %=(const <Type><System>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator %=(const <Type><System>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] %= N[i];
    }
  return *this;
  }
')dnl

// add and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator +=(const <Type><System>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator +=(const <Type><System>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] += N[i];
    }
  return *this;
  }

// subtract and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator -=(const <Type><System>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator -=(const <Type><System>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] -= N[i];
    }
  return *this;
  }

ifelse(floating, _point_, `',
`// shift left and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator <<=(int n) {
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] <<= n;
    }
  return *this;
  }

// shift right and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator >>=(int n) {
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] >>= n;
    }
  return *this;
  }

// AND and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator &=(const <Type><System>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator &=(const <Type><System>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] &= N[i];
    }
  return *this;
  }

// exclusive OR and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator ^=(const <Type><System>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator ^=(const <Type><System>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] ^= N[i];
    }
  return *this;
  }

// inclusive OR and assign
<Type><System>SubMatrix&
  <Type><System>SubMatrix::operator |=(const <Type><System>SubMatrix& N) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubMatrix::operator |=(const <Type><System>SubMatrix&)",
    extent2(), N.extent2(), extent1(), N.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubMatrix&	M = *this;
  for (Offset i = 0; i < extent2(); i++) {
    M[i] |= N[i];
    }
  return *this;
  }
')dnl
popdef(`index')dnl
popdef(`shift')dnl


/*
// LU decomposition with partial pivoting
const
offsetVector
  <Type><System>SubMatrix::lud(void) {
  const
  <Type><System>SubMatrix&	M = *this;
  Extent			m = M.extent2();
  Extent			n = M.extent1();
  Extent			l = (m < n)? m: n;
  offsetVector			p(m);
  p.ramp();			// Initialize the permutation vector.
  
  for (Offset h = 0; h+1 < l; h++) {
    // First, find the pivot element.
    Offset			k = m;
    <type>			x = (<type>)0;
    for (Offset i = h; i < m; i++) {
      <type>			s = abs(M[p[i]][h]);
      if (x < s) {
	x = s;
	k = i;
	}
      }

    if (k < m) {		// There is a non-zero pivot element.
      if (h < k) {
        p.swap(h, k);
	}
      <Type><System>SubVector	v = M[p[h]];
      for (Offset i = h+1; i < m; i++) {
	<Type><System>SubVector	w = M[p[i]];
	TYPE			t = -w[h]/v[h];
	w += t*v;
	w[h] = t;
	}
      }
    }
  return p;
  }
*/
