dnl This is a class prototype input file for .../svmt/bin/genclass.
pushdef(`index', `ifelse($#, 0, ``index'', ``index($*)'')')dnl
pushdef(`shift', `ifelse($#, 0, ``shift'', ``shift($*)'')')dnl
/*
The C++ Digital Signal Processing classes.
Copyright (C) 1998 E. Robert Tisdale

This file is part of The C++ Scalar, Vector, Matrix and Tensor classes.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#include<<Type><System>Signal.h>

define(_point_,
ifelse(_type_, long double, floating,
ifelse(_type_,      double, floating,
ifelse(_type_,       float, floating, fixed))))dnl
define(TYPE, ifelse(_system_, complex, `<Type><System>', `<type>'))dnl
define(ZERO, ifelse(_system_, complex,
  `<Type><System>((<type>)0, (<type>)0)', `(<type>)0'))dnl
ifelse(_system_, complex,
`ifelse(floating, _point_, `
// 1 Dimensional Complex to Complex Direct Discrete Fourier Transform
// Functions
<Type>1DCCDDFT&	<Type>1DCCDDFT::resize(Extent n, Extent invocations) {
  resize(n).twiddle(-1);
  return *this; }

// Operators
<Type><System>SubVector&
		<Type>1DCCDDFT::operator ()(<Type><System>SubVector& v) const {
  v.transpose().fdft(*this);
  return v; }
<Type><System>SubMatrix&
		<Type>1DCCDDFT::operator ()(<Type><System>SubMatrix& M) const {
  Extent	m = M.extent2();
  for (Offset i = 0; i < m; i++)
    M[i].transpose().fdft(*this);
  return M; }
<Type><System>SubTensor&
		<Type>1DCCDDFT::operator ()(<Type><System>SubTensor& T) const {
  Extent	m = T.extent2();
  Extent	l = T.extent3();
  for (Offset h = 0; h < l; h++)
    for (Offset i = 0; i < m; i++)
      T[h][i].transpose().fdft(*this);
  return T; }

// 1 Dimensional Complex Direct Discrete Fourier Transform from Complex
// Functions
<Type>1DCDDFTC&	<Type>1DCDDFTC::resize(Extent n, Extent invocations) {
  resize(n).twiddle(-1);
  return *this; }

// Operators
<Type><System>Vector
	<Type>1DCDDFTC::operator ()(const <Type><System>SubVector& v) const {
  <Type><System>Vector
		u = v;
  u.transpose().fdft(*this);
  return u; }
<Type><System>Matrix
	<Type>1DCDDFTC::operator ()(const <Type><System>SubMatrix& M) const {
  <Type><System>Matrix
		L = M;
  Extent	m = L.extent2();
  for (Offset i = 0; i < m; i++)
    L[i].transpose().fdft(*this);
  return L; }
<Type><System>Tensor
	<Type>1DCDDFTC::operator ()(const <Type><System>SubTensor& T) const {
  <Type><System>Tensor
		S = T;
  Extent	m = S.extent2();
  Extent	l = S.extent3();
  for (Offset h = 0; h < l; h++)
    for (Offset i = 0; i < m; i++)
      S[h][i].transpose().fdft(*this);
  return S; }

// 1 Dimensional Complex to Complex Inverse Discrete Fourier Transform
// Functions
<Type>1DCCIDFT&	<Type>1DCCIDFT::resize(Extent n, Extent invocations) {
  resize(n).twiddle(+1);
  return *this; }

// Operators
<Type><System>SubVector&
		<Type>1DCCIDFT::operator ()(<Type><System>SubVector& v) const {
  v.transpose().fdft(*this);
  return v; }
<Type><System>SubMatrix&
		<Type>1DCCIDFT::operator ()(<Type><System>SubMatrix& M) const {
  Extent	m = M.extent2();
  for (Offset i = 0; i < m; i++)
    M[i].transpose().fdft(*this);
  return M; }
<Type><System>SubTensor&
		<Type>1DCCIDFT::operator ()(<Type><System>SubTensor& T) const {
  Extent	m = T.extent2();
  Extent	l = T.extent3();
  for (Offset h = 0; h < l; h++)
    for (Offset i = 0; i < m; i++)
      T[h][i].transpose().fdft(*this);
  return T; }

// 1 Dimensional Complex Inverse Discrete Fourier Transform from Complex
// Functions
<Type>1DCIDFTC&	<Type>1DCIDFTC::resize(Extent n, Extent invocations) {
  resize(n).twiddle(+1);
  return *this; }

// Operators
<Type><System>Vector
	<Type>1DCIDFTC::operator ()(const <Type><System>SubVector& v) const {
  <Type><System>Vector
		u = v;
  u.transpose().fdft(*this);
  return u; }
<Type><System>Matrix
	<Type>1DCIDFTC::operator ()(const <Type><System>SubMatrix& M) const {
  <Type><System>Matrix
		L = M;
  Extent	m = L.extent2();
  for (Offset i = 0; i < m; i++)
    L[i].transpose().fdft(*this);
  return L; }
<Type><System>Tensor
	<Type>1DCIDFTC::operator ()(const <Type><System>SubTensor& T) const {
  <Type><System>Tensor
		S = T;
  Extent	m = S.extent2();
  Extent	l = S.extent3();
  for (Offset h = 0; h < l; h++)
    for (Offset i = 0; i < m; i++)
      S[h][i].transpose().fdft(*this);
  return S; }

// 1 Dimensional Real to Complex Direct Discrete Fourier Transform
// Functions
<Type>1DRCDDFT&	<Type>1DRCDDFT::resize(Extent n, Extent invocations) {
  resize(n).twiddle(-1);
  return *this; }

// Operators
<Type>SubVector&
		<Type>1DRCDDFT::operator ()(<Type>SubVector& v) const {
  v.transpose(false).rcfdft(*this);
  return v; }
<Type>SubMatrix&
		<Type>1DRCDDFT::operator ()(<Type>SubMatrix& M) const {
  Extent	m = M.extent2();
  for (Offset i = 0; i < m; i++)
    M[i].transpose(false).rcfdft(*this);
  return M; }
<Type>SubTensor&
		<Type>1DRCDDFT::operator ()(<Type>SubTensor& T) const {
  Extent	m = T.extent2();
  Extent	l = T.extent3();
  for (Offset h = 0; h < l; h++)
    for (Offset i = 0; i < m; i++)
      T[h][i].transpose(false).rcfdft(*this);
  return T; }

// 1 Dimensional Complex Direct Discrete Fourier Transform from Real
// Functions
<Type>1DCDDFTR&	<Type>1DCDDFTR::resize(Extent n, Extent invocations) {
  resize(n).twiddle(-1);
  return *this; }

// Operators
<Type>Vector
	<Type>1DCDDFTR::operator ()(const <Type>SubVector& v) const {
  <Type>Vector	u = v;
  u.transpose(false).rcfdft(*this);
  return u; }
<Type>Matrix
	<Type>1DCDDFTR::operator ()(const <Type>SubMatrix& M) const {
  <Type>Matrix	L = M;
  Extent	m = L.extent2();
  for (Offset i = 0; i < m; i++)
    L[i].transpose(false).rcfdft(*this);
  return L; }
<Type>Tensor
	<Type>1DCDDFTR::operator ()(const <Type>SubTensor& T) const {
  <Type>Tensor	S = T;
  Extent	m = S.extent2();
  Extent	l = S.extent3();
  for (Offset h = 0; h < l; h++)
    for (Offset i = 0; i < m; i++)
      S[h][i].transpose(false).rcfdft(*this);
  return S; }

// 1 Dimensional Real to Complex Inverse Discrete Fourier Transform
// Functions
<Type>1DRCIDFT&	<Type>1DRCIDFT::resize(Extent n, Extent invocations) {
  resize(n).twiddle(+1);
  return *this; }

// Operators
<Type>SubVector&
		<Type>1DRCIDFT::operator ()(<Type>SubVector& v) const {
  v.transpose(false).rcfdft(*this);
  return v; }
<Type>SubMatrix&
		<Type>1DRCIDFT::operator ()(<Type>SubMatrix& M) const {
  Extent	m = M.extent2();
  for (Offset i = 0; i < m; i++)
    M[i].transpose(false).rcfdft(*this);
  return M; }
<Type>SubTensor&
		<Type>1DRCIDFT::operator ()(<Type>SubTensor& T) const {
  Extent	m = T.extent2();
  Extent	l = T.extent3();
  for (Offset h = 0; h < l; h++)
    for (Offset i = 0; i < m; i++)
      T[h][i].transpose(false).rcfdft(*this);
  return T; }

// 1 Dimensional Complex Inverse Discrete Fourier Transform from Real
// Functions
<Type>1DCIDFTR&	<Type>1DCIDFTR::resize(Extent n, Extent invocations) {
  resize(n).twiddle(+1);
  return *this; }

// Operators
<Type>Vector
	<Type>1DCIDFTR::operator ()(const <Type>SubVector& v) const {
  <Type>Vector	u = v;
  u.transpose(false).rcfdft(*this);
  return u; }
<Type>Matrix
	<Type>1DCIDFTR::operator ()(const <Type>SubMatrix& M) const {
  <Type>Matrix	L = M;
  Extent	m = L.extent2();
  for (Offset i = 0; i < m; i++)
    L[i].transpose(false).rcfdft(*this);
  return L; }
<Type>Tensor
	<Type>1DCIDFTR::operator ()(const <Type>SubTensor& T) const {
  <Type>Tensor	S = T;
  Extent	m = S.extent2();
  Extent	l = S.extent3();
  for (Offset h = 0; h < l; h++)
    for (Offset i = 0; i < m; i++)
      S[h][i].transpose(false).rcfdft(*this);
  return S; }

// 1 Dimensional Complex to Real Direct Discrete Fourier Transform
// Functions
<Type>1DCRDDFT&	<Type>1DCRDDFT::resize(Extent n, Extent invocations) {
  resize(n).twiddle(-1);
  return *this; }

// Operators
<Type>SubVector&
		<Type>1DCRDDFT::operator ()(<Type>SubVector& v) const {
  v.crfdft(*this).transpose(true);
  return v; }
<Type>SubMatrix&
		<Type>1DCRDDFT::operator ()(<Type>SubMatrix& M) const {
  Extent	m = M.extent2();
  for (Offset i = 0; i < m; i++)
    M[i].crfdft(*this).transpose(true);
  return M; }
<Type>SubTensor&
		<Type>1DCRDDFT::operator ()(<Type>SubTensor& T) const {
  Extent	m = T.extent2();
  Extent	l = T.extent3();
  for (Offset h = 0; h < l; h++)
    for (Offset i = 0; i < m; i++)
      T[h][i].crfdft(*this).transpose(true);
  return T; }

// 1 Dimensional Real Direct Discrete Fourier Transform from Complex
// Functions
<Type>1DRDDFTC&	<Type>1DRDDFTC::resize(Extent n, Extent invocations) {
  resize(n).twiddle(-1);
  return *this; }

// Operators
<Type>Vector
	<Type>1DRDDFTC::operator ()(const <Type>SubVector& v) const {
  <Type>Vector	u = v;
  u.crfdft(*this).transpose(true);
  return u; }
<Type>Matrix
	<Type>1DRDDFTC::operator ()(const <Type>SubMatrix& M) const {
  <Type>Matrix	L = M;
  Extent	m = L.extent2();
  for (Offset i = 0; i < m; i++)
    L[i].crfdft(*this).transpose(true);
  return L; }
<Type>Tensor
	<Type>1DRDDFTC::operator ()(const <Type>SubTensor& T) const {
  <Type>Tensor	S = T;
  Extent	m = S.extent2();
  Extent	l = S.extent3();
  for (Offset h = 0; h < l; h++)
    for (Offset i = 0; i < m; i++)
      S[h][i].crfdft(*this).transpose(true);
  return S; }

// 1 Dimensional Complex to Real Inverse Discrete Fourier Transform
// Functions
<Type>1DCRIDFT&	<Type>1DCRIDFT::resize(Extent n, Extent invocations) {
  resize(n).twiddle(+1);
  return *this; }

// Operators
<Type>SubVector&
		<Type>1DCRIDFT::operator ()(<Type>SubVector& v) const {
  v.crfdft(*this).transpose(true);
  return v; }
<Type>SubMatrix&
		<Type>1DCRIDFT::operator ()(<Type>SubMatrix& M) const {
  Extent	m = M.extent2();
  for (Offset i = 0; i < m; i++)
    M[i].crfdft(*this).transpose(true);
  return M; }
<Type>SubTensor&
		<Type>1DCRIDFT::operator ()(<Type>SubTensor& T) const {
  Extent	m = T.extent2();
  Extent	l = T.extent3();
  for (Offset h = 0; h < l; h++)
    for (Offset i = 0; i < m; i++)
      T[h][i].crfdft(*this).transpose(true);
  return T; }

// 1 Dimensional Real Inverse Discrete Fourier Transform from Complex
// Functions
<Type>1DRIDFTC&	<Type>1DRIDFTC::resize(Extent n, Extent invocations) {
  resize(n).twiddle(+1);
  return *this; }

// Operators
<Type>Vector
	<Type>1DRIDFTC::operator ()(const <Type>SubVector& v) const {
  <Type>Vector	u = v;
  u.crfdft(*this).transpose(true);
  return u; }
<Type>Matrix
	<Type>1DRIDFTC::operator ()(const <Type>SubMatrix& M) const {
  <Type>Matrix	L = M;
  Extent	m = L.extent2();
  for (Offset i = 0; i < m; i++)
    L[i].crfdft(*this).transpose(true);
  return L; }
<Type>Tensor
	<Type>1DRIDFTC::operator ()(const <Type>SubTensor& T) const {
  <Type>Tensor	S = T;
  Extent	m = S.extent2();
  Extent	l = S.extent3();
  for (Offset h = 0; h < l; h++)
    for (Offset i = 0; i < m; i++)
      S[h][i].crfdft(*this).transpose(true);
  return S; }
')dnl
',
`ifelse(floating, _point_, `
const
<Type><System>Vector
	<Type><System>Blackman(Extent n) {
  <Type><System>Vector
		w(n);
  if (0 < n) {
    if (1 < n) {
      w.ramp() *= M_PI/(n-1);		// w_j = j*pi/(n-1)
      w  = sin(w);			// w_j = sin(j*pi/(n-1))
      w *= w;				// w_j = sin^2(j*pi/(n-1))
      w *= ((TYPE)16/(TYPE)25)*(((TYPE)9/(TYPE)16) + w);
		// w_j = (16/25)*(9/16 + sin^2(j*pi/(n-1)))*sin^2(j*pi/(n-1))
      }
    else {
      w[0] = (TYPE)1;
      }
    }
  return w;
  }
const
<Type><System>Vector
	<Type><System>Chebyshev(Extent n, TYPE ripple) {
  <Type><System>Vector
		w(n);
  if (0 < n) {
    if (1 < n) {
      TYPE	delta = exp((-log((TYPE)10)/(TYPE)20)*ripple);
      TYPE	tau   = ((TYPE)1 + delta)/delta;
      TYPE	sigma = cosh(acosh(tau)/(n-1));
      <Type><System>SubVector		// real and imaginary parts of vector w
		r = w.even(), i = w.odd();
		r.ramp() *= M_PI/n;	// r_j = pi*j/n
		r = sigma*cos(r);	// r_j = sigma*cos(pi*j/n)
		r = 2*r*r - 1;		// r_j = 2*(sigma*cos(pi*j/n))^{2} - 1
      for (Offset j = 0; 2*j < n; j++) {
	if (1 < r[j])
	  r[j] = cosh(0.5*(n-1)*acosh(r[j]));
	else
	  r[j] = cos(0.5*(n-1)*acos(r[j]));
	}
      if (1&n)				// n is odd
	i = (TYPE)0;
      else {				// n is even
	i.ramp() *= M_PI/n;
	for (Offset j = 0; (j << 1) < n; j++) {
	  TYPE	x = r[j];
	  TYPE	y = i[j];
		r[j] = x*cos(y);
		i[j] = x*sin(y);
	  }
	}
      w.rdft(+1);
      w /= w[0];
      w.rotate((int)((n+1)/2));
      }
    else {
      w[0] = (TYPE)1;
      }
    }
  return w;
  }
const
<Type><System>Vector
	<Type><System>Hamming(Extent n) {
  <Type><System>Vector
		w(n);
  if (0 < n) {
    if (1 < n) {
      w.ramp() *= M_PI/(n-1);		// w_j = j*pi/(n-1)
      w  = sin(w);			// w_j = sin(j*pi/(n-1))
      w *= (TYPE)0.92*w;		// w_j = 0.92*sin^2(j*pi/(n-1))
      w += (TYPE)0.08;			// w_j = 0.08 + 0.92*sin^2(j*pi/(n-1))
      }
    else {
      w[0] = (TYPE)1;
      }
    }
  return w;
  }
const
<Type><System>Vector
	<Type><System>Hanning(Extent n) {
  <Type><System>Vector
		w(n);
  if (0 < n) {
    if (1 < n) {
      w.ramp() += (TYPE)1;		// w_j = j+1
      w *= M_PI/(n+1);			// w_j = (j+1)*pi/(n+1)
      w  = sin(w);			// w_j = sin((j+1)*pi/(n+1))
      w *= w;				// w_j = sin^2((j+1)*pi/(n+1))
      }
    else {
      w[0] = (<type>)1;
      }
    }
  return w;
  }
inline
TYPE I_0(TYPE x) {			// Modified Bessel Function
  // This approximation was lifted from Numerical Recipes in C page 237.
  TYPE		ans = 0.0;
  TYPE		ax = fabs((TYPE)x);
  if (ax < 3.75) {
    TYPE	y = (TYPE)x/3.75;
    y *= y;
    ans = 1.0 + y*(+3.5156229 + y*(+3.0899424 + y*(+1.2067492
	      + y*(+0.2659732 + y*(+0.0360768 + y*(+0.0045813))))));
    }
  else {
    TYPE	y = 3.75/ax;
    ans = (exp(ax)/sqrt(ax))*(+0.39894228 + y*(+0.01328592 + y*(+0.00225319
	+ y*(-0.00157565 + y*(+0.00916281 + y*(-0.02057706 + y*(+0.02635537
	+ y*(-0.01647633 + y*(+0.00392377)))))))));
    }
  return ans;
  }
inline
const
<Type><System>Vector
   I_0(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,  I_0(h.get(o)));
    o += s;
    }
  return result;
  }
const
<Type><System>Vector
	<Type><System>Kaiser(Extent n, TYPE beta) {
  <Type><System>Vector
		w(n);
  if (0 < n) {
    if (1 < n) {
      w.ramp();				// w_j = j
      w *= (TYPE)(n-1) - w;		// w_j = j*(n-1-j)
      w  = I_0((2*beta/(n-1))*sqrt(w))/I_0(beta);
      }
    else {
      w[0] = (TYPE)1;
      }
    }
  return w;
  }
')dnl
')dnl
popdef(`index')dnl
popdef(`shift')dnl

