dnl This is a class prototype input file for .../svmt/bin/genclass.
pushdef(`index', `ifelse($#, 0, ``index'', ``index($*)'')')dnl
pushdef(`shift', `ifelse($#, 0, ``shift'', ``shift($*)'')')dnl
/*
The C++ Scalar, Vector, Matrix and Tensor classes.
Copyright (C) 1998 E. Robert Tisdale

This file is part of The C++ Scalar, Vector, Matrix and Tensor classes.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#include<<Type><System>Tensor.h>

define(_point_,
ifelse(_type_, long double, floating,
ifelse(_type_,      double, floating,
ifelse(_type_,       float, floating, fixed))))dnl
define(TYPE, ifelse(_system_, complex, `<Type><System>', `<type>'))dnl
define(ZERO, ifelse(_system_, complex,
  `<Type><System>((<type>)0, (<type>)0)', `(<type>)0'))dnl

// Functions
<Type><System>SubTensor&
  <Type><System>SubTensor::reverse1(void) {
  <Type><System>SubTensor&      T = *this;
  Extent                        l = extent3();
  for (Offset h = 0; h < l; h++)
    T[h].reverse1();
  return *this;
  }
<Type><System>SubTensor&
  <Type><System>SubTensor::reverse2(void) {
  <Type><System>SubTensor&      T = *this;
  Extent                        l = extent3();
  for (Offset h = 0; h < l; h++)
    T[h].reverse2();
  return *this;
  }
<Type><System>SubTensor&
  <Type><System>SubTensor::reverse3(void) {
  Extent                        l = extent3();
  for (Offset h = 0; h < l/2; h++)
    swap(h, l-1 - h);
  return *this;
  }

ifelse(_system_, complex,
`ifelse(floating, _point_,
`<Type>SubTensor&
  <Type>SubTensor::cdft(int sign) {	// real to complex dft
  <Type>SubTensor&
		T = *this;
  Extent	n = T.extent1();
  if (1 < n) {
    <Type><System>Vector
		w(n);
		w.twiddle(sign);
    Extent	m = T.extent2();
    Extent	l = T.extent3();
    for (Offset h = 0; h < l; h++)
      for (Offset i = 0; i < m; i++)
	T[h][i].transpose(false).rcfdft(w);
    }
  return T;
  }

<Type>SubTensor&
  <Type>SubTensor::rdft(int sign) {	// complex to real dft
  <Type>SubTensor&
		T = *this;
  Extent	n = T.extent1();
  if (1 < n) {
    <Type><System>Vector
		w(n);
		w.twiddle(sign);
    Extent	m = T.extent2();
    Extent	l = T.extent3();
    for (Offset h = 0; h < l; h++)
      for (Offset i = 0; i < m; i++)
	T[h][i].crfdft(w).transpose(true);
    }
  return T;
  }

<Type><System>SubTensor&
  <Type><System>SubTensor::dft(int sign) {	// complex to complex dft
  <Type><System>SubTensor&
		T = *this;
  Extent	n = T.extent1();
  if (1 < n) {
    <Type><System>Vector
		w(n);
		w.twiddle(sign);
    Extent	m = T.extent2();
    Extent	l = T.extent3();
    for (Offset h = 0; h < l; h++)
      for (Offset i = 0; i < m; i++)
	T[h][i].transpose().fdft(w);
    }
  return T;
  }
')dnl
', `
// T_{h,i,j} = j
<Type><System>SubTensor&
  <Type><System>SubTensor::ramp(void) {
  <Type><System>SubTensor&	T = *this;
  Extent			l = extent3();
  for (Offset h = 0; h < l; h++) {
    T[h].ramp();
    }
  return *this;
  }
')dnl

<Type><System>SubTensor&
  <Type><System>SubTensor::swap(<Type><System>SubTensor& U) {
  <Type><System>SubTensor	T = (*this);
  Extent			l = extent3();
  for (Offset h = 0; h < l; h++) {
    <Type><System>SubMatrix	M = T[h];
    <Type><System>SubMatrix	N = U[h];
    M.swap(N);
    }
  return *this;
  }

<Type><System>SubTensor&
  <Type><System>SubTensor::rotate(Stride n) {
  <Type><System>SubTensor&	T = *this;
  Extent			l = extent3();
  for (Offset h = 0; h < l; h++) {
    <Type><System>SubMatrix	M = T[h];
    M.rotate(n);
    }
  return *this;
  }

<Type><System>SubTensor&
  <Type><System>SubTensor::shift(Stride n, const TYPE& s) {
  <Type><System>SubTensor&	T = *this;
  Extent			l = extent3();
  for (Offset h = 0; h < l; h++) {
    <Type><System>SubMatrix	M = T[h];
    M.shift(n, s);
    }
  return *this;
  }

ifelse(_system_, complex, `', `
const
<Type><System>Matrix
  min(const <Type><System>SubTensor& T) {
  <Type><System>Matrix	result(T.extent3(), T.extent2());
  Extent			l = T.extent3();
  for (Offset h = 0; h < l; h++) {
    result[h] = min(T[h]);
    }
  return result;
  }

const
<Type><System>Matrix
  max(const <Type><System>SubTensor& T) {
  <Type><System>Matrix	result(T.extent3(), T.extent2());
  Extent			l = T.extent3();
  for (Offset h = 0; h < l; h++) {
    result[h] = max(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
  min(const <Type><System>SubTensor& T, const <Type><System>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "min(const <Type><System>SubTensor&, const <Type><System>SubTensor&)",
  T.extent3(), U.extent3(), T.extent2(), U.extent2(), T.extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>Tensor	result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = min(T[h], U[h]);
    }
  return result;
  }

const
<Type><System>Tensor
  max(const <Type><System>SubTensor& T, const <Type><System>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "max(const <Type><System>SubTensor&, const <Type><System>SubTensor&)",
  T.extent3(), U.extent3(), T.extent2(), U.extent2(), T.extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>Tensor	result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = max(T[h], U[h]);
    }
  return result;
  }
')dnl

// T.sum() = T_{h,i,0} + T_{h,i,1} + ... + T_{h,i,n-1}
const
<Type><System>Matrix
  <Type><System>SubTensor::sum(void) const {
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Matrix		result(extent3(), extent2(), ZERO);
  for (Offset h = 0; h < extent3(); h++) {
    result[h] += T[h].sum();
    }
  return result;
  }
const
<Type><System>Tensor
  <Type><System>SubTensor::permutation(const offsetSubVector& p) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubTensor::permutation(const offsetSubVector&) const",
    extent1(), p.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  Offset			l = extent3();
  Offset			m = extent2();
  Offset			n = extent1();
  <Type><System>Tensor		U(l, m, n);
  for (Offset h = 0; h < l; h++) {
    const
    <Type><System>SubMatrix	M = T[h];
    <Type><System>SubMatrix	N = U[h];
    for (Offset i = 0; i < m; i++) {
      const
      <Type><System>SubVector	v = M[i];
      <Type><System>SubVector	w = N[i];
      for (Offset j = 0; j < n; j++) {
	w[j] = v[p[j]];
	}
      }
    }
  return U;
  }
ifelse(_system_, complex, `
const
boolTensor
  <Type><System>SubTensor::eq(const <Type>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::eq(const <Type>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  boolTensor			result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h]	= T[h].eq(U[h]);
    }
  return result;
  }

const
boolTensor
  <Type><System>SubTensor::ne(const <Type>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::ne(const <Type>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  boolTensor			result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h].ne(U[h]);
    }
  return result;
  }
',

`const
boolTensor
  <Type><System>SubTensor::lt(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::lt(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  boolTensor			result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h].lt(U[h]);
    }
  return result;
  }

const
boolTensor
  <Type><System>SubTensor::le(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::le(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  boolTensor			result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h].le(U[h]);
    }
  return result;
  }
')dnl

const
boolTensor
  <Type><System>SubTensor::eq(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::eq(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  boolTensor			result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); ++h) {
    result[h] = T[h].eq(U[h]);
    }
  return result;
  }

const
boolTensor
  <Type><System>SubTensor::ne(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::ne(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor	T = *this;
  boolTensor			result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h].ne(U[h]);
    }
  return result;
  }

Extent
  <Type><System>SubTensor::zeros(void) const {
  const
  <Type><System>SubTensor&	T = *this;
  Extent			result = 0;
  for (Offset h = 0; h < extent3(); h++) {
    result += T[h].zeros();
    }
  return result;
  }
/*
const
offsetVector
  <Type><System>SubTensor::index(void) const {
  offsetVector		result(extent3()*extent2()*extent1() - zeros());
  Offset		k = 0;
  for (Offset h = 0; h < extent3(); h++) {
    }
  return result;
  }

const
<Type><System>Vector
  <Type><System>SubTensor::gather(const offsetSubVector& x) const {
  <Type><System>Vector	result(x.extent());
  for (Offset h = 0; h < extent3(); h++) {
#ifdef SVMT_DEBUG_MODE
    svmt_check_vector_index_range(
      "<Type><System>SubTensor::gather(const offsetSubVector&) const",
      h, extent2());
#endif // SVMT_DEBUG_MODE
    }
  return result;
  }

<Type><System>SubTensor&
  <Type><System>SubTensor::scatter(const offsetSubVector& x,
    const <Type><System>SubVector& v) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubTensor::scatter(\n"
    "const offsetSubVector&, const <Type><System>SubVector&)",
    x.extent(), v.extent());
#endif // SVMT_DEBUG_MODE
  return *this;
  }

const
<Type><System>Tensor
  <Type><System>SubTensor::aside(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubTensor::aside(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2());
#endif // SVMT_DEBUG_MODE
  <Type><System>Tensor		result(extent3(), extent2(), extent1()
							 + U.extent1());
  result.sub((Offset)0, extent3(), (Stride)1,
	     (Offset)0, extent2(), (Stride)1,
	     (Offset)0, extent1(), (Stride)1) = *this;
  result.sub((Offset)0, extent3(), (Stride)1,
	     (Offset)0, extent2(), (Stride)1,
	     extent1(), U.extent1(), (Stride)1) = U;
  return result;
  }

const
<Type><System>Tensor
  <Type><System>SubTensor::above(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubTensor::above(const <Type><System>SubTensor&) const",
    extent1(), U.extent1(), extent3(), U.extent3());
#endif // SVMT_DEBUG_MODE
  <Type><System>Tensor		result(extent3(), extent2()
					      + U.extent2(), extent1());
  result.sub((Offset)0, extent3(), (Stride)1,
	     (Offset)0, extent2(), (Stride)1) = *this;
  result.sub((Offset)0, extent3(), (Stride)1,
	     extent2(), U.extent2(), (Stride)1) = U;
  return result;
  }

const
<Type><System>Tensor
  <Type><System>SubTensor::afore(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_matrix_conformance(
    "<Type><System>SubTensor::afore(const <Type><System>SubTensor&) const",
    extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>Tensor		result(extent3()
				   + U.extent3(), extent2(), extent1());
  result.sub((Offset)0, extent3(), (Stride)1) = *this;
  result.sub(extent3(), U.extent3(), (Stride)1) = U
  return result;
  }

const
<Type><System>Tensor
  <Type><System>SubVector::kron(const <Type><System>SubTensor& T) const {
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Tensor		result(T.extent2(), extent()*T.extent1());

  for (Offset h = 0; h < extent3(); h++) {
    result[h] = v.kron(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
  <Type><System>SubTensor::kron(const <Type><System>SubTensor& U) const {
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor	result(extent2()*U.extent2(), extent1()*U.extent1());
  for (Offset h = 0; h < extent3(); h++) {
    for (Offset j = 0; j < extent1(); j++) {
      result.sub(h*U.extent2(), U.extent2(), (Stride)1,
		 j*U.extent1(), U.extent1(), (Stride)1)
	= T[h][j]*U;
      }
    }
  return result;
  }

const
<Type><System>Tensor
  <Type><System>SubTensor::kron(const <Type><System>SubVector& v) const {
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(T.extent2(), T.extent1()*extent());

  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h].kron(v);
    }
  return result;
  }

ifelse(_system_, complex, `
const
<Type><System>Tensor
  <Type>SubVector::kron(const <Type><System>SubTensor& T) const {
  const
  <Type>SubVector&	v = *this;
  <Type><System>Tensor	result(T.extent2(), extent()*T.extent1());

  for (Offset h = 0; h < extent3(); h++) {
    result[h] = v.kron(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
  <Type>SubTensor::kron(const <Type><System>SubTensor& U) const {
  const
  <Type>SubTensor&	T = *this;
  <Type><System>Tensor	result(extent2()*U.extent2(), extent1()*U.extent1());
  for (Offset h = 0; h < extent3(); h++) {
    for (Offset j = 0; j < extent1(); j++) {
      result.sub(h*U.extent2(), U.extent2(), (Stride)1,
		 j*U.extent1(), U.extent1(), (Stride)1)
	= T[h][j]*U;
      }
    }
  return result;
  }

const
<Type><System>Tensor
  <Type>SubTensor::kron(const <Type><System>SubVector& v) const {
  const
  <Type>SubTensor&	T = *this;
  <Type><System>Tensor	result(T.extent2(), T.extent1()*extent());

  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h].kron(v);
    }
  return result;
  }

const
<Type><System>Tensor
  <Type><System>SubVector::kron(const <Type>SubTensor& T) const {
  const
  <Type><System>SubVector&	v = *this;
  <Type><System>Tensor		result(T.extent2(), extent()*T.extent1());

  for (Offset h = 0; h < extent3(); h++) {
    result[h] = v.kron(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
  <Type><System>SubTensor::kron(const <Type>SubTensor& U) const {
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor	result(extent2()*U.extent2(), extent1()*U.extent1());
  for (Offset h = 0; h < extent3(); h++) {
    for (Offset j = 0; j < extent1(); j++) {
      result.sub(h*U.extent2(), U.extent2(), (Stride)1,
		 j*U.extent1(), U.extent1(), (Stride)1)
	= T[h][j]*U;
      }
    }
  return result;
  }

const
<Type><System>Tensor
  <Type><System>SubTensor::kron(const <Type>SubVector& v) const {
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(T.extent2(), T.extent1()*extent());

  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h].kron(v);
    }
  return result;
  }
')dnl
*/

const
<Type><System>Tensor
  <Type><System>SubTensor::apply(const TYPE (*f)(const TYPE&)) const {
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h].apply(f);
    }
  return result;
  }

const
<Type><System>Tensor
  <Type><System>SubTensor::apply(      TYPE (*f)(const TYPE&)) const {
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h].apply(f);
    }
  return result;
  }

const
<Type><System>Tensor
  <Type><System>SubTensor::apply(      TYPE (*f)(      TYPE )) const {
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h].apply(f);
    }
  return result;
  }

const
<Type><System>Tensor
    exp(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = exp(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
    log(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = log(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
   sqrt(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = sqrt(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
    cos(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = cos(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
    sin(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = sin(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
    tan(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = tan(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
   acos(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = acos(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
   asin(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = asin(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
   atan(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = atan(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
   cosh(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = cosh(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
   sinh(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = sinh(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
   tanh(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = tanh(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
  acosh(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = acosh(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
  asinh(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = asinh(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
  atanh(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = atanh(T[h]);
    }
  return result;
  }

ifelse(_system_, complex,
`// Complex Functions
const
<Type><System>Tensor
   conj(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = conj(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
  iconj(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = iconj(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
  polar(const <Type>SubTensor& T, const <Type>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "polar(const <Type>SubTensor&, const <Type>SubTensor&)",
  T.extent3(), U.extent3(), T.extent2(), U.extent2(), T.extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = polar(T[h], U[h]);
    }
  return result;
  }

const
<Type>Tensor
   norm(const <Type><System>SubTensor& T) {
  <Type>Tensor			result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = norm(T[h]);
    }
  return result;
  }
', `
const
<Type><System>Tensor
    sgn(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = sgn(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
    abs(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = abs(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
  floor(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = floor(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
   ceil(const <Type><System>SubTensor& T) {
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = ceil(T[h]);
    }
  return result;
  }

const
<Type><System>Tensor
  hypot(const <Type><System>SubTensor& T, const <Type><System>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "hypot(const <Type><System>SubTensor&, const <Type><System>SubTensor&)",
  T.extent3(), U.extent3(), T.extent2(), U.extent2(), T.extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>Tensor		result(T.extent3(), T.extent2(), T.extent1());
  for (Offset h = 0; h < T.extent3(); h++) {
    result[h] = hypot(T[h], U[h]);
    }
  return result;
  }

const
<Type><System>Tensor
  atan2(const <Type><System>SubTensor& U, const <Type><System>SubTensor& T) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "atan2(const <Type><System>SubTensor&, const <Type><System>SubTensor&)",
  U.extent3(), T.extent3(), U.extent2(), T.extent2(), U.extent1(), T.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>Tensor		result(U.extent3(), U.extent2(), U.extent1());
  for (Offset h = 0; h < U.extent3(); h++) {
    result[h] = atan2(U[h], T[h]);
    }
  return result;
  }
')dnl

// Operators
ifelse(floating, _point_, `',
`// complement
const
<Type><System>Tensor
  <Type><System>SubTensor::operator ~(void) const {
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = ~T[h];
    }
  return result;
  }
')dnl

// minus
const
<Type><System>Tensor
  <Type><System>SubTensor::operator -(void) const {
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = -T[h];
    }
  return result;
  }

ifelse(_system_, complex, `
// multiply
const
<Type><System>Tensor
  <Type><System>SubTensor::operator *(const <Type>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator *(const <Type>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h]*U[h];
    }
  return result;
  }

// divide
const
<Type><System>Tensor
  <Type><System>SubTensor::operator /(const <Type>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator /(const <Type>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h]/U[h];
    }
  return result;
  }

// add
const
<Type><System>Tensor
  <Type><System>SubTensor::operator +(const <Type>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator +(const <Type>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h] + U[h];
    }
  return result;
  }

// subtract
const
<Type><System>Tensor
  <Type><System>SubTensor::operator -(const <Type>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator -(const <Type>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h] - U[h];
    }
  return result;
  }

// divide
const
<Type><System>Tensor
  <Type>SubTensor::operator /(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type>SubTensor::operator /(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type>SubTensor&		T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h]/U[h];
    }
  return result;
  }

// subtract
const
<Type><System>Tensor
  <Type>SubTensor::operator -(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type>SubTensor::operator -(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type>SubTensor&		T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h] - U[h];
    }
  return result;
  }
')dnl

// multiply
const
<Type><System>Tensor
  <Type><System>SubTensor::operator *(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator *(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h]*U[h];
    }
  return result;
  }

// divide
const
<Type><System>Tensor
  <Type><System>SubTensor::operator /(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator /(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h]/U[h];
    }
  return result;
  }

ifelse(floating, _point_, `',
`// modulo (remainder)
const
<Type><System>Tensor
  <Type><System>SubTensor::operator %(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator %(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h]%U[h];
    }
  return result;
  }
')dnl

// add
const
<Type><System>Tensor
  <Type><System>SubTensor::operator +(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator +(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h] + U[h];
    }
  return result;
  }

// subtract
const
<Type><System>Tensor
  <Type><System>SubTensor::operator -(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator -(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h] - U[h];
    }
  return result;
  }

ifelse(floating, _point_, `',
`// shift left
const
<Type><System>Tensor
  <Type><System>SubTensor::operator <<(int n) const {
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h] << n;
    }
  return result;
  }

// shift right
const
<Type><System>Tensor
  <Type><System>SubTensor::operator >>(int n) const {
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h] >> n;
    }
  return result;
  }
')dnl

std::istream&	operator >>(std::istream& is,       <Type><System>SubTensor& T) {
  for (Offset h = 0; h < T.extent3(); h++) {
    <Type><System>SubMatrix	M = T[h];
    if (!(is >> M)) break;
    }
  return is;
  }

std::ostream&	operator <<(std::ostream& os, const <Type><System>SubTensor& T) {
  int			width = os.width();
  for (Offset h = 0; h < T.extent3(); h++) {
    os << std::setw(width) << T[h];
    }
  return os;
  }

ifelse(_system_, complex, `
// equal
bool	<Type><System>SubTensor::operator ==(const <Type>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator ==(const <Type>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  Offset			h = (Offset)0;
  while (h < extent3() && T[h] == U[h]) {
   ++h;
   }
  return h == extent3();
  }

// not equal
bool	<Type><System>SubTensor::operator !=(const <Type>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator !=(const <Type>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  Offset			h = (Offset)0;
  while (h < extent3() && T[h] != U[h]) {
   ++h;
   }
  return h == extent3();
  }
',

`// less than
bool	<Type><System>SubTensor::operator < (
  const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator < (const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  Offset			h = (Offset)0;
  while (h < extent3() && T[h] <  U[h]) {
   ++h;
   }
  return h == extent3();
  }

// less than or equal
bool	<Type><System>SubTensor::operator <=(
  const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator <=(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  Offset			h = (Offset)0;
  while (h < extent3() && T[h] <= U[h]) {
   ++h;
   }
  return h == extent3();
  }
')dnl

// equal
bool	<Type><System>SubTensor::operator ==(
  const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator ==(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  Offset			h = (Offset)0;
  while (h < extent3() && T[h] == U[h]) {
   ++h;
   }
  return h == extent3();
  }

// not equal
bool	<Type><System>SubTensor::operator !=(
  const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator !=(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  Offset			h = (Offset)0;
  while (h < extent3() && T[h] != U[h]) {
   ++h;
   }
  return h == extent3();
  }

ifelse(floating, _point_, `',
`// bitwise AND
const
<Type><System>Tensor
  <Type><System>SubTensor::operator &(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator &(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h]&U[h];
    }
  return result;
  }

// bitwise exclusive OR
const
<Type><System>Tensor
  <Type><System>SubTensor::operator ^(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator ^(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h]^U[h];
    }
  return result;
  }

// bitwise inclusive OR
const
<Type><System>Tensor
  <Type><System>SubTensor::operator |(const <Type><System>SubTensor& U) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator |(const <Type><System>SubTensor&) const",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubTensor&	T = *this;
  <Type><System>Tensor		result(extent3(), extent2(), extent1());
  for (Offset h = 0; h < extent3(); h++) {
    result[h] = T[h]|U[h];
    }
  return result;
  }
')dnl

ifelse(_system_, complex, `
// simple assignment
<Type><System>SubTensor&
  <Type><System>SubTensor::operator  =(const <Type>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator  =(const <Type>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] = U[h];
    }
  return *this;
  }

// multiply and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator *=(const <Type>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator *=(const <Type>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] *= U[h];
    }
  return *this;
  }

// divide and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator /=(const <Type>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator /=(const <Type>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] /= U[h];
    }
  return *this;
  }

// add and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator +=(const <Type>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator +=(const <Type>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] += U[h];
    }
  return *this;
  }

// subtract and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator -=(const <Type>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator -=(const <Type>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] -= U[h];
    }
  return *this;
  }
')dnl

// simple assignment
<Type><System>SubTensor&
  <Type><System>SubTensor::operator  =(const <Type><System>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator  =(const <Type><System>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] = U[h];
    }
  return *this;
  }

// multiply and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator *=(const <Type><System>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator *=(const <Type><System>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] *= U[h];
    }
  return *this;
  }

// divide and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator /=(const <Type><System>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator /=(const <Type><System>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] /= U[h];
    }
  return *this;
  }

ifelse(floating, _point_, `',
`// modulo and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator %=(const <Type><System>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator %=(const <Type><System>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] %= U[h];
    }
  return *this;
  }
')dnl

// add and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator +=(const <Type><System>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator +=(const <Type><System>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] += U[h];
    }
  return *this;
  }

// subtract and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator -=(const <Type><System>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator -=(const <Type><System>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] -= U[h];
    }
  return *this;
  }

ifelse(floating, _point_, `',
`// shift left and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator <<=(int n) {
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] <<= n;
    }
  return *this;
  }

// shift right and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator >>=(int n) {
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] >>= n;
    }
  return *this;
  }

// AND and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator &=(const <Type><System>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator &=(const <Type><System>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] &= U[h];
    }
  return *this;
  }

// exclusive OR and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator ^=(const <Type><System>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator ^=(const <Type><System>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] ^= U[h];
    }
  return *this;
  }

// inclusive OR and assign
<Type><System>SubTensor&
  <Type><System>SubTensor::operator |=(const <Type><System>SubTensor& U) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_tensor_conformance(
    "<Type><System>SubTensor::operator |=(const <Type><System>SubTensor&)",
    extent3(), U.extent3(), extent2(), U.extent2(), extent1(), U.extent1());
#endif // SVMT_DEBUG_MODE
  <Type><System>SubTensor&	T = *this;
  for (Offset h = 0; h < extent3(); h++) {
    T[h] |= U[h];
    }
  return *this;
  }
')dnl
popdef(`index')dnl
popdef(`shift')dnl

