dnl This is a class prototype input file for .../svmt/bin/genclass.
pushdef(`index', `ifelse($#, 0, ``index'', ``index($*)'')')dnl
pushdef(`shift', `ifelse($#, 0, ``shift'', ``shift($*)'')')dnl
/*
The C++ Scalar, Vector, Matrix and Tensor classes.
Copyright (C) 1998 E. Robert Tisdale

This file is part of The C++ Scalar, Vector, Matrix and Tensor classes.
This library is free software which you can redistribute and/or modify
under the terms of the GNU Library General Public License
as published by the Free Software Foundation;
either version 2, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this library.  If not, write to the Free Software Foundation,
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Written by E. Robert Tisdale
*/

#include<<Type><System>Vector.h>

define(_point_,
ifelse(_type_, long double, floating,
ifelse(_type_,      double, floating,
ifelse(_type_,       float, floating, fixed))))dnl
define(TYPE, ifelse(_system_, complex, `<Type><System>', `<type>'))dnl
define(ZERO, ifelse(_system_, complex,
  `<Type><System>((<type>)0, (<type>)0)', `(<type>)0'))dnl

// Functions
// v_{j} = v_{n-1-j}
<Type><System>SubVector&
  <Type><System>SubVector::reverse(void) {
  const
  Extent        n = extent();
  for (Offset j = 0; j < n/2; j++)
    swap(j, n-1 - j);
  return *this;
  }
ifelse(_system_, complex,
`<Type>SubVector&
  <Type>SubVector::pack(const <Type><System>SubVector& y) {
  <Type>SubVector&
		x = *this;
  Extent	n = x.extent();
  if (0 < n) {
    x.even() = y.real().sub(0, (n+1)/2, 1);	// (n+1)/2 = n/2 if n is even
    x.odd()  = y.imag().sub(0,     n/2, 1);	// (n-1)/2 = n/2 if n is odd
    }
  return x;
  }

<Type><System>SubVector&
  <Type><System>SubVector::pack(const <Type>SubVector& x) {
  <Type><System>SubVector&
		y = *this;
  Extent	n = x.extent();
  if (0 < n) {
    y.real().sub(0, (n+1)/2, 1) = x.even();	// (n+1)/2 = n/2 if n is even
    y.imag().sub(0,     n/2, 1) = x.odd();	// (n-1)/2 = n/2 if n is odd
    }
  return y;
  }
')dnl

<Type><System>SubVector&
	<Type><System>SubVector::transpose(bool b) {
  // This function transposes the elements of a vector v of extent n
  // in-place using a mixed radix digit-reverse algorithm
  // where the radix vector r contains the prime factors of n
  // sorted in order from least to greatest.  No element of the vector
  // is moved to another location in the vector more than once.
  <Type><System>SubVector&
		v = *this;
  Extent	n = v.extent();
  offsetVector	r = offsetPrimeFactor(n);
  if (1 < r.extent()) {			// n is not prime
    offsetMixedRadixDigitReverse
		t(r);
    // The algorithm searches for every sequence k_s of S indices
    // such that a circular shift of elements v_{k_s} <-- v_{k_{s+1}}
    // and v_{k_{S-1}} <-- v_{k_0} effects an in-place transpose.
    // The first and last elements of the vector never move
    // so the transpose is complete after no more than n-2 moves.
    Extent	m = 0;			// count up to n-2 moves
    Offset	l = 0;			// 1 <= l <= n-2
    while (++l < n-1 && m < n-2) {
      Offset	k = l;
      Offset	j = k;
      while (l < (k = (b? t.next(j): t.last(j)))) {
	j = k;				// Search backward for k < l.
	}
      // If a sequence of indices beginning with l has any index k < l,
      // it has already been transposed.  The sequence length S = 1
      // and diagonal element v_k is its own transpose if k = j.
      // Skip every index sequence that has already been transposed.
      if (k == l) {			// a new sequence
	if (k < j) {			// with 1 < S
	  TYPE	x = v[k];		// save v_{k_0}
	  do {
	    v[k] = v[j];		// v_{k_{s}} <-- v_{k_{s+1}}
	    k = j;
	    ++m;
	    } while (l < (j = (b? t.last(k): t.next(k))));
	  v[k] = x;			// v_{k_{S-1}} <-- v_{k_0}
	  }
	++m;
	}
      }
    }
  return v;
  }

<Type><System>SubVector&
	<Type><System>SubVector::transpose(Extent p, Extent q) {
  <Type><System>SubVector&
		v = *this;
  if (1 < p && 1 < q) {
    // A vector v of extent n = qp is viewed as a q by p matrix U and
    // a p by q matrix V where U_{ij} = v_{p*i+j} and V_{ij} = v_{q*i+j}.
    // The vector v is modified in-place so that V is the transpose of U.
    // The algorithm searches for every sequence k_s of S indices
    // such that a circular shift of elements v_{k_s} <-- v_{k_{s+1}}
    // and v_{k_{S-1}} <-- v_{k_0} effects an in-place transpose.
    Extent	n = q*p;
    Extent	m = 0;			// count up to n-2
    Offset	l = 0;			// 1 <= l <= n-2
    while (++l < n-1 && m < n-2) {
      Offset	k = l;
      Offset	j = k;
      while (l < (k = (j%p)*q + j/p)) {	// Search backward for k < l.
	j = k;
	}
      // If a sequence of indices beginning with l has any index k < l,
      // it has already been transposed.  The sequence length S = 1
      // and diagonal element v_k is its own transpose if k = j.
      // Skip every index sequence that has already been transposed.
      if (k == l) {			// a new sequence
	if (k < j) {			// with 1 < S
	  TYPE	x = v[k];		// save v_{k_0}
	  do {
	    v[k] = v[j];		// v_{k_{s}} <-- v_{k_{s+1}}
	    k = j;
	    ++m;
	    } while (l < (j = (k%q)*p + k/q));
	  v[k] = x;			// v_{k_{S-1}} <-- v_{k_0}
	  }
	++m;
	}
      }
    }
  return v;
  }
ifelse(_system_, complex,
`ifelse(floating, _point_,
`<Type><System>SubVector&
  <Type><System>SubVector::twiddle(int sign) {
  <Type><System>SubVector&
		w = *this;
  Extent	n = w.extent();
  if (0 < n) {
    sign = (sign < 0)? -1: +1;
    w.imag().ramp() *= sign*2*M_PI/n;
    w.real() = cos(w.imag());
    w.imag() = sin(w.imag());
    }
  return w;
  }

<Type>Vector
  <Type><System>SubVector::sdftcr(const <Type>SubVector& x) const {
  // Slow Discrete Fourier Transform Complex from Real  y = w.sdftcr(x)
  // (y_{2k}, y_{2k+1}) = x_0*w_0^k + x_1*w_1^k + ... + x_{n-1}*w_{n-1}^k
  // where w_j = e^{-i*2*pi*j/n} for forward Discrete Fourier Transforms
  //   and w_j = e^{+i*2*pi*j/n} for reverse Discrete Fourier Transforms.
  // Because y_{2(n-k)} = +y_{2k} and y_{2(n-k)+1} = -y_{2k+1},
  // the complex result is packed into a real vector y of length n with
  // the real and imaginary parts in the even and odd elements respectively.
  // Since the result is real for k = 0, y_{n} is stored in y_{1}.
  // The computational complexity of the algorithm is order n^2.

  const
  <Type><System>SubVector&
		w = *this;
  Extent	n = w.extent();
  <Type>Vector	y(n);			// return vector
  if (0 < n) {
    y[0] = x.sum();			// w_{j}^0 = 1
    for (Offset m = 2; m < n; m += 2) {
      Offset	k = m >> 1;		// m = 2k
      Offset	l = (m+1 < n)? m+1: 1;	// l = 2k+1 or 1 if 2k+1 = n
      <Type><System>
		t = <Type><System>(x[0], (<type>)0);	// w_{0}^k = 1
      for (Offset j = 1; j < n; j++)
	t += x[j]*w[j*k%n];
      y[m] = t.real();			// real and
      y[l] = t.imag();			// imaginary parts
      }
    if (0 == (1&n)) {			// n is even and y_{1} <-- y_{n}
      y[1] = x.even().sum()		// w_{n/2}^{2k}   = +1
  	   - x.odd().sum();		// w_{n/2}^{2k+1} = -1
      }
    }
  return y;
  }

<Type>SubVector&
  <Type>SubVector::rcfdft(const <Type><System>SubVector& w) {
  // (in-place) Real to Complex Fast Discrete Fourier Transform  x.rcfdft(w)
  // (y_{2k}, y_{2k+1}) = x_0*w_0^k + x_1*w_1^k + ... + x_{n-1}*w_{n-1}^k
  // where w_j = e^{-i*2*pi*j/n} for forward Fast Fourier Transforms
  //   and w_j = e^{+i*2*pi*j/n} for reverse Fast Fourier Transforms
  // Because y_{2(n-k)} = +y_{2k} and y_{2(n-k)+1} = -y_{2k+1},
  // the complex result vector y is packed back into real vector x with
  // the real and imaginary parts in the even and odd elements respectively.
  // Since the result is real for k = 0, y_{n} is stored in y_{1}.

  <Type>SubVector&
		x = *this;
  Extent	n = x.extent();
  if (1 < n) {
    Extent	p = lpf(n);		// least prime factor of n = qp
    if (p == n) {			// n is prime
      x.rcsdft(w);// (in-place) Real to Complex Slow Discrete Fourier Transform
      }

    else {				// n is not prime
      // computational complexity order n*log(n) recursive algorithm
      // The recursive in-place algorithm has four parts.
      // 1.) transpose matrix X (vector x viewed as a q by p matrix) in-place
      //     to get matrix Y = X^{T} (vector x viewed as a p by q matrix),
      // 2.) apply the fast Fourier transform to each row of matrix Y,
      // 3.) multiply each column of matrix Y by the corresponding column
      //     of matrix M where M_{lk} = w_{l*k} = e^{-/+i*2*pi*l*k/n} then
      //     apply the discrete Fourier transform to each column of Y and
      // 4.) reverse elements Y_{2h+1, 2} through Y_{2h+1, p-1} in place,
      //     swap element Y_{2h, 1} with Y_{2h+1, 0} for 0 < 2h+1 < p and
      //     swap element Y_{0, 1} with Y_{p-1, 1} if p is odd.

      Extent	q = n/p;
      <Type>SubVector&			// part 1
//		y = x.transpose(p, q);
		y = x;			// Call x.transpose(false) before entry.
      <Type><System>SubVector		// part 2
		w_q = w.sub(0, q, p);
      for (Offset h = 0; h < p; h++)
	y.sub(q*h, q, 1).rcfdft(w_q);
      <Type><System>SubVector		// part 3
		w_p = w.sub(0, p, q);
      y.sub(0, p, q).rcsdft(w_p);		// column 0
      for (Offset m = 2; m < q; m += 2) {	// column m and column l
	Offset	k = m >> 1;		// m = 2k
	Offset	l = (m+1 < q)? m+1: 1;	// l = 2k+1 or 1 if 2k+1 = q
	<Type>SubVector
		r_k = y.sub(m, p, q);	// real and
	<Type>SubVector
		i_k = y.sub(l, p, q);	// imaginary parts
	<Type><System>Vector		// column k
		t = <Type><System>Vector(r_k, i_k);
		t *= w.sub(0, p, k);	// multiply by column k of matrix M
		t.sdft(w_p);		// apply DFT in-place
	<Type><System>SubVector		// t = [u, v.r()]
		u = t.sub(0, (p+1)/2, +1);	// (p+1)/2 = p/2 if p is even
	<Type><System>SubVector		// t = [u, v.r()]
		v = t.sub(p-1,   p/2, -1);	// (p-1)/2 = p/2 if p is odd
		v = conj(v);	// y_{2(qh + q-k)}   = +y_{2(q(p-1-h) + k)} and
				// y_{2(qh + q-k)+1} = -y_{2(q(p-1-h) + k)+1}
	r_k.even() = u.real(); i_k.even() = u.imag();	// standard order
	r_k.odd()  = v.imag(); i_k.odd()  = v.real();	// reversed order
	}
      if (0 == (1&q)) {			// q is even
	<Type>SubVector
		y_1 = y.sub(1, p, q);	// Y_{h, 1} = Y_{h, q}
	<Type><System>Vector
		t = y_1*w.sub(0, p, q/2);
		t.sdft(w_p);
		y_1.pack(t);
	}
					// part 4
      for (Offset l = 1; l < p; l += 2) {
	Offset	m = l-1;
	y.sub(q*l+2, q-2, 1).reverse();
	<type>	t = y[q*m+1]; y[q*m+1] = y[q*l]; y[q*l] = t;
	}
      if (1&p) {			// p is odd
	<type>	t = y[1]; y[1] = y[n-q+1]; y[n-q+1] = t;
	}
      }
    }
  return x;
  }

<Type>SubVector&
  <Type>SubVector::cdft(int sign) {	// real to complex dft
  Extent	n = length();
  if (1 < n) {
    <Type><System>Vector
		w(n);
		w.twiddle(sign);
    transpose(false).rcfdft(w);
    }
  return *this;
  }

<Type>Vector
  <Type><System>SubVector::sdftrc(const <Type>SubVector& y) const {
  // Slow Discrete Fourier Transform Real from Complex  x = w.sdftrc(y)
  // x_j = (y_0,y_1)*w_0^j + (y_2,y_3)*w_1^j +...+ (y_{2n-2},y_{2n-1})*w_{n-1}^j
  // where w_k = e^{-i*2*pi*k/n} for forward Discrete Fourier Transforms
  //   and w_k = e^{+i*2*pi*k/n} for reverse Discrete Fourier Transforms
  // Because y_{2(n-k)} = +y_{2k} and y_{2(n-k)+1} = -y_{2k+1},
  // the complex source is packed into a real vector y of length n with
  // the real and imaginary parts in the even and odd elements respectively.
  // Since the source is real for k = 0, y_{n} is stored in y_{1}.
  // The computational complexity of the algorithm is order n^2.

  const
  <Type><System>SubVector&
		w = *this;
  Extent	n = w.extent();
  <Type>Vector	x(n);			// result vector
  if (0 < n) {
    x[0] = 2*y.even().sum() - y[0];	// w_{k}^0 = 1
    if (0 == (1&n))			// n is even
      x[0] += y[1];			// y_{1} = y{n}

    for (Offset j = 1; j < n; j++) {
      <type>	t = 0.5*y[0];		// w_{0}^j = 1
      for (Offset m = 2; m < n; m += 2) {
	Offset	k = m >> 1;		// m = 2k
	Offset	l = (m+1 < n)? m+1: 1;	// l = 2k+1 or 1 if 2k+1 = n
	t += y[m]*w[k*j%n].real() - y[l]*w[k*j%n].imag();
	}
      t *= 2.0;
      if (0 == (1&n))			// n is even and y_{n} = y_{1}
	if (1&j)			// j is odd	
	  t -= y[1];			// w_{n/2}^j = -1
	else				// j is even
	  t += y[1];			// w_{n/2}^j = +1
      x[j] = t;
      }
    }
  return x;
  }

<Type>SubVector&
  <Type>SubVector::crfdft(const <Type><System>SubVector& w) {
  // (in-place) Complex to Real Fast Discrete Fourier Transform  y.crfdft(w)
  // x_j = (y_0,y_1)*w_0^j + (y_2,y_3)*w_1^j +...+ (y_{2n-2},y_{2n-1})*w_{n-1}^j
  // where w_k = e^{-i*2*pi*k/n} for forward Discrete Fourier Transforms
  //   and w_k = e^{+i*2*pi*k/n} for reverse Discrete Fourier Transforms
  // Because y_{2(n-k)} = +y_{2k} and y_{2(n-k)+1} = -y_{2k+1},
  // the complex source is packed into a real vector y of length n with
  // the real and imaginary parts in the even and odd elements respectively.
  // Since the source is real for k = 0, y_{n} is stored in y_{1}.

  <Type>SubVector&
		y = *this;
  Extent	n = y.extent();
  if (1 < n) {
    Extent	p = lpf(n);		// least prime factor of n = qp
    if (p == n) {			// n is prime
      y.crsdft(w);// (in-place) Complex to Real Slow Discrete Fourier Transform
      }

    else {				// n is not prime
      // computational complexity order n*log(n) recursive algorithm
      // The recursive in-place algorithm has four parts.
      // 1.) swap element Y_{0, 1} with Y_{p-1, 1} if p is odd then
      //     swap element Y_{2h, 1} with Y_{2h+1, 0} for 0 < 2h+1 < p and
      //     reverse elements Y_{2h+1, 2} through Y_{2h+1, p-1} in place,
      // 2.) apply the discrete Fourier transform to each column of Y then
      //     multiply each column of matrix Y by the corresponding column
      //     of matrix M where M_{lk} = w_{l*k} = e^{-/+i*2*pi*l*k/n},
      // 3.) apply the fast Fourier transform to each row of matrix Y,
      // 4.) transpose matrix Y (vector x viewed as a p by q matrix) in-place
      //     to get matrix X = Y^{T} (vector x viewed as a q by p matrix).

      Extent	q = n/p;		// part 1
      if (1&p) {			// p is odd
	<type>	t = y[1]; y[1] = y[n-q+1]; y[n-q+1] = t;
	}
      for (Offset l = 1; l < p; l += 2) {
	Offset	m = l-1;
	<type>	t = y[q*m+1]; y[q*m+1] = y[q*l]; y[q*l] = t;
	y.sub(q*l+2, q-2, 1).reverse();
	}
      <Type><System>SubVector		// part 2
		w_p = w.sub(0, p, q);
      y.sub(0, p, q).crsdft(w_p);		// column 0
      for (Offset m = 2; m < q; m += 2) {	// column m and column l
	Offset	k = m >> 1;		// m = 2k
	Offset	l = (m+1 < q)? m+1: 1;	// l = 2k+1 or 1 if 2k+1 = q
	<Type>SubVector
		r_k = y.sub(m, p, q);	// real and
	<Type>SubVector
		i_k = y.sub(l, p, q);	// imaginary parts
	<Type><System>Vector		// column k
		t = <Type><System>Vector(r_k.even().aside( i_k.odd().r()),
					 i_k.even().aside(-r_k.odd().r()));
		t.sdft(w_p) *= w.sub(0, p, k);
	r_k = t.real(); i_k = t.imag();
	}
      if (0 == (1&q)) {			// q is even
	<Type>SubVector
		y_1 = y.sub(1, p, q);	// Y_{h, 1} = Y_{h, q}
        <Type><System>Vector
		t(p);
		t.pack(y_1);
	if (1&p)			// p is odd
	  t[(p-1)/2].imag() = (<type>)0;
		t.sub(p-1, p/2, -1) = conj(t.sub(0, p/2, +1));
        	t.sdft(w_p) *= w.sub(0, p, q/2);
		y_1 = t.real();
	}
      <Type><System>SubVector		// part 3
		w_q = w.sub(0, q, p);
      for (Offset h = 0; h < p; h++)
	y.sub(q*h, q, 1).crfdft(w_q);
//    <Type>SubVector&			// part 4
//		x = y.transpose(q, p);	// Call y.transpose(true) after return.
      }
    }
  return y;
  }

<Type>SubVector&
  <Type>SubVector::rdft(int sign) {	// complex to real dft
  Extent	n = length();
  if (1 < n) {
    <Type><System>Vector
		w(n);
		w.twiddle(sign);
    crfdft(w).transpose(true);
    }
  return *this;
  }

<Type><System>SubVector&		// Slow Discrete Fourier Transform
  <Type><System>SubVector::sdft(const <Type><System>SubVector& w) {
  // v <-- dft(v) (in-place)
  // v_k <-- v_0*w_0^k + v_1*w_1^k + v_2*w_2^k + ... + v_{n-1}*w_{n-1}^k
  // where w_j = e^{-i*2*pi*j/n} for forward Discrete Fourier Transforms
  //   and w_j = e^{+i*2*pi*j/n} for inverse Discrete Fourier Transforms
  // computational complexity order n^2	algorithm
  Extent	n = extent();
  <Type><System>Vector
		t(n);			// temporary workspace
  <Type><System>SubVector&
		v = *this;
  for (Offset k = 0; k < n; k++) {
    <Type><System>	z = v[0];
    for (Offset j = 1; j < n; j++) {
      z += v[j]*w[j*k%n];
      }
    t[k] = z;
    }
  return v = t;
  }

<Type><System>SubVector&		// Fast Discrete Fourier Transform
  <Type><System>SubVector::fdft(const <Type><System>SubVector& w) {
  // v <-- dft(v) (in-place)
  // v_k = v_0*w_0^k + v_1*w_1^k + v_2*w_2^k + ... + v_{n-1}*w_{n-1}^k
  // where w_j = e^{-i*2*pi*j/n} for forward Fast Fourier Transforms
  //   and w_j = e^{+i*2*pi*j/n} for inverse Fast Fourier Transforms
  <Type><System>SubVector&
		v = *this;
  Extent	n = v.extent();
  Extent	p = lpf(n);		// least prime factor of n = qp
  if (p == n) {				// n is prime
    v.sdft(w);				// Slow Discrete Fourier Transform
    }
  else {				// n is not prime
    // computational complexity order n*log(n) recursive algorithm
    // The recursive in-place algorithm has three parts.
    // 1.) transpose matrix U (vector v viewed as a q by p matrix)
    //     in-place to get matrix V (vector v viewed as a p by q matrix),
    // 2.) apply the fast Fourier transform to each row of V
    //     and multiply by the corresponding row of matrix M
    //     where M_{lk} = w_{l*k} = e^{-/+i*2*pi*l*k/n} then
    // 3.) apply the discrete Fourier transform to each column of V.
    Extent	q = n/p;
//  v.transpose(p, q);			// part 1
    <Type><System>SubVector
		w_q = w.sub(0, q, p);
    for (Offset l = 0; l < p; l++) {	// part 2
      v.sub(q*l, q, 1).fdft(w_q) *= w.sub(0, q, l);
      }
    <Type><System>SubVector
		w_p = w.sub(0, p, q);
    for (Offset k = 0; k < q; k++) {	// part 3
      v.sub(k, p, q).sdft(w_p);
      }
    }
  return v;
  }

<Type><System>SubVector&
  <Type><System>SubVector::dft(int sign) {	// complex to complex dft
  Extent	n = length();
  if (1 < n) {
    <Type><System>Vector
		w(n);
		w.twiddle(sign);
    transpose().fdft(w);
    }
  return *this;
  }
')dnl
', `
// v_{j} = j
<Type><System>SubVector&
  <Type><System>SubVector::ramp(void) {
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();

  for (Offset j = 0; j < n; j++) {
    h.put(o, (<type>)j);
    o += s;
    }
  return *this;
  }
')dnl

<Type><System>SubVector&
  <Type><System>SubVector::swap(<Type><System>SubVector& w) {
  <Type><System>Handle&	h  = (<Type><System>Handle&)handle();
  Offset		o  = offset();
  Extent		n  = extent();
  Stride		s  = stride();
  <Type><System>Handle&	hw = (<Type><System>Handle&)w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();

  for (Offset j = 0; j < n; j++) {
    <Type><System> t = h.get(o); h.put(o, hw.get(ow)); hw.put(ow, t);
    o += s; ow += sw;
    }
  return *this;
  }

<Type><System>SubVector&
  <Type><System>SubVector::rotate(Stride n) {
  if (n < 0) {
    reverse().rotate(-n);
    }
  else {
    if (1 < extent()) {
      <Type><System>Handle&
			h = (<Type><System>Handle&)handle();
      Offset		o = offset();
      Stride		s = stride();

      n %= extent();
      for (Offset i = 0; i < gcd((Extent)n, extent()); i++) {
	TYPE		x = h.get(o + (Stride)i*s);
	Offset		j = i;
	Offset		k = 0;
	while (i != (k = (j+n)%extent())) {
	  h.put(o + (Stride)j*s, h.get(o + (Stride)k*s));
	  j = k;
	  }
	h.put(o + (Stride)j*s, x);
	}
      }
    }
  return *this;
  }

<Type><System>SubVector&
  <Type><System>SubVector::shift(Stride n, const TYPE& x) {
  if (n < 0) {
    reverse().shift(-n, x);
    }
  else {
    <Type><System>Handle&
			h = (<Type><System>Handle&)handle();
    Offset		o = offset();
    Stride		s = stride();
    for (Offset j = 0; n+j < extent(); j++) {
      h.put(o + (Stride)j*s, h.get(o + (Stride)(j+n)*s));
      }
    Offset		k = ((Offset)n < extent())? extent() - n: 0;
    for (Offset j = k;   j < extent(); j++) {
      h.put(o + (Stride)j*s, x);
      }
    }
  return *this;
  }

ifelse(_system_, complex, `', `
Offset
  <Type><System>SubVector::min(void) const {
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();

  <type>		m = h.get(o);
  Offset		k = 0;
  for (Offset j = 1; j < n; j++) {
    o += s;
    if (h.get(o) < m) {
      m = h.get(o);
      k = j;
      }
    }
  return k;
  }

Offset
  <Type><System>SubVector::max(void) const {
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();

  <type>		m = h.get(o);
  Offset		k = 0;
  for (Offset j = 1; j < n; j++) {
    o += s;
    if (h.get(o) > m) {
      m = h.get(o);
      k = j;
      }
    }
  return k;
  }

const
<Type><System>Vector
  min(const <Type><System>SubVector& v, const <Type><System>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "min(const <Type><System>SubVector&, const <Type><System>SubVector&)",
    v.extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	hv = v.handle();
  Offset		ov = v.offset();
  Extent		nv = v.extent();
  Stride		sv = v.stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(nv);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < nv; j++) {
    p.put(j, (hv.get(ov) < hw.get(ow))? hv.get(ov): hw.get(ow));
    ov += sv; ow += sw;
    }
  return result;
  }

const
<Type><System>Vector
  max(const <Type><System>SubVector& v, const <Type><System>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "max(const <Type><System>SubVector&, const <Type><System>SubVector&)",
    v.extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	hv = v.handle();
  Offset		ov = v.offset();
  Extent		nv = v.extent();
  Stride		sv = v.stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(nv);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < nv; j++) {
    p.put(j, (hv.get(ov) > hw.get(ow))? hv.get(ov): hw.get(ow));
    ov += sv; ow += sw;
    }
  return result;
  }
')dnl

// v.sum() = v_{0} + v_{1} + ... + v_{n-1}
TYPE
  <Type><System>SubVector::sum(void) const {
  TYPE			result = ZERO;
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();

  for (Offset j = 0; j < n; j++) {
    result += h.get(o);
    o += s;
    }
  return result;
  }

ifelse(_system_, complex, `
// v.dot(w) = v_{0}*w_{0} + v_{1}*w_{1} + ... + v_{n-1}*w_{n-1}
TYPE
  <Type><System>SubVector::dot(const <Type>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::dot(const <Type>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  TYPE			result = ZERO;
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();

  for (Offset j = 0; j < n; j++) {
    result += h.get(o)*hw.get(ow);
    o += s; ow += sw;
    }
  return result;
  }
')dnl

// v.dot(w) = v_{0}*w_{0} + v_{1}*w_{1} + ... + v_{n-1}*w_{n-1}
TYPE
  <Type><System>SubVector::dot(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::dot(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  TYPE			result = ZERO;
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();

  for (Offset j = 0; j < n; j++) {
    result += h.get(o)*hw.get(ow);
    o += s; ow += sw;
    }
  return result;
  }

const
<Type><System>Vector
  <Type><System>SubVector::permutation(const offsetSubVector& p) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::permutation(const offsetSubVector&) const",
    extent(), p.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>SubVector&	v = *this;
  Offset			n = extent();
  <Type><System>Vector		w(n);
  for (Offset j = 0; j < n; j++) {
    w[j] = v[p[j]];
    }
  return w;
  }

ifelse(_system_, complex, `
const
boolVector
  <Type><System>SubVector::eq(const <Type>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::eq(const <Type>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  boolVector		result(extent());
  boolHandle&		p = (boolHandle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o) == hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

const
boolVector
  <Type><System>SubVector::ne(const <Type>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::ne(const <Type>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  boolVector		result(extent());
  boolHandle&		p = (boolHandle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o) != hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }
',

`const
boolVector
  <Type><System>SubVector::lt(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::lt(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  boolVector		result(extent());
  boolHandle&		p = (boolHandle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o) <  hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

const
boolVector
  <Type><System>SubVector::le(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::le(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  boolVector		result(extent());
  boolHandle&		p = (boolHandle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o) <= hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }
')dnl

const
boolVector
  <Type><System>SubVector::eq(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::eq(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  boolVector		result(extent());
  boolHandle&		p = (boolHandle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o) == hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

const
boolVector
  <Type><System>SubVector::ne(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::ne(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  boolVector		result(extent());
  boolHandle&		p = (boolHandle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o) != hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

Extent
  <Type><System>SubVector::zeros(void) const {
  TYPE			zero = ZERO;
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();

  Extent		result = 0;
  for (Offset j = 0; j < n; j++) {
    if (zero == h.get(o)) ++result;
    o += s;
    }
  return result;
  }

const
offsetVector
  <Type><System>SubVector::index(void) const {
  TYPE			zero = ZERO;
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  offsetVector		result(n - zeros());
  offsetHandle&		p = (offsetHandle&)result.handle();

  Offset		k = 0;
  for (Offset j = 0; j < n; j++) {
    if (zero != h.get(o))
      p.put(k++, j);
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
  <Type><System>SubVector::gather(const offsetSubVector& x) const {
  const
  <Type><System>Handle&	h  =   handle();
  Offset		o  =   offset();
  Extent		n  =   extent();
  Stride		s  =   stride();
  const
  offsetHandle&		hx = x.handle();
  Offset		ox = x.offset();
  Extent		nx = x.extent();
  Stride		sx = x.stride();
  <Type><System>Vector	result(nx);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < nx; j++) {
    Offset		k = hx.get(ox);
#ifdef SVMT_DEBUG_MODE
    svmt_check_vector_index_range(
      "<Type><System>SubVector::gather(const offsetSubVector&) const", k, n);
#endif // SVMT_DEBUG_MODE
    p.put(j, h.get(o + (Stride)k*s));
    ox += sx;
    }
  return result;
  }

<Type><System>SubVector&
  <Type><System>SubVector::scatter(const offsetSubVector& x,
    const <Type><System>SubVector& t) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::scatter(\n"
    "const offsetSubVector&, const <Type><System>SubVector&)",
    x.extent(), t.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h  =   (<Type><System>Handle&)handle();
  Offset		o  =   offset();
  Extent		n  =   extent();
  Stride		s  =   stride();
  const
  offsetHandle&		hx = x.handle();
  Offset		ox = x.offset();
  Extent		nx = x.extent();
  Stride		sx = x.stride();
  const
  <Type><System>Handle&	ht = t.handle();
  Offset		ot = t.offset();
  Stride		st = t.stride();

  for (Offset j = 0; j < nx; j++) {
    Offset		k = hx.get(ox);
#ifdef SVMT_DEBUG_MODE
    svmt_check_vector_index_range(
      "<Type><System>SubVector::scatter(\n"
      "const offsetSubVector&, const <Type><System>SubVector&)", k, n);
#endif // SVMT_DEBUG_MODE
    h.put(o + (Stride)k*s, ht.get(ot));
    ox += sx;
    ot += st;
    }
  return *this;
  }

const
<Type><System>Vector
  <Type><System>SubVector::aside(const <Type><System>SubVector& w) const {
  <Type><System>Vector	result(extent() + w.extent());
  result.sub((Offset)0, extent(), (Stride)1) = *this;
  result.sub(extent(), w.extent(), (Stride)1) = w;
  return result;
  }
const
<Type><System>Vector
  <Type><System>SubVector::kron(const <Type><System>SubVector& w) const {
  const
  <Type><System>SubVector&
			v = *this;
  Extent		nv = v.extent();
  Extent		nw = w.extent();
  <Type><System>Vector	result(nv*nw);
  <Type><System>Vector&	shadow = result;

  for (Offset j = 0; j < nv; j++) {
    shadow.sub((Stride)j*nw, nw, (Stride)1) = v[j]*w;
    }
  return result;
  }
ifelse(_system_, complex, `
const
<Type><System>Vector
  <Type><System>SubVector::kron(const <Type>SubVector& w) const {
  const
  <Type><System>SubVector&
			v = *this;
  Extent		nv = v.extent();
  Extent		nw = w.extent();
  <Type><System>Vector	result(nv*nw);
  <Type><System>Vector&	shadow = result;

  for (Offset j = 0; j < nv; j++) {
    shadow.sub(j*nw, nw, (Stride)1) = v[j]*w;
    }
  return result;
  }

const
<Type><System>Vector
  <Type>SubVector::kron(const <Type><System>SubVector& w) const {
  const
  <Type>SubVector&	v = *this;
  Extent		nv = v.extent();
  Extent		nw = w.extent();
  <Type><System>Vector	result(nv*nw);
  <Type><System>Vector&	shadow = result;

  for (Offset j = 0; j < nv; j++) {
    shadow.sub(j*nw, nw, (Stride)1) = v[j]*w;
    }
  return result;
  }
')dnl

const
<Type><System>Vector
  <Type><System>SubVector::apply(const TYPE (*f)(const TYPE&)) const {
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, f(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
  <Type><System>SubVector::apply(      TYPE (*f)(const TYPE&)) const {
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, f(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
  <Type><System>SubVector::apply(      TYPE (*f)(      TYPE )) const {
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, f(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
    exp(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,   exp(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
    log(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,   log(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
   sqrt(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,  sqrt(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
    cos(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,   cos(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
    sin(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,   sin(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
    tan(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,   tan(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
   acos(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,  acos(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
   asin(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,  asin(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
   atan(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,  atan(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
   cosh(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,  cosh(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
   sinh(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,  sinh(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
   tanh(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,  tanh(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
  acosh(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, acosh(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
  asinh(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, asinh(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
  atanh(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, atanh(h.get(o)));
    o += s;
    }
  return result;
  }

ifelse(_system_, complex, `
// Complex Functions
const
<Type><System>Vector
   conj(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	hv = v.handle();
  Offset		ov = v.offset();
  Extent		nv = v.extent();
  Stride		sv = v.stride();
  <Type><System>Vector	result(nv);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < nv; j++) {
    p.put(j,  conj(hv.get(ov)));
    ov += sv;
    }
  return result;
  }

const
<Type><System>Vector
  iconj(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	hv = v.handle();
  Offset		ov = v.offset();
  Extent		nv = v.extent();
  Stride		sv = v.stride();
  <Type><System>Vector	result(nv);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < nv; j++) {
    p.put(j, iconj(hv.get(ov)));
    ov += sv;
    }
  return result;
  }

const
<Type><System>Vector
  polar(const <Type>SubVector& v, const <Type>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "polar(const <Type>SubVector&, const <Type>SubVector&)",
    v.extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type>Handle&		hv = v.handle();
  Offset		ov = v.offset();
  Extent		nv = v.extent();
  Stride		sv = v.stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(nv);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < nv; j++) {
    p.put(j, polar(hv.get(ov), hw.get(ow)));
    ov += sv; ow += sw;
    }
  return result;
  }

const
<Type>Vector
   norm(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	hv = v.handle();
  Offset		ov = v.offset();
  Extent		nv = v.extent();
  Stride		sv = v.stride();
  <Type>Vector		result(nv);
  <Type>Handle&		p = (<Type>Handle&)result.handle();

  for (Offset j = 0; j < nv; j++) {
    p.put(j,  norm(hv.get(ov)));
    ov += sv;
    }
  return result;
  }
', `
const
<Type><System>Vector
    sgn(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,   sgn(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
    abs(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

ifelse(_system_, complex, `',
`  using std::abs;
')dnl
  for (Offset j = 0; j < n; j++) {
    p.put(j,   abs(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
  floor(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, floor(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
   ceil(const <Type><System>SubVector& v) {
  const
  <Type><System>Handle&	h = v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j,  ceil(h.get(o)));
    o += s;
    }
  return result;
  }

const
<Type><System>Vector
  hypot(const <Type><System>SubVector& v, const <Type><System>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "hypot(const <Type><System>SubVector&, const <Type><System>SubVector&)",
    v.extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	hv = v.handle();
  Offset		ov = v.offset();
  Extent		nv = v.extent();
  Stride		sv = v.stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(nv);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < nv; j++) {
    p.put(j, hypot(hv.get(ov), hw.get(ow)));
    ov += sv; ow += sw;
    }
  return result;
  }

const
<Type><System>Vector
  atan2(const <Type><System>SubVector& w, const <Type><System>SubVector& v) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "atan2(const <Type><System>SubVector&, const <Type><System>SubVector&)",
    w.extent(), v.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Extent		nw = w.extent();
  Stride		sw = w.stride();
  const
  <Type><System>Handle&	hv = v.handle();
  Offset		ov = v.offset();
  Stride		sv = v.stride();
  <Type><System>Vector	result(nw);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < nw; j++) {
    p.put(j, atan2(hw.get(ow), hv.get(ov)));
    ow += sw; ov += sv;
    }
  return result;
  }
')dnl

// Operators
ifelse(floating, _point_, `',
`// complement
const
<Type><System>Vector
  <Type><System>SubVector::operator ~(void) const {
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, ~h.get(o));
    o += s;
    }
  return result;
  }
')dnl

// minus
const
<Type><System>Vector
  <Type><System>SubVector::operator -(void) const {
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, -h.get(o));
    o += s;
    }
  return result;
  }

ifelse(_system_, complex, `
// multiply
const
<Type><System>Vector
  <Type><System>SubVector::operator *(const <Type>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator *(const <Type>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o)*hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

// divide
const
<Type><System>Vector
  <Type><System>SubVector::operator /(const <Type>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator /(const <Type>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o)/hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

// add
const
<Type><System>Vector
  <Type><System>SubVector::operator +(const <Type>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator +(const <Type>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o) + hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

// subtract
const
<Type><System>Vector
  <Type><System>SubVector::operator -(const <Type>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator -(const <Type>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o) - hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

// divide
const
<Type><System>Vector
  <Type>SubVector::operator /(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type>SubVector::operator /(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type>SubVector&	v = *this;
  const
  <Type>Handle&		hv = v.handle();
  Offset		ov = v.offset();
  Extent		nv = v.extent();
  Stride		sv = v.stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(nv);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < nv; j++) {
    p.put(j, hv.get(ov)/hw.get(ow));
    ov += sv; ow += sw;
    }
  return result;
  }

// subtract
const
<Type><System>Vector
  <Type>SubVector::operator -(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type>SubVector::operator -(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type>SubVector&	v = *this;
  const
  <Type>Handle&		hv = v.handle();
  Offset		ov = v.offset();
  Extent		nv = v.extent();
  Stride		sv = v.stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(nv);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < nv; j++) {
    p.put(j, hv.get(ov) - hw.get(ow));
    ov += sv; ow += sw;
    }
  return result;
  }
')dnl

// multiply
const
<Type><System>Vector
  <Type><System>SubVector::operator *(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator *(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o)*hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

// divide
const
<Type><System>Vector
  <Type><System>SubVector::operator /(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator /(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o)/hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

ifelse(floating, _point_, `',
`// modulo (remainder)
const
<Type><System>Vector
  <Type><System>SubVector::operator %(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator %(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o)%hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }
')dnl

// add
const
<Type><System>Vector
  <Type><System>SubVector::operator +(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator +(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o) + hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

// subtract
const
<Type><System>Vector
  <Type><System>SubVector::operator -(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator -(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o) - hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

ifelse(floating, _point_, `',
`// shift left
const
<Type><System>Vector
  <Type><System>SubVector::operator <<(int n) const {
  const
  <Type><System>Handle&	h  = handle();
  Offset		o  = offset();
  Extent		n1 = extent();
  Stride		s1 = stride();
  <Type><System>Vector	result(n1);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n1; j++) {
    p.put(j, h.get(o) << n);
    o += s1;
    }
  return result;
  }

// shift right
const
<Type><System>Vector
  <Type><System>SubVector::operator >>(int n) const {
  const
  <Type><System>Handle&	h  = handle();
  Offset		o  = offset();
  Extent		n1 = extent();
  Stride		s1 = stride();
  <Type><System>Vector	result(n1);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n1; j++) {
    p.put(j, h.get(o) >> n);
    o += s1;
    }
  return result;
  }
')dnl

std::istream&	operator >>(std::istream& is,       <Type><System>SubVector& v) {
  <Type><System>Handle&	h = (<Type><System>Handle&)v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  TYPE			x;

  for (Offset j = 0; j < n && is >> x; j++) {
    h.put(o, x);
    o += s;
    }
  return is;
  }

Extent		<Type><System>SubVector::C = 0;
std::ostream&	operator <<(std::ostream& os, const <Type><System>SubVector& v) {
  <Type><System>Handle&	h = (<Type><System>Handle&)v.handle();
  Offset		o = v.offset();
  Extent		n = v.extent();
  Stride		s = v.stride();
  int			width = os.width()%65536;
  int			columns = os.width()/65536;
  if (columns <= 0) {
    columns = <Type><System>SubVector::columns();
    if (columns <= 0) {
      columns = ifelse(_system_, complex, 2, 4);
      }
    }
  for (Offset j = 1; j <= n; j++) {
    if (os << std::setw(width) << h.get(o))
      if (j < n && j%columns)
	os << ' ';
      else
	os << '\n';
    o += s;
    }
  return os;
  }

ifelse(_system_, complex, `
// equal
bool	<Type><System>SubVector::operator ==(const <Type>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator ==(const <Type>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  Offset		j = 0;
  while (j < n && h.get(o) == hw.get(ow)) {
   ++j; o += s; ow += sw;
   }
  return j == n;
  }

// not equal
bool	<Type><System>SubVector::operator !=(const <Type>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator !=(const <Type>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  Offset		j = 0;
  while (j < n && h.get(o) != hw.get(ow)) {
   ++j; o += s; ow += sw;
   }
  return j == n;
  }
',

`// less than
bool	<Type><System>SubVector::operator < (
  const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator < (const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  Offset		j = 0;
  while (j < n && h.get(o) <  hw.get(ow)) {
   ++j; o += s; ow += sw;
   }
  return j == n;
  }

// less than or equal
bool	<Type><System>SubVector::operator <=(
  const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator <=(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  Offset		j = 0;
  while (j < n && h.get(o) <= hw.get(ow)) {
   ++j; o += s; ow += sw;
   }
  return j == n;
  }
')dnl

// equal
bool	<Type><System>SubVector::operator ==(
  const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator ==(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  Offset		j = 0;
  while (j < n && h.get(o) == hw.get(ow)) {
   ++j; o += s; ow += sw;
   }
  return j == n;
  }

// not equal
bool	<Type><System>SubVector::operator !=(
  const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator !=(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  Offset		j = 0;
  while (j < n && h.get(o) != hw.get(ow)) {
   ++j; o += s; ow += sw;
   }
  return j == n;
  }

ifelse(floating, _point_, `',
`// bitwise AND
const
<Type><System>Vector
  <Type><System>SubVector::operator &(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator &(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o)&hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

// bitwise exclusive OR
const
<Type><System>Vector
  <Type><System>SubVector::operator ^(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator ^(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o)^hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }

// bitwise inclusive OR
const
<Type><System>Vector
  <Type><System>SubVector::operator |(const <Type><System>SubVector& w) const {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator |(const <Type><System>SubVector&) const",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  const
  <Type><System>Handle&	h = handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  <Type><System>Vector	result(n);
  <Type><System>Handle&	p = (<Type><System>Handle&)result.handle();

  for (Offset j = 0; j < n; j++) {
    p.put(j, h.get(o)|hw.get(ow));
    o += s; ow += sw;
    }
  return result;
  }
')dnl

ifelse(_system_, complex, `
// simple assignment
<Type><System>SubVector&
  <Type><System>SubVector::operator  =(const <Type>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator  =(const <Type>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }

// multiply and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator *=(const <Type>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator *=(const <Type>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, h.get(o)*hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }

// divide and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator /=(const <Type>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator /=(const <Type>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, h.get(o)/hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }

// add and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator +=(const <Type>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator +=(const <Type>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, h.get(o) + hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }

// subtract and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator -=(const <Type>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator -=(const <Type>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type>Handle&		hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, h.get(o) - hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }
')dnl

// simple assignment
<Type><System>SubVector&
  <Type><System>SubVector::operator  =(const <Type><System>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator  =(const <Type><System>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }

// multiply and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator *=(const <Type><System>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator *=(const <Type><System>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, h.get(o)*hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }

// divide and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator /=(const <Type><System>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator /=(const <Type><System>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, h.get(o)/hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }

ifelse(floating, _point_, `',
`// modulo and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator %=(const <Type><System>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator %=(const <Type><System>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, h.get(o)%hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }
')dnl

// add and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator +=(const <Type><System>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator +=(const <Type><System>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, h.get(o) + hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }

// subtract and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator -=(const <Type><System>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator -=(const <Type><System>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, h.get(o) - hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }

ifelse(floating, _point_, `',
`// shift left and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator <<=(int n) {
  <Type><System>Handle&	h  = (<Type><System>Handle&)handle();
  Offset		o  = offset();
  Extent		n1 = extent();
  Stride		s1 = stride();
  for (Offset j = 0; j < n1; j++) {
    h.put(o, h.get(o) << n);
    o += s1;
    }
  return *this;
  }

// shift right and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator >>=(int n) {
  <Type><System>Handle&	h  = (<Type><System>Handle&)handle();
  Offset		o  = offset();
  Extent		n1 = extent();
  Stride		s1 = stride();
  for (Offset j = 0; j < n1; j++) {
    h.put(o, h.get(o) >> n);
    o += s1;
    }
  return *this;
  }

// AND and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator &=(const <Type><System>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator &=(const <Type><System>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, h.get(o)&hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }

// exclusive OR and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator ^=(const <Type><System>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator ^=(const <Type><System>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, h.get(o)^hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }

// inclusive OR and assign
<Type><System>SubVector&
  <Type><System>SubVector::operator |=(const <Type><System>SubVector& w) {
#ifdef SVMT_DEBUG_MODE
  svmt_check_vector_conformance(
    "<Type><System>SubVector::operator |=(const <Type><System>SubVector&)",
    extent(), w.extent());
#endif // SVMT_DEBUG_MODE
  <Type><System>Handle&	h = (<Type><System>Handle&)handle();
  Offset		o = offset();
  Extent		n = extent();
  Stride		s = stride();
  const
  <Type><System>Handle&	hw = w.handle();
  Offset		ow = w.offset();
  Stride		sw = w.stride();
  for (Offset j = 0; j < n; j++) {
    h.put(o, h.get(o)|hw.get(ow));
    o += s; ow += sw;
    }
  return *this;
  }
')dnl
popdef(`index')dnl
popdef(`shift')dnl

